

set -ex



echo linux-64
echo 2.17
echo conda
echo 3.10.0
test -f ${PREFIX}/x86_64-conda-linux-gnu/sysroot/lib/libgomp.so
test `readlink ${PREFIX}/x86_64-conda-linux-gnu/sysroot/lib/libgomp.so` == "../../../lib/libgomp.so"
test `readlink ${PREFIX}/lib/libgomp.so` == "libgomp.so.1.0.0"
test -f ${PREFIX}/bin/x86_64-conda_cos7-linux-gnu-gcc
CC=$(${PREFIX}/bin/*-gcc -dumpmachine)-gcc
${CC} -Wall tests/aligned_alloc.c -c -o c_aligned.o -v
${CC} -Wall c_aligned.o -o c_aligned -v && ./c_aligned
exit 0
