/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_x86_sahf_1 = 41,
  CODE_FOR_insvhi_1 = 108,
  CODE_FOR_insvsi_1 = 109,
  CODE_FOR_insvdi_1 = 110,
  CODE_FOR_zero_extendqidi2 = 139,
  CODE_FOR_zero_extendhidi2 = 140,
  CODE_FOR_zero_extendqisi2_and = 141,
  CODE_FOR_zero_extendhisi2_and = 142,
  CODE_FOR_zero_extendqihi2_and = 145,
  CODE_FOR_extendsidi2_1 = 148,
  CODE_FOR_extendqidi2 = 149,
  CODE_FOR_extendhidi2 = 150,
  CODE_FOR_extendhisi2 = 151,
  CODE_FOR_extendqisi2 = 153,
  CODE_FOR_extendqihi2 = 155,
  CODE_FOR_truncdfsf2 = 159,
  CODE_FOR_truncxfsf2 = 160,
  CODE_FOR_truncxfdf2 = 161,
  CODE_FOR_fixuns_truncsfdi2 = 162,
  CODE_FOR_fixuns_truncdfdi2 = 163,
  CODE_FOR_fixuns_truncsfsi2_avx512f = 164,
  CODE_FOR_fixuns_truncdfsi2_avx512f = 165,
  CODE_FOR_fix_truncsfsi_sse = 170,
  CODE_FOR_fix_truncsfdi_sse = 171,
  CODE_FOR_fix_truncdfsi_sse = 172,
  CODE_FOR_fix_truncdfdi_sse = 173,
  CODE_FOR_fix_trunchi_i387_fisttp = 174,
  CODE_FOR_fix_truncsi_i387_fisttp = 175,
  CODE_FOR_fix_truncdi_i387_fisttp = 176,
  CODE_FOR_fix_truncdi_i387 = 180,
  CODE_FOR_fix_trunchi_i387 = 181,
  CODE_FOR_fix_truncsi_i387 = 182,
  CODE_FOR_x86_fnstcw_1 = 183,
  CODE_FOR_floathisf2 = 184,
  CODE_FOR_floathidf2 = 185,
  CODE_FOR_floathixf2 = 186,
  CODE_FOR_floatsixf2 = 187,
  CODE_FOR_floatdixf2 = 188,
  CODE_FOR_floatdisf2_i387_with_xmm = 195,
  CODE_FOR_floatdidf2_i387_with_xmm = 196,
  CODE_FOR_floatdixf2_i387_with_xmm = 197,
  CODE_FOR_floatunssisf2_i387_with_xmm = 202,
  CODE_FOR_floatunssidf2_i387_with_xmm = 203,
  CODE_FOR_floatunssixf2_i387_with_xmm = 204,
  CODE_FOR_addsi_1_zext = 211,
  CODE_FOR_addvqi4_1 = 244,
  CODE_FOR_addvhi4_1 = 245,
  CODE_FOR_addvsi4_1 = 246,
  CODE_FOR_addvdi4_1 = 247,
  CODE_FOR_subvqi4_1 = 296,
  CODE_FOR_subvhi4_1 = 297,
  CODE_FOR_subvsi4_1 = 298,
  CODE_FOR_subvdi4_1 = 299,
  CODE_FOR_addqi3_carry = 317,
  CODE_FOR_addhi3_carry = 318,
  CODE_FOR_addsi3_carry = 319,
  CODE_FOR_adddi3_carry = 320,
  CODE_FOR_addcarrysi = 327,
  CODE_FOR_addcarrydi = 328,
  CODE_FOR_subqi3_carry = 331,
  CODE_FOR_subhi3_carry = 332,
  CODE_FOR_subsi3_carry = 333,
  CODE_FOR_subdi3_carry = 334,
  CODE_FOR_subsi3_carry_ccc = 341,
  CODE_FOR_subdi3_carry_ccc = 342,
  CODE_FOR_subsi3_carry_ccgz = 345,
  CODE_FOR_subdi3_carry_ccgz = 346,
  CODE_FOR_subborrowsi = 347,
  CODE_FOR_subborrowdi = 348,
  CODE_FOR_divmodsi4_1 = 437,
  CODE_FOR_divmoddi4_1 = 438,
  CODE_FOR_udivmodsi4_1 = 439,
  CODE_FOR_udivmoddi4_1 = 440,
  CODE_FOR_divmodsi4_zext_1 = 441,
  CODE_FOR_udivmodsi4_zext_1 = 442,
  CODE_FOR_divmodsi4_zext_2 = 443,
  CODE_FOR_udivmodsi4_zext_2 = 444,
  CODE_FOR_divmodhiqi3 = 469,
  CODE_FOR_udivmodhiqi3 = 470,
  CODE_FOR_copysignsf3_const = 622,
  CODE_FOR_copysigndf3_const = 623,
  CODE_FOR_copysigntf3_const = 624,
  CODE_FOR_copysignsf3_var = 625,
  CODE_FOR_copysigndf3_var = 626,
  CODE_FOR_copysigntf3_var = 627,
  CODE_FOR_xorsignsf3_1 = 628,
  CODE_FOR_xorsigndf3_1 = 629,
  CODE_FOR_x86_64_shld = 647,
  CODE_FOR_x86_shld = 648,
  CODE_FOR_x86_64_shrd = 692,
  CODE_FOR_x86_shrd = 693,
  CODE_FOR_ashrsi3_cvt = 694,
  CODE_FOR_ashrdi3_cvt = 695,
  CODE_FOR_ix86_rotldi3_doubleword = 743,
  CODE_FOR_ix86_rotlti3_doubleword = 744,
  CODE_FOR_ix86_rotrdi3_doubleword = 745,
  CODE_FOR_ix86_rotrti3_doubleword = 746,
  CODE_FOR_setcc_sf_sse = 804,
  CODE_FOR_setcc_df_sse = 805,
  CODE_FOR_jump = 807,
  CODE_FOR_blockage = 834,
  CODE_FOR_prologue_use = 836,
  CODE_FOR_simple_return_internal = 837,
  CODE_FOR_interrupt_return = 838,
  CODE_FOR_simple_return_internal_long = 839,
  CODE_FOR_simple_return_pop_internal = 840,
  CODE_FOR_nop = 843,
  CODE_FOR_nops = 844,
  CODE_FOR_pad = 845,
  CODE_FOR_set_got_rex64 = 848,
  CODE_FOR_set_rip_rex64 = 849,
  CODE_FOR_set_got_offset_rex64 = 850,
  CODE_FOR_eh_return_internal = 851,
  CODE_FOR_split_stack_return = 854,
  CODE_FOR_ffssi2_no_cmove = 855,
  CODE_FOR_ctzsi2 = 862,
  CODE_FOR_ctzdi2 = 863,
  CODE_FOR_bsr_rex64 = 868,
  CODE_FOR_bsr = 869,
  CODE_FOR_clzsi2_lzcnt = 870,
  CODE_FOR_clzdi2_lzcnt = 871,
  CODE_FOR_tzcnt_si = 877,
  CODE_FOR_lzcnt_si = 878,
  CODE_FOR_tzcnt_di = 879,
  CODE_FOR_lzcnt_di = 880,
  CODE_FOR_tzcnt_hi = 885,
  CODE_FOR_lzcnt_hi = 886,
  CODE_FOR_bmi_bextr_si = 887,
  CODE_FOR_bmi_bextr_di = 888,
  CODE_FOR_bmi2_pdep_si3 = 915,
  CODE_FOR_bmi2_pdep_di3 = 916,
  CODE_FOR_bmi2_pext_si3 = 917,
  CODE_FOR_bmi2_pext_di3 = 918,
  CODE_FOR_tbm_bextri_si = 919,
  CODE_FOR_tbm_bextri_di = 920,
  CODE_FOR_popcountsi2 = 939,
  CODE_FOR_popcountdi2 = 940,
  CODE_FOR_popcounthi2 = 946,
  CODE_FOR_bswaphi_lowpart = 952,
  CODE_FOR_parityhi2_cmp = 953,
  CODE_FOR_parityqi2_cmp = 954,
   CODE_FOR_tls_initial_exec_64_sun = CODE_FOR_nothing,
  CODE_FOR_truncxfsf2_i387_noop_unspec = 1007,
  CODE_FOR_truncxfdf2_i387_noop_unspec = 1008,
  CODE_FOR_sqrtxf2 = 1009,
  CODE_FOR_x86_fnstsw_1 = 1013,
  CODE_FOR_fpremxf4_i387 = 1014,
  CODE_FOR_fprem1xf4_i387 = 1015,
  CODE_FOR_sinxf2 = 1016,
  CODE_FOR_cosxf2 = 1017,
  CODE_FOR_sincosxf3 = 1018,
  CODE_FOR_fptanxf4_i387 = 1019,
  CODE_FOR_atan2xf3 = 1020,
  CODE_FOR_fyl2xxf3_i387 = 1021,
  CODE_FOR_fyl2xp1xf3_i387 = 1022,
  CODE_FOR_fxtractxf3_i387 = 1023,
  CODE_FOR_fscalexf4_i387 = 1025,
  CODE_FOR_sse4_1_roundsf2 = 1026,
  CODE_FOR_sse4_1_rounddf2 = 1027,
  CODE_FOR_rintxf2 = 1028,
  CODE_FOR_lrintxfdi2 = 1029,
  CODE_FOR_lrintxfhi2 = 1030,
  CODE_FOR_lrintxfsi2 = 1031,
  CODE_FOR_frndintxf2_roundeven = 1032,
  CODE_FOR_frndintxf2_floor = 1033,
  CODE_FOR_frndintxf2_ceil = 1034,
  CODE_FOR_frndintxf2_trunc = 1035,
  CODE_FOR_frndintxf2_roundeven_i387 = 1036,
  CODE_FOR_frndintxf2_floor_i387 = 1037,
  CODE_FOR_frndintxf2_ceil_i387 = 1038,
  CODE_FOR_frndintxf2_trunc_i387 = 1039,
  CODE_FOR_fistdi2_floor = 1046,
  CODE_FOR_fistdi2_ceil = 1047,
  CODE_FOR_fisthi2_floor = 1048,
  CODE_FOR_fisthi2_ceil = 1049,
  CODE_FOR_fistsi2_floor = 1050,
  CODE_FOR_fistsi2_ceil = 1051,
  CODE_FOR_fxamsf2_i387 = 1052,
  CODE_FOR_fxamdf2_i387 = 1053,
  CODE_FOR_fxamxf2_i387 = 1054,
  CODE_FOR_movmsk_df = 1055,
  CODE_FOR_cld = 1056,
  CODE_FOR_smaxsf3 = 1109,
  CODE_FOR_sminsf3 = 1110,
  CODE_FOR_smaxdf3 = 1111,
  CODE_FOR_smindf3 = 1112,
  CODE_FOR_pro_epilogue_adjust_stack_add_si = 1141,
  CODE_FOR_pro_epilogue_adjust_stack_add_di = 1142,
  CODE_FOR_pro_epilogue_adjust_stack_sub_si = 1143,
  CODE_FOR_pro_epilogue_adjust_stack_sub_di = 1144,
  CODE_FOR_allocate_stack_worker_probe_si = 1145,
  CODE_FOR_allocate_stack_worker_probe_di = 1146,
  CODE_FOR_probe_stack_1_si = 1147,
  CODE_FOR_probe_stack_1_di = 1148,
  CODE_FOR_adjust_stack_and_probe_si = 1149,
  CODE_FOR_adjust_stack_and_probe_di = 1150,
  CODE_FOR_probe_stack_range_si = 1151,
  CODE_FOR_probe_stack_range_di = 1152,
  CODE_FOR_trap = 1153,
  CODE_FOR_ud2 = 1154,
  CODE_FOR_stack_protect_set_1_si = 1158,
  CODE_FOR_stack_protect_set_1_di = 1159,
  CODE_FOR_stack_protect_test_1_si = 1163,
  CODE_FOR_stack_protect_test_1_di = 1164,
  CODE_FOR_sse4_2_crc32qi = 1165,
  CODE_FOR_sse4_2_crc32hi = 1166,
  CODE_FOR_sse4_2_crc32si = 1167,
  CODE_FOR_sse4_2_crc32di = 1168,
  CODE_FOR_rdpmc = 1169,
  CODE_FOR_rdpmc_rex64 = 1170,
  CODE_FOR_rdtsc = 1171,
  CODE_FOR_rdtsc_rex64 = 1172,
  CODE_FOR_rdtscp = 1173,
  CODE_FOR_rdtscp_rex64 = 1174,
  CODE_FOR_fxsave = 1175,
  CODE_FOR_fxsave64 = 1176,
  CODE_FOR_fxrstor = 1177,
  CODE_FOR_fxrstor64 = 1178,
  CODE_FOR_xsave = 1179,
  CODE_FOR_xsaveopt = 1180,
  CODE_FOR_xsavec = 1181,
  CODE_FOR_xsaves = 1182,
  CODE_FOR_xsave_rex64 = 1183,
  CODE_FOR_xsaveopt_rex64 = 1184,
  CODE_FOR_xsavec_rex64 = 1185,
  CODE_FOR_xsaves_rex64 = 1186,
  CODE_FOR_xsave64 = 1187,
  CODE_FOR_xsaveopt64 = 1188,
  CODE_FOR_xsavec64 = 1189,
  CODE_FOR_xsaves64 = 1190,
  CODE_FOR_xrstor = 1191,
  CODE_FOR_xrstors = 1192,
  CODE_FOR_xrstor_rex64 = 1193,
  CODE_FOR_xrstors_rex64 = 1194,
  CODE_FOR_xrstor64 = 1195,
  CODE_FOR_xrstors64 = 1196,
  CODE_FOR_xsetbv = 1197,
  CODE_FOR_xsetbv_rex64 = 1198,
  CODE_FOR_xgetbv = 1199,
  CODE_FOR_xgetbv_rex64 = 1200,
  CODE_FOR_fnstenv = 1201,
  CODE_FOR_fldenv = 1202,
  CODE_FOR_fnstsw = 1203,
  CODE_FOR_fnclex = 1204,
  CODE_FOR_lwp_llwpcbsi = 1205,
  CODE_FOR_lwp_llwpcbdi = 1206,
  CODE_FOR_lwp_slwpcbsi = 1207,
  CODE_FOR_lwp_slwpcbdi = 1208,
  CODE_FOR_lwp_lwpvalsi = 1209,
  CODE_FOR_lwp_lwpvaldi = 1210,
  CODE_FOR_lwp_lwpinssi = 1211,
  CODE_FOR_lwp_lwpinsdi = 1212,
  CODE_FOR_rdfsbasesi = 1213,
  CODE_FOR_rdgsbasesi = 1214,
  CODE_FOR_rdfsbasedi = 1215,
  CODE_FOR_rdgsbasedi = 1216,
  CODE_FOR_wrfsbasesi = 1217,
  CODE_FOR_wrgsbasesi = 1218,
  CODE_FOR_wrfsbasedi = 1219,
  CODE_FOR_wrgsbasedi = 1220,
  CODE_FOR_ptwritesi = 1221,
  CODE_FOR_ptwritedi = 1222,
  CODE_FOR_rdrandhi = 1223,
  CODE_FOR_rdrandsi = 1224,
  CODE_FOR_rdranddi = 1225,
  CODE_FOR_rdseedhi = 1226,
  CODE_FOR_rdseedsi = 1227,
  CODE_FOR_rdseeddi = 1228,
  CODE_FOR_rdsspsi = 1230,
  CODE_FOR_rdsspdi = 1231,
  CODE_FOR_incsspsi = 1232,
  CODE_FOR_incsspdi = 1233,
  CODE_FOR_saveprevssp = 1234,
  CODE_FOR_rstorssp = 1235,
  CODE_FOR_wrsssi = 1236,
  CODE_FOR_wrssdi = 1237,
  CODE_FOR_wrusssi = 1238,
  CODE_FOR_wrussdi = 1239,
  CODE_FOR_setssbsy = 1240,
  CODE_FOR_clrssbsy = 1241,
  CODE_FOR_nop_endbr = 1242,
  CODE_FOR_xbegin_1 = 1243,
  CODE_FOR_xend = 1244,
  CODE_FOR_xabort = 1245,
  CODE_FOR_xtest_1 = 1246,
  CODE_FOR_clwb = 1247,
  CODE_FOR_clflushopt = 1248,
  CODE_FOR_mwaitx = 1249,
  CODE_FOR_monitorx_si = 1250,
  CODE_FOR_monitorx_di = 1251,
  CODE_FOR_clzero_si = 1252,
  CODE_FOR_clzero_di = 1253,
  CODE_FOR_rdpid = 1256,
  CODE_FOR_rdpid_rex64 = 1257,
  CODE_FOR_wbinvd = 1258,
  CODE_FOR_wbnoinvd = 1259,
  CODE_FOR_movdirisi = 1260,
  CODE_FOR_movdiridi = 1261,
  CODE_FOR_movdir64b_si = 1262,
  CODE_FOR_movdir64b_di = 1263,
  CODE_FOR_xsusldtrk = 1264,
  CODE_FOR_xresldtrk = 1265,
  CODE_FOR_enqcmd_si = 1266,
  CODE_FOR_enqcmds_si = 1267,
  CODE_FOR_enqcmd_di = 1268,
  CODE_FOR_enqcmds_di = 1269,
  CODE_FOR_clui = 1270,
  CODE_FOR_stui = 1271,
  CODE_FOR_testui = 1272,
  CODE_FOR_senduipi = 1273,
  CODE_FOR_umwait = 1274,
  CODE_FOR_umwait_rex64 = 1275,
  CODE_FOR_umonitor_si = 1276,
  CODE_FOR_umonitor_di = 1277,
  CODE_FOR_tpause = 1278,
  CODE_FOR_tpause_rex64 = 1279,
  CODE_FOR_cldemote = 1280,
  CODE_FOR_speculation_barrier = 1281,
  CODE_FOR_serialize = 1282,
  CODE_FOR_patchable_area = 1283,
  CODE_FOR_hreset = 1284,
  CODE_FOR_sse_movntq = 1290,
  CODE_FOR_mmx_ieee_maxv2sf3 = 1299,
  CODE_FOR_mmx_ieee_minv2sf3 = 1300,
  CODE_FOR_mmx_rcpv2sf2 = 1301,
  CODE_FOR_mmx_rcpit1v2sf3 = 1302,
  CODE_FOR_mmx_rcpit2v2sf3 = 1303,
  CODE_FOR_sqrtv2sf2 = 1304,
  CODE_FOR_mmx_rsqrtv2sf2 = 1305,
  CODE_FOR_mmx_rsqit1v2sf3 = 1306,
  CODE_FOR_mmx_hsubv2sf3 = 1309,
  CODE_FOR_mmx_gtv2sf3 = 1313,
  CODE_FOR_mmx_gev2sf3 = 1314,
  CODE_FOR_fmav2sf4 = 1319,
  CODE_FOR_fmsv2sf4 = 1320,
  CODE_FOR_fnmav2sf4 = 1321,
  CODE_FOR_fnmsv2sf4 = 1322,
  CODE_FOR_mmx_fix_truncv2sfv2si2 = 1323,
  CODE_FOR_fixuns_truncv2sfv2si2 = 1324,
  CODE_FOR_mmx_floatv2siv2sf2 = 1325,
  CODE_FOR_floatunsv2siv2sf2 = 1326,
  CODE_FOR_mmx_pf2iw = 1327,
  CODE_FOR_mmx_pi2fw = 1328,
  CODE_FOR_mmx_pswapdv2sf2 = 1329,
  CODE_FOR_mmx_ashrv4hi3 = 1363,
  CODE_FOR_mmx_ashrv2si3 = 1364,
  CODE_FOR_mmx_ashlv4hi3 = 1365,
  CODE_FOR_mmx_lshrv4hi3 = 1366,
  CODE_FOR_mmx_ashlv2si3 = 1367,
  CODE_FOR_mmx_lshrv2si3 = 1368,
  CODE_FOR_mmx_ashlv1di3 = 1369,
  CODE_FOR_mmx_lshrv1di3 = 1370,
  CODE_FOR_mmx_gtv8qi3 = 1374,
  CODE_FOR_mmx_gtv4hi3 = 1375,
  CODE_FOR_mmx_gtv2si3 = 1376,
  CODE_FOR_mmx_andnotv8qi3 = 1377,
  CODE_FOR_mmx_andnotv4hi3 = 1378,
  CODE_FOR_mmx_andnotv2si3 = 1379,
  CODE_FOR_mmx_packsswb = 1389,
  CODE_FOR_mmx_packuswb = 1390,
  CODE_FOR_mmx_packssdw = 1391,
  CODE_FOR_mmx_punpckhbw = 1392,
  CODE_FOR_mmx_punpcklbw = 1393,
  CODE_FOR_mmx_punpckhwd = 1394,
  CODE_FOR_mmx_punpcklwd = 1395,
  CODE_FOR_mmx_punpckhdq = 1396,
  CODE_FOR_mmx_punpckldq = 1397,
  CODE_FOR_mmx_pshufw_1 = 1408,
  CODE_FOR_mmx_pswapdv2si2 = 1410,
  CODE_FOR_mmx_psadbw = 1422,
  CODE_FOR_mmx_pmovmskb = 1423,
  CODE_FOR_movv64qi_internal = 1428,
  CODE_FOR_movv32qi_internal = 1429,
  CODE_FOR_movv16qi_internal = 1430,
  CODE_FOR_movv32hi_internal = 1431,
  CODE_FOR_movv16hi_internal = 1432,
  CODE_FOR_movv8hi_internal = 1433,
  CODE_FOR_movv16si_internal = 1434,
  CODE_FOR_movv8si_internal = 1435,
  CODE_FOR_movv4si_internal = 1436,
  CODE_FOR_movv8di_internal = 1437,
  CODE_FOR_movv4di_internal = 1438,
  CODE_FOR_movv2di_internal = 1439,
  CODE_FOR_movv4ti_internal = 1440,
  CODE_FOR_movv2ti_internal = 1441,
  CODE_FOR_movv1ti_internal = 1442,
  CODE_FOR_movv16sf_internal = 1443,
  CODE_FOR_movv8sf_internal = 1444,
  CODE_FOR_movv4sf_internal = 1445,
  CODE_FOR_movv8df_internal = 1446,
  CODE_FOR_movv4df_internal = 1447,
  CODE_FOR_movv2df_internal = 1448,
  CODE_FOR_avx512f_movsf_mask = 1485,
  CODE_FOR_avx512f_movdf_mask = 1486,
  CODE_FOR_avx512f_storesf_mask = 1489,
  CODE_FOR_avx512f_storedf_mask = 1490,
  CODE_FOR_avx512f_blendmv16si = 1491,
  CODE_FOR_avx512vl_blendmv8si = 1492,
  CODE_FOR_avx512vl_blendmv4si = 1493,
  CODE_FOR_avx512f_blendmv8di = 1494,
  CODE_FOR_avx512vl_blendmv4di = 1495,
  CODE_FOR_avx512vl_blendmv2di = 1496,
  CODE_FOR_avx512f_blendmv16sf = 1497,
  CODE_FOR_avx512vl_blendmv8sf = 1498,
  CODE_FOR_avx512vl_blendmv4sf = 1499,
  CODE_FOR_avx512f_blendmv8df = 1500,
  CODE_FOR_avx512vl_blendmv4df = 1501,
  CODE_FOR_avx512vl_blendmv2df = 1502,
  CODE_FOR_avx512bw_blendmv64qi = 1503,
  CODE_FOR_avx512vl_blendmv16qi = 1504,
  CODE_FOR_avx512vl_blendmv32qi = 1505,
  CODE_FOR_avx512bw_blendmv32hi = 1506,
  CODE_FOR_avx512vl_blendmv16hi = 1507,
  CODE_FOR_avx512vl_blendmv8hi = 1508,
  CODE_FOR_avx512f_storev16si_mask = 1509,
  CODE_FOR_avx512vl_storev8si_mask = 1510,
  CODE_FOR_avx512vl_storev4si_mask = 1511,
  CODE_FOR_avx512f_storev8di_mask = 1512,
  CODE_FOR_avx512vl_storev4di_mask = 1513,
  CODE_FOR_avx512vl_storev2di_mask = 1514,
  CODE_FOR_avx512f_storev16sf_mask = 1515,
  CODE_FOR_avx512vl_storev8sf_mask = 1516,
  CODE_FOR_avx512vl_storev4sf_mask = 1517,
  CODE_FOR_avx512f_storev8df_mask = 1518,
  CODE_FOR_avx512vl_storev4df_mask = 1519,
  CODE_FOR_avx512vl_storev2df_mask = 1520,
  CODE_FOR_avx512bw_storev64qi_mask = 1521,
  CODE_FOR_avx512vl_storev16qi_mask = 1522,
  CODE_FOR_avx512vl_storev32qi_mask = 1523,
  CODE_FOR_avx512bw_storev32hi_mask = 1524,
  CODE_FOR_avx512vl_storev16hi_mask = 1525,
  CODE_FOR_avx512vl_storev8hi_mask = 1526,
  CODE_FOR_sse2_movq128 = 1527,
  CODE_FOR_movdi_to_sse = 1528,
  CODE_FOR_avx_lddqu256 = 1529,
  CODE_FOR_sse3_lddqu = 1530,
  CODE_FOR_sse2_movntisi = 1531,
  CODE_FOR_sse2_movntidi = 1532,
  CODE_FOR_avx512f_movntv16sf = 1533,
  CODE_FOR_avx_movntv8sf = 1534,
  CODE_FOR_sse_movntv4sf = 1535,
  CODE_FOR_avx512f_movntv8df = 1536,
  CODE_FOR_avx_movntv4df = 1537,
  CODE_FOR_sse2_movntv2df = 1538,
  CODE_FOR_avx512f_movntv8di = 1539,
  CODE_FOR_avx_movntv4di = 1540,
  CODE_FOR_sse2_movntv2di = 1541,
  CODE_FOR_kandqi = 1542,
  CODE_FOR_kiorqi = 1543,
  CODE_FOR_kxorqi = 1544,
  CODE_FOR_kandhi = 1545,
  CODE_FOR_kiorhi = 1546,
  CODE_FOR_kxorhi = 1547,
  CODE_FOR_kandsi = 1548,
  CODE_FOR_kiorsi = 1549,
  CODE_FOR_kxorsi = 1550,
  CODE_FOR_kanddi = 1551,
  CODE_FOR_kiordi = 1552,
  CODE_FOR_kxordi = 1553,
  CODE_FOR_kandnqi = 1554,
  CODE_FOR_kandnhi = 1555,
  CODE_FOR_kandnsi = 1556,
  CODE_FOR_kandndi = 1557,
  CODE_FOR_kxnorqi = 1558,
  CODE_FOR_kxnorhi = 1559,
  CODE_FOR_kxnorsi = 1560,
  CODE_FOR_kxnordi = 1561,
  CODE_FOR_knotqi = 1562,
  CODE_FOR_knothi = 1563,
  CODE_FOR_knotsi = 1564,
  CODE_FOR_knotdi = 1565,
  CODE_FOR_kaddqi = 1567,
  CODE_FOR_kaddhi = 1568,
  CODE_FOR_kaddsi = 1569,
  CODE_FOR_kadddi = 1570,
  CODE_FOR_kashiftqi = 1571,
  CODE_FOR_klshiftrtqi = 1572,
  CODE_FOR_kashifthi = 1573,
  CODE_FOR_klshiftrthi = 1574,
  CODE_FOR_kashiftsi = 1575,
  CODE_FOR_klshiftrtsi = 1576,
  CODE_FOR_kashiftdi = 1577,
  CODE_FOR_klshiftrtdi = 1578,
  CODE_FOR_ktestqi = 1579,
  CODE_FOR_ktesthi = 1580,
  CODE_FOR_ktestsi = 1581,
  CODE_FOR_ktestdi = 1582,
  CODE_FOR_kortestqi = 1583,
  CODE_FOR_kortesthi = 1584,
  CODE_FOR_kortestsi = 1585,
  CODE_FOR_kortestdi = 1586,
  CODE_FOR_kunpckhi = 1587,
  CODE_FOR_kunpcksi = 1588,
  CODE_FOR_kunpckdi = 1589,
  CODE_FOR_sse_vmaddv4sf3 = 1660,
  CODE_FOR_sse_vmaddv4sf3_round = 1661,
  CODE_FOR_sse_vmaddv4sf3_mask = 1662,
  CODE_FOR_sse_vmaddv4sf3_mask_round = 1663,
  CODE_FOR_sse_vmsubv4sf3 = 1664,
  CODE_FOR_sse_vmsubv4sf3_round = 1665,
  CODE_FOR_sse_vmsubv4sf3_mask = 1666,
  CODE_FOR_sse_vmsubv4sf3_mask_round = 1667,
  CODE_FOR_sse2_vmaddv2df3 = 1668,
  CODE_FOR_sse2_vmaddv2df3_round = 1669,
  CODE_FOR_sse2_vmaddv2df3_mask = 1670,
  CODE_FOR_sse2_vmaddv2df3_mask_round = 1671,
  CODE_FOR_sse2_vmsubv2df3 = 1672,
  CODE_FOR_sse2_vmsubv2df3_round = 1673,
  CODE_FOR_sse2_vmsubv2df3_mask = 1674,
  CODE_FOR_sse2_vmsubv2df3_mask_round = 1675,
  CODE_FOR_sse_vmmulv4sf3 = 1704,
  CODE_FOR_sse_vmmulv4sf3_round = 1705,
  CODE_FOR_sse_vmmulv4sf3_mask = 1706,
  CODE_FOR_sse_vmmulv4sf3_mask_round = 1707,
  CODE_FOR_sse_vmdivv4sf3 = 1708,
  CODE_FOR_sse_vmdivv4sf3_round = 1709,
  CODE_FOR_sse_vmdivv4sf3_mask = 1710,
  CODE_FOR_sse_vmdivv4sf3_mask_round = 1711,
  CODE_FOR_sse2_vmmulv2df3 = 1712,
  CODE_FOR_sse2_vmmulv2df3_round = 1713,
  CODE_FOR_sse2_vmmulv2df3_mask = 1714,
  CODE_FOR_sse2_vmmulv2df3_mask_round = 1715,
  CODE_FOR_sse2_vmdivv2df3 = 1716,
  CODE_FOR_sse2_vmdivv2df3_round = 1717,
  CODE_FOR_sse2_vmdivv2df3_mask = 1718,
  CODE_FOR_sse2_vmdivv2df3_mask_round = 1719,
  CODE_FOR_avx512f_divv16sf3 = 1720,
  CODE_FOR_avx512f_divv16sf3_round = 1721,
  CODE_FOR_avx512f_divv16sf3_mask = 1722,
  CODE_FOR_avx512f_divv16sf3_mask_round = 1723,
  CODE_FOR_avx_divv8sf3 = 1724,
   CODE_FOR_avx_divv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv8sf3_mask = 1725,
   CODE_FOR_avx_divv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3 = 1726,
   CODE_FOR_sse_divv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3_mask = 1727,
   CODE_FOR_sse_divv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv8df3 = 1728,
  CODE_FOR_avx512f_divv8df3_round = 1729,
  CODE_FOR_avx512f_divv8df3_mask = 1730,
  CODE_FOR_avx512f_divv8df3_mask_round = 1731,
  CODE_FOR_avx_divv4df3 = 1732,
   CODE_FOR_avx_divv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv4df3_mask = 1733,
   CODE_FOR_avx_divv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3 = 1734,
   CODE_FOR_sse2_divv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3_mask = 1735,
   CODE_FOR_sse2_divv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx_rcpv8sf2 = 1736,
  CODE_FOR_sse_rcpv4sf2 = 1737,
  CODE_FOR_sse_vmrcpv4sf2 = 1738,
  CODE_FOR_rcp14v16sf_mask = 1741,
  CODE_FOR_rcp14v8sf_mask = 1743,
  CODE_FOR_rcp14v4sf_mask = 1745,
  CODE_FOR_rcp14v8df_mask = 1747,
  CODE_FOR_rcp14v4df_mask = 1749,
  CODE_FOR_rcp14v2df_mask = 1751,
  CODE_FOR_srcp14v4sf = 1752,
  CODE_FOR_srcp14v2df = 1753,
  CODE_FOR_srcp14v4sf_mask = 1754,
  CODE_FOR_srcp14v2df_mask = 1755,
  CODE_FOR_avx512f_sqrtv16sf2 = 1756,
  CODE_FOR_avx512f_sqrtv16sf2_round = 1757,
  CODE_FOR_avx512f_sqrtv16sf2_mask = 1758,
  CODE_FOR_avx512f_sqrtv16sf2_mask_round = 1759,
  CODE_FOR_avx_sqrtv8sf2 = 1760,
   CODE_FOR_avx_sqrtv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv8sf2_mask = 1761,
   CODE_FOR_avx_sqrtv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2 = 1762,
   CODE_FOR_sse_sqrtv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2_mask = 1763,
   CODE_FOR_sse_sqrtv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv8df2 = 1764,
  CODE_FOR_avx512f_sqrtv8df2_round = 1765,
  CODE_FOR_avx512f_sqrtv8df2_mask = 1766,
  CODE_FOR_avx512f_sqrtv8df2_mask_round = 1767,
  CODE_FOR_avx_sqrtv4df2 = 1768,
   CODE_FOR_avx_sqrtv4df2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv4df2_mask = 1769,
   CODE_FOR_avx_sqrtv4df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2 = 1770,
   CODE_FOR_sse2_sqrtv2df2_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2_mask = 1771,
   CODE_FOR_sse2_sqrtv2df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsqrtv4sf2 = 1772,
  CODE_FOR_sse_vmsqrtv4sf2_round = 1773,
  CODE_FOR_sse_vmsqrtv4sf2_mask = 1774,
  CODE_FOR_sse_vmsqrtv4sf2_mask_round = 1775,
  CODE_FOR_sse2_vmsqrtv2df2 = 1776,
  CODE_FOR_sse2_vmsqrtv2df2_round = 1777,
  CODE_FOR_sse2_vmsqrtv2df2_mask = 1778,
  CODE_FOR_sse2_vmsqrtv2df2_mask_round = 1779,
  CODE_FOR_avx_rsqrtv8sf2 = 1788,
  CODE_FOR_sse_rsqrtv4sf2 = 1789,
  CODE_FOR_rsqrt14v16sf_mask = 1791,
  CODE_FOR_rsqrt14v8sf_mask = 1793,
  CODE_FOR_rsqrt14v4sf_mask = 1795,
  CODE_FOR_rsqrt14v8df_mask = 1797,
  CODE_FOR_rsqrt14v4df_mask = 1799,
  CODE_FOR_rsqrt14v2df_mask = 1801,
  CODE_FOR_rsqrt14v4sf = 1802,
  CODE_FOR_rsqrt14v2df = 1803,
  CODE_FOR_rsqrt14_v4sf_mask = 1804,
  CODE_FOR_rsqrt14_v2df_mask = 1805,
  CODE_FOR_sse_vmrsqrtv4sf2 = 1806,
  CODE_FOR_ieee_maxv16sf3 = 1840,
  CODE_FOR_ieee_maxv16sf3_round = 1841,
  CODE_FOR_ieee_maxv16sf3_mask = 1842,
  CODE_FOR_ieee_maxv16sf3_mask_round = 1843,
  CODE_FOR_ieee_minv16sf3 = 1844,
  CODE_FOR_ieee_minv16sf3_round = 1845,
  CODE_FOR_ieee_minv16sf3_mask = 1846,
  CODE_FOR_ieee_minv16sf3_mask_round = 1847,
  CODE_FOR_ieee_maxv8sf3 = 1848,
   CODE_FOR_ieee_maxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8sf3_mask = 1849,
   CODE_FOR_ieee_maxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3 = 1850,
   CODE_FOR_ieee_minv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3_mask = 1851,
   CODE_FOR_ieee_minv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3 = 1852,
   CODE_FOR_ieee_maxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3_mask = 1853,
   CODE_FOR_ieee_maxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3 = 1854,
   CODE_FOR_ieee_minv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3_mask = 1855,
   CODE_FOR_ieee_minv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8df3 = 1856,
  CODE_FOR_ieee_maxv8df3_round = 1857,
  CODE_FOR_ieee_maxv8df3_mask = 1858,
  CODE_FOR_ieee_maxv8df3_mask_round = 1859,
  CODE_FOR_ieee_minv8df3 = 1860,
  CODE_FOR_ieee_minv8df3_round = 1861,
  CODE_FOR_ieee_minv8df3_mask = 1862,
  CODE_FOR_ieee_minv8df3_mask_round = 1863,
  CODE_FOR_ieee_maxv4df3 = 1864,
   CODE_FOR_ieee_maxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4df3_mask = 1865,
   CODE_FOR_ieee_maxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3 = 1866,
   CODE_FOR_ieee_minv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3_mask = 1867,
   CODE_FOR_ieee_minv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3 = 1868,
   CODE_FOR_ieee_maxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3_mask = 1869,
   CODE_FOR_ieee_maxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3 = 1870,
   CODE_FOR_ieee_minv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3_mask = 1871,
   CODE_FOR_ieee_minv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsmaxv4sf3 = 1876,
  CODE_FOR_sse_vmsmaxv4sf3_mask = 1877,
  CODE_FOR_sse_vmsmaxv4sf3_round = 1878,
  CODE_FOR_sse_vmsmaxv4sf3_mask_round = 1879,
  CODE_FOR_sse_vmsminv4sf3 = 1880,
  CODE_FOR_sse_vmsminv4sf3_mask = 1881,
  CODE_FOR_sse_vmsminv4sf3_round = 1882,
  CODE_FOR_sse_vmsminv4sf3_mask_round = 1883,
  CODE_FOR_sse2_vmsmaxv2df3 = 1884,
  CODE_FOR_sse2_vmsmaxv2df3_mask = 1885,
  CODE_FOR_sse2_vmsmaxv2df3_round = 1886,
  CODE_FOR_sse2_vmsmaxv2df3_mask_round = 1887,
  CODE_FOR_sse2_vmsminv2df3 = 1888,
  CODE_FOR_sse2_vmsminv2df3_mask = 1889,
  CODE_FOR_sse2_vmsminv2df3_round = 1890,
  CODE_FOR_sse2_vmsminv2df3_mask_round = 1891,
  CODE_FOR_avx_addsubv4df3 = 1892,
  CODE_FOR_sse3_addsubv2df3 = 1893,
  CODE_FOR_avx_addsubv8sf3 = 1894,
  CODE_FOR_sse3_addsubv4sf3 = 1895,
  CODE_FOR_avx_haddv4df3 = 1896,
  CODE_FOR_avx_hsubv4df3 = 1897,
  CODE_FOR_sse3_hsubv2df3 = 1899,
  CODE_FOR_avx_haddv8sf3 = 1902,
  CODE_FOR_avx_hsubv8sf3 = 1903,
  CODE_FOR_sse3_haddv4sf3 = 1904,
  CODE_FOR_sse3_hsubv4sf3 = 1905,
  CODE_FOR_reducepv16sf_mask = 1908,
  CODE_FOR_reducepv16sf_mask_round = 1909,
  CODE_FOR_reducepv8sf_mask = 1912,
  CODE_FOR_reducepv8sf_mask_round = 1913,
  CODE_FOR_reducepv4sf_mask = 1916,
  CODE_FOR_reducepv4sf_mask_round = 1917,
  CODE_FOR_reducepv8df_mask = 1920,
  CODE_FOR_reducepv8df_mask_round = 1921,
  CODE_FOR_reducepv4df_mask = 1924,
  CODE_FOR_reducepv4df_mask_round = 1925,
  CODE_FOR_reducepv2df_mask = 1928,
  CODE_FOR_reducepv2df_mask_round = 1929,
  CODE_FOR_reducesv4sf = 1930,
  CODE_FOR_reducesv4sf_mask = 1931,
  CODE_FOR_reducesv4sf_round = 1932,
  CODE_FOR_reducesv4sf_mask_round = 1933,
  CODE_FOR_reducesv2df = 1934,
  CODE_FOR_reducesv2df_mask = 1935,
  CODE_FOR_reducesv2df_round = 1936,
  CODE_FOR_reducesv2df_mask_round = 1937,
  CODE_FOR_avx_cmpv8sf3 = 1938,
  CODE_FOR_avx_cmpv4sf3 = 1939,
  CODE_FOR_avx_cmpv4df3 = 1940,
  CODE_FOR_avx_cmpv2df3 = 1941,
  CODE_FOR_avx_vmcmpv4sf3 = 1958,
  CODE_FOR_avx_vmcmpv2df3 = 1959,
  CODE_FOR_avx_maskcmpv8sf3 = 1964,
  CODE_FOR_sse_maskcmpv4sf3 = 1965,
  CODE_FOR_avx_maskcmpv4df3 = 1966,
  CODE_FOR_sse2_maskcmpv2df3 = 1967,
  CODE_FOR_sse_vmmaskcmpv4sf3 = 1968,
  CODE_FOR_sse2_vmmaskcmpv2df3 = 1969,
  CODE_FOR_avx512f_cmpv16si3 = 1970,
  CODE_FOR_avx512f_cmpv16si3_round = 1971,
  CODE_FOR_avx512f_cmpv16si3_mask = 1972,
  CODE_FOR_avx512f_cmpv16si3_mask_round = 1973,
  CODE_FOR_avx512vl_cmpv8si3 = 1974,
   CODE_FOR_avx512vl_cmpv8si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8si3_mask = 1975,
   CODE_FOR_avx512vl_cmpv8si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3 = 1976,
   CODE_FOR_avx512vl_cmpv4si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3_mask = 1977,
   CODE_FOR_avx512vl_cmpv4si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8di3 = 1978,
  CODE_FOR_avx512f_cmpv8di3_round = 1979,
  CODE_FOR_avx512f_cmpv8di3_mask = 1980,
  CODE_FOR_avx512f_cmpv8di3_mask_round = 1981,
  CODE_FOR_avx512vl_cmpv4di3 = 1982,
   CODE_FOR_avx512vl_cmpv4di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4di3_mask = 1983,
   CODE_FOR_avx512vl_cmpv4di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3 = 1984,
   CODE_FOR_avx512vl_cmpv2di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3_mask = 1985,
   CODE_FOR_avx512vl_cmpv2di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv16sf3 = 1986,
  CODE_FOR_avx512f_cmpv16sf3_round = 1987,
  CODE_FOR_avx512f_cmpv16sf3_mask = 1988,
  CODE_FOR_avx512f_cmpv16sf3_mask_round = 1989,
  CODE_FOR_avx512vl_cmpv8sf3 = 1990,
   CODE_FOR_avx512vl_cmpv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8sf3_mask = 1991,
   CODE_FOR_avx512vl_cmpv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3 = 1992,
   CODE_FOR_avx512vl_cmpv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3_mask = 1993,
   CODE_FOR_avx512vl_cmpv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8df3 = 1994,
  CODE_FOR_avx512f_cmpv8df3_round = 1995,
  CODE_FOR_avx512f_cmpv8df3_mask = 1996,
  CODE_FOR_avx512f_cmpv8df3_mask_round = 1997,
  CODE_FOR_avx512vl_cmpv4df3 = 1998,
   CODE_FOR_avx512vl_cmpv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4df3_mask = 1999,
   CODE_FOR_avx512vl_cmpv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3 = 2000,
   CODE_FOR_avx512vl_cmpv2df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3_mask = 2001,
   CODE_FOR_avx512vl_cmpv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv64qi3 = 2014,
  CODE_FOR_avx512bw_cmpv64qi3_mask = 2015,
  CODE_FOR_avx512vl_cmpv16qi3 = 2016,
  CODE_FOR_avx512vl_cmpv16qi3_mask = 2017,
  CODE_FOR_avx512vl_cmpv32qi3 = 2018,
  CODE_FOR_avx512vl_cmpv32qi3_mask = 2019,
  CODE_FOR_avx512bw_cmpv32hi3 = 2020,
  CODE_FOR_avx512bw_cmpv32hi3_mask = 2021,
  CODE_FOR_avx512vl_cmpv16hi3 = 2022,
  CODE_FOR_avx512vl_cmpv16hi3_mask = 2023,
  CODE_FOR_avx512vl_cmpv8hi3 = 2024,
  CODE_FOR_avx512vl_cmpv8hi3_mask = 2025,
  CODE_FOR_avx512bw_ucmpv64qi3 = 2038,
  CODE_FOR_avx512bw_ucmpv64qi3_mask = 2039,
  CODE_FOR_avx512vl_ucmpv16qi3 = 2040,
  CODE_FOR_avx512vl_ucmpv16qi3_mask = 2041,
  CODE_FOR_avx512vl_ucmpv32qi3 = 2042,
  CODE_FOR_avx512vl_ucmpv32qi3_mask = 2043,
  CODE_FOR_avx512bw_ucmpv32hi3 = 2044,
  CODE_FOR_avx512bw_ucmpv32hi3_mask = 2045,
  CODE_FOR_avx512vl_ucmpv16hi3 = 2046,
  CODE_FOR_avx512vl_ucmpv16hi3_mask = 2047,
  CODE_FOR_avx512vl_ucmpv8hi3 = 2048,
  CODE_FOR_avx512vl_ucmpv8hi3_mask = 2049,
  CODE_FOR_avx512f_ucmpv16si3 = 2050,
  CODE_FOR_avx512f_ucmpv16si3_mask = 2051,
  CODE_FOR_avx512vl_ucmpv8si3 = 2052,
  CODE_FOR_avx512vl_ucmpv8si3_mask = 2053,
  CODE_FOR_avx512vl_ucmpv4si3 = 2054,
  CODE_FOR_avx512vl_ucmpv4si3_mask = 2055,
  CODE_FOR_avx512f_ucmpv8di3 = 2056,
  CODE_FOR_avx512f_ucmpv8di3_mask = 2057,
  CODE_FOR_avx512vl_ucmpv4di3 = 2058,
  CODE_FOR_avx512vl_ucmpv4di3_mask = 2059,
  CODE_FOR_avx512vl_ucmpv2di3 = 2060,
  CODE_FOR_avx512vl_ucmpv2di3_mask = 2061,
  CODE_FOR_avx512f_vmcmpv4sf3 = 2068,
  CODE_FOR_avx512f_vmcmpv4sf3_round = 2069,
  CODE_FOR_avx512f_vmcmpv2df3 = 2070,
  CODE_FOR_avx512f_vmcmpv2df3_round = 2071,
  CODE_FOR_avx512f_vmcmpv4sf3_mask = 2072,
  CODE_FOR_avx512f_vmcmpv4sf3_mask_round = 2073,
  CODE_FOR_avx512f_vmcmpv2df3_mask = 2074,
  CODE_FOR_avx512f_vmcmpv2df3_mask_round = 2075,
  CODE_FOR_sse_comi = 2076,
  CODE_FOR_sse_comi_round = 2077,
  CODE_FOR_sse_ucomi = 2078,
  CODE_FOR_sse_ucomi_round = 2079,
  CODE_FOR_sse2_comi = 2080,
  CODE_FOR_sse2_comi_round = 2081,
  CODE_FOR_sse2_ucomi = 2082,
  CODE_FOR_sse2_ucomi_round = 2083,
  CODE_FOR_avx_andnotv8sf3 = 2084,
  CODE_FOR_avx_andnotv8sf3_mask = 2085,
  CODE_FOR_sse_andnotv4sf3 = 2086,
  CODE_FOR_sse_andnotv4sf3_mask = 2087,
  CODE_FOR_avx_andnotv4df3 = 2088,
  CODE_FOR_avx_andnotv4df3_mask = 2089,
  CODE_FOR_sse2_andnotv2df3 = 2090,
  CODE_FOR_sse2_andnotv2df3_mask = 2091,
  CODE_FOR_avx512f_andnotv16sf3 = 2092,
  CODE_FOR_avx512f_andnotv16sf3_mask = 2093,
  CODE_FOR_avx512f_andnotv8df3 = 2094,
  CODE_FOR_avx512f_andnotv8df3_mask = 2095,
  CODE_FOR_fma_fmadd_v16sf_maskz_1 = 2152,
  CODE_FOR_fma_fmadd_v16sf_maskz_1_round = 2154,
  CODE_FOR_fma_fmadd_v8sf_maskz_1 = 2156,
   CODE_FOR_fma_fmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v4sf_maskz_1 = 2158,
   CODE_FOR_fma_fmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8df_maskz_1 = 2161,
  CODE_FOR_fma_fmadd_v8df_maskz_1_round = 2163,
  CODE_FOR_fma_fmadd_v4df_maskz_1 = 2165,
   CODE_FOR_fma_fmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v2df_maskz_1 = 2167,
   CODE_FOR_fma_fmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask = 2168,
  CODE_FOR_avx512f_fmadd_v16sf_mask_round = 2169,
  CODE_FOR_avx512vl_fmadd_v8sf_mask = 2170,
   CODE_FOR_avx512vl_fmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v4sf_mask = 2171,
   CODE_FOR_avx512vl_fmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v8df_mask = 2172,
  CODE_FOR_avx512f_fmadd_v8df_mask_round = 2173,
  CODE_FOR_avx512vl_fmadd_v4df_mask = 2174,
   CODE_FOR_avx512vl_fmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v2df_mask = 2175,
   CODE_FOR_avx512vl_fmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask3 = 2176,
  CODE_FOR_avx512f_fmadd_v16sf_mask3_round = 2177,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3 = 2178,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3_round = 2179,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3 = 2180,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3_round = 2181,
  CODE_FOR_avx512f_fmadd_v8df_mask3 = 2182,
  CODE_FOR_avx512f_fmadd_v8df_mask3_round = 2183,
  CODE_FOR_avx512vl_fmadd_v4df_mask3 = 2184,
  CODE_FOR_avx512vl_fmadd_v4df_mask3_round = 2185,
  CODE_FOR_avx512vl_fmadd_v2df_mask3 = 2186,
  CODE_FOR_avx512vl_fmadd_v2df_mask3_round = 2187,
  CODE_FOR_fma_fmsub_v16sf_maskz_1 = 2196,
  CODE_FOR_fma_fmsub_v16sf_maskz_1_round = 2198,
  CODE_FOR_fma_fmsub_v8sf_maskz_1 = 2200,
   CODE_FOR_fma_fmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v4sf_maskz_1 = 2202,
   CODE_FOR_fma_fmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8df_maskz_1 = 2205,
  CODE_FOR_fma_fmsub_v8df_maskz_1_round = 2207,
  CODE_FOR_fma_fmsub_v4df_maskz_1 = 2209,
   CODE_FOR_fma_fmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v2df_maskz_1 = 2211,
   CODE_FOR_fma_fmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v16sf_mask = 2212,
  CODE_FOR_avx512f_fmsub_v16sf_mask_round = 2213,
  CODE_FOR_avx512vl_fmsub_v8sf_mask = 2214,
  CODE_FOR_avx512vl_fmsub_v8sf_mask_round = 2215,
  CODE_FOR_avx512vl_fmsub_v4sf_mask = 2216,
  CODE_FOR_avx512vl_fmsub_v4sf_mask_round = 2217,
  CODE_FOR_avx512f_fmsub_v8df_mask = 2218,
  CODE_FOR_avx512f_fmsub_v8df_mask_round = 2219,
  CODE_FOR_avx512vl_fmsub_v4df_mask = 2220,
  CODE_FOR_avx512vl_fmsub_v4df_mask_round = 2221,
  CODE_FOR_avx512vl_fmsub_v2df_mask = 2222,
  CODE_FOR_avx512vl_fmsub_v2df_mask_round = 2223,
  CODE_FOR_avx512f_fmsub_v16sf_mask3 = 2224,
  CODE_FOR_avx512f_fmsub_v16sf_mask3_round = 2225,
  CODE_FOR_avx512vl_fmsub_v8sf_mask3 = 2226,
   CODE_FOR_avx512vl_fmsub_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v4sf_mask3 = 2227,
   CODE_FOR_avx512vl_fmsub_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v8df_mask3 = 2228,
  CODE_FOR_avx512f_fmsub_v8df_mask3_round = 2229,
  CODE_FOR_avx512vl_fmsub_v4df_mask3 = 2230,
   CODE_FOR_avx512vl_fmsub_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v2df_mask3 = 2231,
   CODE_FOR_avx512vl_fmsub_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1 = 2240,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1_round = 2242,
  CODE_FOR_fma_fnmadd_v8sf_maskz_1 = 2244,
   CODE_FOR_fma_fnmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v4sf_maskz_1 = 2246,
   CODE_FOR_fma_fnmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8df_maskz_1 = 2249,
  CODE_FOR_fma_fnmadd_v8df_maskz_1_round = 2251,
  CODE_FOR_fma_fnmadd_v4df_maskz_1 = 2253,
   CODE_FOR_fma_fnmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v2df_maskz_1 = 2255,
   CODE_FOR_fma_fnmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask = 2256,
  CODE_FOR_avx512f_fnmadd_v16sf_mask_round = 2257,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask = 2258,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask = 2259,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask = 2260,
  CODE_FOR_avx512f_fnmadd_v8df_mask_round = 2261,
  CODE_FOR_avx512vl_fnmadd_v4df_mask = 2262,
   CODE_FOR_avx512vl_fnmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask = 2263,
   CODE_FOR_avx512vl_fnmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3 = 2264,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3_round = 2265,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask3 = 2266,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask3 = 2267,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask3 = 2268,
  CODE_FOR_avx512f_fnmadd_v8df_mask3_round = 2269,
  CODE_FOR_avx512vl_fnmadd_v4df_mask3 = 2270,
   CODE_FOR_avx512vl_fnmadd_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask3 = 2271,
   CODE_FOR_avx512vl_fnmadd_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1 = 2294,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1_round = 2296,
  CODE_FOR_fma_fnmsub_v8sf_maskz_1 = 2298,
   CODE_FOR_fma_fnmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v4sf_maskz_1 = 2300,
   CODE_FOR_fma_fnmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8df_maskz_1 = 2303,
  CODE_FOR_fma_fnmsub_v8df_maskz_1_round = 2305,
  CODE_FOR_fma_fnmsub_v4df_maskz_1 = 2307,
   CODE_FOR_fma_fnmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v2df_maskz_1 = 2309,
   CODE_FOR_fma_fnmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask = 2310,
  CODE_FOR_avx512f_fnmsub_v16sf_mask_round = 2311,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask = 2312,
   CODE_FOR_avx512vl_fnmsub_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask = 2313,
   CODE_FOR_avx512vl_fnmsub_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v8df_mask = 2314,
  CODE_FOR_avx512f_fnmsub_v8df_mask_round = 2315,
  CODE_FOR_avx512vl_fnmsub_v4df_mask = 2316,
   CODE_FOR_avx512vl_fnmsub_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v2df_mask = 2317,
   CODE_FOR_avx512vl_fnmsub_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3 = 2318,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3_round = 2319,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3 = 2320,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3_round = 2321,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3 = 2322,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3_round = 2323,
  CODE_FOR_avx512f_fnmsub_v8df_mask3 = 2324,
  CODE_FOR_avx512f_fnmsub_v8df_mask3_round = 2325,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3 = 2326,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3_round = 2327,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3 = 2328,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3_round = 2329,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1 = 2336,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1_round = 2338,
  CODE_FOR_fma_fmaddsub_v8sf_maskz_1 = 2340,
   CODE_FOR_fma_fmaddsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v4sf_maskz_1 = 2342,
   CODE_FOR_fma_fmaddsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1 = 2345,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1_round = 2347,
  CODE_FOR_fma_fmaddsub_v4df_maskz_1 = 2349,
   CODE_FOR_fma_fmaddsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v2df_maskz_1 = 2351,
   CODE_FOR_fma_fmaddsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask = 2352,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask_round = 2353,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask = 2354,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask_round = 2355,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask = 2356,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask_round = 2357,
  CODE_FOR_avx512f_fmaddsub_v8df_mask = 2358,
  CODE_FOR_avx512f_fmaddsub_v8df_mask_round = 2359,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask = 2360,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask_round = 2361,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask = 2362,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask_round = 2363,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3 = 2364,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3_round = 2365,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3 = 2366,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3_round = 2367,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3 = 2368,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3_round = 2369,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3 = 2370,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3_round = 2371,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3 = 2372,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3_round = 2373,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3 = 2374,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3_round = 2375,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1 = 2382,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1_round = 2384,
  CODE_FOR_fma_fmsubadd_v8sf_maskz_1 = 2386,
   CODE_FOR_fma_fmsubadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v4sf_maskz_1 = 2388,
   CODE_FOR_fma_fmsubadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1 = 2391,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1_round = 2393,
  CODE_FOR_fma_fmsubadd_v4df_maskz_1 = 2395,
   CODE_FOR_fma_fmsubadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v2df_maskz_1 = 2397,
   CODE_FOR_fma_fmsubadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask = 2398,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask_round = 2399,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask = 2400,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask_round = 2401,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask = 2402,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask_round = 2403,
  CODE_FOR_avx512f_fmsubadd_v8df_mask = 2404,
  CODE_FOR_avx512f_fmsubadd_v8df_mask_round = 2405,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask = 2406,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask_round = 2407,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask = 2408,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask_round = 2409,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3 = 2410,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3_round = 2411,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3 = 2412,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3_round = 2413,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3 = 2414,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3_round = 2415,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3 = 2416,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3_round = 2417,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3 = 2418,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3_round = 2419,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3 = 2420,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3_round = 2421,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask = 2438,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask_round = 2439,
  CODE_FOR_avx512f_vmfmadd_v2df_mask = 2440,
  CODE_FOR_avx512f_vmfmadd_v2df_mask_round = 2441,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3 = 2442,
  CODE_FOR_avx512f_vmfmadd_v4sf_mask3_round = 2443,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3 = 2444,
  CODE_FOR_avx512f_vmfmadd_v2df_mask3_round = 2445,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1 = 2446,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_1_round = 2447,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1 = 2448,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_1_round = 2449,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3 = 2454,
  CODE_FOR_avx512f_vmfmsub_v4sf_mask3_round = 2455,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3 = 2456,
  CODE_FOR_avx512f_vmfmsub_v2df_mask3_round = 2457,
  CODE_FOR_sse_cvtpi2ps = 2494,
  CODE_FOR_sse_cvtps2pi = 2495,
  CODE_FOR_sse_cvttps2pi = 2496,
  CODE_FOR_sse_cvtsi2ss = 2497,
  CODE_FOR_sse_cvtsi2ss_round = 2498,
  CODE_FOR_sse_cvtsi2ssq = 2499,
  CODE_FOR_sse_cvtsi2ssq_round = 2500,
  CODE_FOR_sse_cvtss2si = 2501,
  CODE_FOR_sse_cvtss2si_round = 2502,
  CODE_FOR_sse_cvtss2siq = 2503,
  CODE_FOR_sse_cvtss2siq_round = 2504,
  CODE_FOR_sse_cvtss2si_2 = 2505,
  CODE_FOR_sse_cvtss2siq_2 = 2506,
  CODE_FOR_sse_cvttss2si = 2507,
  CODE_FOR_sse_cvttss2si_round = 2508,
  CODE_FOR_sse_cvttss2siq = 2509,
  CODE_FOR_sse_cvttss2siq_round = 2510,
  CODE_FOR_cvtusi2ss32 = 2511,
  CODE_FOR_cvtusi2ss32_round = 2512,
  CODE_FOR_cvtusi2sd32 = 2513,
   CODE_FOR_cvtusi2sd32_round = CODE_FOR_nothing,
  CODE_FOR_cvtusi2ss64 = 2514,
  CODE_FOR_cvtusi2ss64_round = 2515,
  CODE_FOR_cvtusi2sd64 = 2516,
  CODE_FOR_cvtusi2sd64_round = 2517,
  CODE_FOR_floatv16siv16sf2 = 2518,
  CODE_FOR_floatv16siv16sf2_round = 2519,
  CODE_FOR_floatv16siv16sf2_mask = 2520,
  CODE_FOR_floatv16siv16sf2_mask_round = 2521,
  CODE_FOR_floatv8siv8sf2 = 2522,
   CODE_FOR_floatv8siv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv8siv8sf2_mask = 2523,
   CODE_FOR_floatv8siv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2 = 2524,
   CODE_FOR_floatv4siv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2_mask = 2525,
   CODE_FOR_floatv4siv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv16siv16sf2 = 2526,
  CODE_FOR_ufloatv16siv16sf2_round = 2527,
  CODE_FOR_ufloatv16siv16sf2_mask = 2528,
  CODE_FOR_ufloatv16siv16sf2_mask_round = 2529,
  CODE_FOR_ufloatv8siv8sf2 = 2530,
  CODE_FOR_ufloatv8siv8sf2_round = 2531,
  CODE_FOR_ufloatv8siv8sf2_mask = 2532,
  CODE_FOR_ufloatv8siv8sf2_mask_round = 2533,
  CODE_FOR_ufloatv4siv4sf2 = 2534,
  CODE_FOR_ufloatv4siv4sf2_round = 2535,
  CODE_FOR_ufloatv4siv4sf2_mask = 2536,
  CODE_FOR_ufloatv4siv4sf2_mask_round = 2537,
  CODE_FOR_avx_fix_notruncv8sfv8si = 2538,
  CODE_FOR_avx_fix_notruncv8sfv8si_mask = 2539,
  CODE_FOR_sse2_fix_notruncv4sfv4si = 2540,
  CODE_FOR_sse2_fix_notruncv4sfv4si_mask = 2541,
  CODE_FOR_avx512f_fix_notruncv16sfv16si = 2542,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_round = 2543,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask = 2544,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask_round = 2545,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask = 2548,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask_round = 2549,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask = 2552,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask_round = 2553,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask = 2556,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask_round = 2557,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask = 2560,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask_round = 2561,
  CODE_FOR_avx512dq_cvtps2qqv4di_mask = 2563,
   CODE_FOR_avx512dq_cvtps2qqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2qqv2di_mask = 2565,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask = 2568,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask_round = 2569,
  CODE_FOR_avx512dq_cvtps2uqqv4di_mask = 2571,
   CODE_FOR_avx512dq_cvtps2uqqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2uqqv2di_mask = 2573,
  CODE_FOR_fix_truncv16sfv16si2 = 2574,
  CODE_FOR_fix_truncv16sfv16si2_round = 2575,
  CODE_FOR_fix_truncv16sfv16si2_mask = 2576,
  CODE_FOR_fix_truncv16sfv16si2_mask_round = 2577,
  CODE_FOR_ufix_truncv16sfv16si2 = 2578,
  CODE_FOR_ufix_truncv16sfv16si2_round = 2579,
  CODE_FOR_ufix_truncv16sfv16si2_mask = 2580,
  CODE_FOR_ufix_truncv16sfv16si2_mask_round = 2581,
  CODE_FOR_fix_truncv8sfv8si2 = 2582,
  CODE_FOR_fix_truncv8sfv8si2_mask = 2583,
  CODE_FOR_fix_truncv4sfv4si2 = 2584,
  CODE_FOR_fix_truncv4sfv4si2_mask = 2585,
  CODE_FOR_sse2_cvtpi2pd = 2586,
  CODE_FOR_floatunsv2siv2df2 = 2587,
  CODE_FOR_sse2_cvtpd2pi = 2588,
  CODE_FOR_sse2_cvttpd2pi = 2589,
  CODE_FOR_fixuns_truncv2dfv2si2 = 2590,
  CODE_FOR_sse2_cvtsi2sd = 2591,
  CODE_FOR_sse2_cvtsi2sdq = 2592,
  CODE_FOR_sse2_cvtsi2sdq_round = 2593,
  CODE_FOR_avx512f_vcvtss2usi = 2594,
  CODE_FOR_avx512f_vcvtss2usi_round = 2595,
  CODE_FOR_avx512f_vcvtss2usiq = 2596,
  CODE_FOR_avx512f_vcvtss2usiq_round = 2597,
  CODE_FOR_avx512f_vcvttss2usi = 2598,
  CODE_FOR_avx512f_vcvttss2usi_round = 2599,
  CODE_FOR_avx512f_vcvttss2usiq = 2600,
  CODE_FOR_avx512f_vcvttss2usiq_round = 2601,
  CODE_FOR_avx512f_vcvtsd2usi = 2602,
  CODE_FOR_avx512f_vcvtsd2usi_round = 2603,
  CODE_FOR_avx512f_vcvtsd2usiq = 2604,
  CODE_FOR_avx512f_vcvtsd2usiq_round = 2605,
  CODE_FOR_avx512f_vcvttsd2usi = 2606,
  CODE_FOR_avx512f_vcvttsd2usi_round = 2607,
  CODE_FOR_avx512f_vcvttsd2usiq = 2608,
  CODE_FOR_avx512f_vcvttsd2usiq_round = 2609,
  CODE_FOR_sse2_cvtsd2si = 2610,
  CODE_FOR_sse2_cvtsd2si_round = 2611,
  CODE_FOR_sse2_cvtsd2siq = 2612,
  CODE_FOR_sse2_cvtsd2siq_round = 2613,
  CODE_FOR_sse2_cvtsd2si_2 = 2614,
  CODE_FOR_sse2_cvtsd2siq_2 = 2615,
  CODE_FOR_sse2_cvttsd2si = 2616,
  CODE_FOR_sse2_cvttsd2si_round = 2617,
  CODE_FOR_sse2_cvttsd2siq = 2618,
  CODE_FOR_sse2_cvttsd2siq_round = 2619,
  CODE_FOR_floatv8siv8df2 = 2620,
  CODE_FOR_floatv8siv8df2_mask = 2621,
  CODE_FOR_floatv4siv4df2 = 2622,
  CODE_FOR_floatv4siv4df2_mask = 2623,
  CODE_FOR_floatv8div8df2 = 2624,
  CODE_FOR_floatv8div8df2_round = 2625,
  CODE_FOR_floatv8div8df2_mask = 2626,
  CODE_FOR_floatv8div8df2_mask_round = 2627,
  CODE_FOR_floatunsv8div8df2 = 2628,
  CODE_FOR_floatunsv8div8df2_round = 2629,
  CODE_FOR_floatunsv8div8df2_mask = 2630,
  CODE_FOR_floatunsv8div8df2_mask_round = 2631,
  CODE_FOR_floatv4div4df2 = 2632,
  CODE_FOR_floatv4div4df2_round = 2633,
  CODE_FOR_floatv4div4df2_mask = 2634,
  CODE_FOR_floatv4div4df2_mask_round = 2635,
  CODE_FOR_floatunsv4div4df2 = 2636,
  CODE_FOR_floatunsv4div4df2_round = 2637,
  CODE_FOR_floatunsv4div4df2_mask = 2638,
  CODE_FOR_floatunsv4div4df2_mask_round = 2639,
  CODE_FOR_floatv2div2df2 = 2640,
  CODE_FOR_floatv2div2df2_round = 2641,
  CODE_FOR_floatv2div2df2_mask = 2642,
  CODE_FOR_floatv2div2df2_mask_round = 2643,
  CODE_FOR_floatunsv2div2df2 = 2644,
  CODE_FOR_floatunsv2div2df2_round = 2645,
  CODE_FOR_floatunsv2div2df2_mask = 2646,
  CODE_FOR_floatunsv2div2df2_mask_round = 2647,
  CODE_FOR_floatv8div8sf2 = 2648,
  CODE_FOR_floatv8div8sf2_round = 2649,
  CODE_FOR_floatv8div8sf2_mask = 2650,
  CODE_FOR_floatv8div8sf2_mask_round = 2651,
  CODE_FOR_floatunsv8div8sf2 = 2652,
  CODE_FOR_floatunsv8div8sf2_round = 2653,
  CODE_FOR_floatunsv8div8sf2_mask = 2654,
  CODE_FOR_floatunsv8div8sf2_mask_round = 2655,
  CODE_FOR_floatv4div4sf2 = 2656,
   CODE_FOR_floatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4div4sf2_mask = 2657,
   CODE_FOR_floatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2 = 2658,
   CODE_FOR_floatunsv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatunsv4div4sf2_mask = 2659,
   CODE_FOR_floatunsv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv8siv8df2 = 2666,
  CODE_FOR_ufloatv8siv8df2_mask = 2667,
  CODE_FOR_ufloatv4siv4df2 = 2668,
  CODE_FOR_ufloatv4siv4df2_mask = 2669,
  CODE_FOR_ufloatv2siv2df2 = 2670,
  CODE_FOR_ufloatv2siv2df2_mask = 2671,
  CODE_FOR_avx512f_cvtdq2pd512_2 = 2672,
  CODE_FOR_avx_cvtdq2pd256_2 = 2673,
  CODE_FOR_sse2_cvtdq2pd = 2674,
  CODE_FOR_sse2_cvtdq2pd_mask = 2675,
  CODE_FOR_avx512f_cvtpd2dq512 = 2676,
  CODE_FOR_avx512f_cvtpd2dq512_round = 2677,
  CODE_FOR_avx512f_cvtpd2dq512_mask = 2678,
  CODE_FOR_avx512f_cvtpd2dq512_mask_round = 2679,
  CODE_FOR_avx_cvtpd2dq256 = 2680,
  CODE_FOR_avx_cvtpd2dq256_mask = 2681,
  CODE_FOR_sse2_cvtpd2dq = 2683,
  CODE_FOR_sse2_cvtpd2dq_mask = 2684,
  CODE_FOR_ufix_notruncv8dfv8si2 = 2686,
  CODE_FOR_ufix_notruncv8dfv8si2_round = 2687,
  CODE_FOR_ufix_notruncv8dfv8si2_mask = 2688,
  CODE_FOR_ufix_notruncv8dfv8si2_mask_round = 2689,
  CODE_FOR_ufix_notruncv4dfv4si2 = 2690,
  CODE_FOR_ufix_notruncv4dfv4si2_round = 2691,
  CODE_FOR_ufix_notruncv4dfv4si2_mask = 2692,
  CODE_FOR_ufix_notruncv4dfv4si2_mask_round = 2693,
  CODE_FOR_ufix_notruncv2dfv2si2 = 2694,
  CODE_FOR_ufix_notruncv2dfv2si2_mask = 2695,
  CODE_FOR_fix_truncv8dfv8si2 = 2697,
  CODE_FOR_fix_truncv8dfv8si2_round = 2698,
  CODE_FOR_fix_truncv8dfv8si2_mask = 2699,
  CODE_FOR_fix_truncv8dfv8si2_mask_round = 2700,
  CODE_FOR_fixuns_truncv8dfv8si2 = 2701,
  CODE_FOR_fixuns_truncv8dfv8si2_round = 2702,
  CODE_FOR_fixuns_truncv8dfv8si2_mask = 2703,
  CODE_FOR_fixuns_truncv8dfv8si2_mask_round = 2704,
  CODE_FOR_ufix_truncv2dfv2si2 = 2705,
  CODE_FOR_ufix_truncv2dfv2si2_mask = 2706,
  CODE_FOR_fix_truncv4dfv4si2 = 2708,
  CODE_FOR_fix_truncv4dfv4si2_mask = 2709,
  CODE_FOR_ufix_truncv4dfv4si2 = 2710,
  CODE_FOR_ufix_truncv4dfv4si2_mask = 2711,
  CODE_FOR_fix_truncv8dfv8di2 = 2712,
  CODE_FOR_fix_truncv8dfv8di2_round = 2713,
  CODE_FOR_fix_truncv8dfv8di2_mask = 2714,
  CODE_FOR_fix_truncv8dfv8di2_mask_round = 2715,
  CODE_FOR_fixuns_truncv8dfv8di2 = 2716,
  CODE_FOR_fixuns_truncv8dfv8di2_round = 2717,
  CODE_FOR_fixuns_truncv8dfv8di2_mask = 2718,
  CODE_FOR_fixuns_truncv8dfv8di2_mask_round = 2719,
  CODE_FOR_fix_truncv4dfv4di2 = 2720,
   CODE_FOR_fix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4dfv4di2_mask = 2721,
   CODE_FOR_fix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2 = 2722,
   CODE_FOR_fixuns_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4dfv4di2_mask = 2723,
   CODE_FOR_fixuns_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2 = 2724,
   CODE_FOR_fix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2_mask = 2725,
   CODE_FOR_fix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2 = 2726,
   CODE_FOR_fixuns_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv2dfv2di2_mask = 2727,
   CODE_FOR_fixuns_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv8dfv8di2 = 2728,
  CODE_FOR_fix_notruncv8dfv8di2_round = 2729,
  CODE_FOR_fix_notruncv8dfv8di2_mask = 2730,
  CODE_FOR_fix_notruncv8dfv8di2_mask_round = 2731,
  CODE_FOR_fix_notruncv4dfv4di2 = 2732,
   CODE_FOR_fix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv4dfv4di2_mask = 2733,
   CODE_FOR_fix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2 = 2734,
   CODE_FOR_fix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2_mask = 2735,
   CODE_FOR_fix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv8dfv8di2 = 2736,
  CODE_FOR_ufix_notruncv8dfv8di2_round = 2737,
  CODE_FOR_ufix_notruncv8dfv8di2_mask = 2738,
  CODE_FOR_ufix_notruncv8dfv8di2_mask_round = 2739,
  CODE_FOR_ufix_notruncv4dfv4di2 = 2740,
   CODE_FOR_ufix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv4dfv4di2_mask = 2741,
   CODE_FOR_ufix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2 = 2742,
   CODE_FOR_ufix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2_mask = 2743,
   CODE_FOR_ufix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv8sfv8di2 = 2744,
  CODE_FOR_fix_truncv8sfv8di2_round = 2745,
  CODE_FOR_fix_truncv8sfv8di2_mask = 2746,
  CODE_FOR_fix_truncv8sfv8di2_mask_round = 2747,
  CODE_FOR_fixuns_truncv8sfv8di2 = 2748,
  CODE_FOR_fixuns_truncv8sfv8di2_round = 2749,
  CODE_FOR_fixuns_truncv8sfv8di2_mask = 2750,
  CODE_FOR_fixuns_truncv8sfv8di2_mask_round = 2751,
  CODE_FOR_fix_truncv4sfv4di2 = 2752,
   CODE_FOR_fix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4sfv4di2_mask = 2753,
   CODE_FOR_fix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2 = 2754,
   CODE_FOR_fixuns_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fixuns_truncv4sfv4di2_mask = 2755,
   CODE_FOR_fixuns_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_fix_truncv2sfv2di2 = 2756,
  CODE_FOR_avx512dq_fix_truncv2sfv2di2_mask = 2757,
  CODE_FOR_avx512dq_fixuns_truncv2sfv2di2 = 2758,
  CODE_FOR_avx512dq_fixuns_truncv2sfv2di2_mask = 2759,
  CODE_FOR_ufix_truncv8sfv8si2 = 2760,
  CODE_FOR_ufix_truncv8sfv8si2_mask = 2761,
  CODE_FOR_ufix_truncv4sfv4si2 = 2762,
  CODE_FOR_ufix_truncv4sfv4si2_mask = 2763,
  CODE_FOR_sse2_cvttpd2dq = 2764,
  CODE_FOR_sse2_cvttpd2dq_mask = 2765,
  CODE_FOR_sse2_cvtsd2ss = 2767,
  CODE_FOR_sse2_cvtsd2ss_round = 2768,
  CODE_FOR_sse2_cvtsd2ss_mask = 2769,
  CODE_FOR_sse2_cvtsd2ss_mask_round = 2770,
  CODE_FOR_sse2_cvtss2sd = 2772,
  CODE_FOR_sse2_cvtss2sd_round = 2773,
  CODE_FOR_sse2_cvtss2sd_mask = 2774,
  CODE_FOR_sse2_cvtss2sd_mask_round = 2775,
  CODE_FOR_avx512f_cvtpd2ps512_mask = 2779,
  CODE_FOR_avx512f_cvtpd2ps512_mask_round = 2780,
  CODE_FOR_avx_cvtpd2ps256 = 2781,
  CODE_FOR_avx_cvtpd2ps256_mask = 2782,
  CODE_FOR_truncv2dfv2sf2 = 2784,
  CODE_FOR_avx512f_cvtps2pd512 = 2787,
  CODE_FOR_avx512f_cvtps2pd512_round = 2788,
  CODE_FOR_avx512f_cvtps2pd512_mask = 2789,
  CODE_FOR_avx512f_cvtps2pd512_mask_round = 2790,
  CODE_FOR_avx_cvtps2pd256 = 2791,
   CODE_FOR_avx_cvtps2pd256_round = CODE_FOR_nothing,
  CODE_FOR_avx_cvtps2pd256_mask = 2792,
   CODE_FOR_avx_cvtps2pd256_mask_round = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_lo_v16sf = 2794,
  CODE_FOR_avx512bw_cvtb2maskv64qi = 2795,
  CODE_FOR_avx512vl_cvtb2maskv16qi = 2796,
  CODE_FOR_avx512vl_cvtb2maskv32qi = 2797,
  CODE_FOR_avx512bw_cvtw2maskv32hi = 2798,
  CODE_FOR_avx512vl_cvtw2maskv16hi = 2799,
  CODE_FOR_avx512vl_cvtw2maskv8hi = 2800,
  CODE_FOR_avx512f_cvtd2maskv16si = 2801,
  CODE_FOR_avx512vl_cvtd2maskv8si = 2802,
  CODE_FOR_avx512vl_cvtd2maskv4si = 2803,
  CODE_FOR_avx512f_cvtq2maskv8di = 2804,
  CODE_FOR_avx512vl_cvtq2maskv4di = 2805,
  CODE_FOR_avx512vl_cvtq2maskv2di = 2806,
  CODE_FOR_sse2_cvtps2pd = 2819,
  CODE_FOR_sse2_cvtps2pd_mask = 2820,
  CODE_FOR_extendv2sfv2df2 = 2821,
  CODE_FOR_sse_movhlps = 2822,
  CODE_FOR_sse_movlhps = 2823,
  CODE_FOR_avx512f_unpckhps512_mask = 2825,
  CODE_FOR_avx_unpckhps256 = 2826,
  CODE_FOR_avx_unpckhps256_mask = 2827,
  CODE_FOR_vec_interleave_highv4sf = 2828,
  CODE_FOR_vec_interleave_highv4sf_mask = 2829,
  CODE_FOR_avx512f_unpcklps512_mask = 2831,
  CODE_FOR_avx_unpcklps256 = 2832,
  CODE_FOR_avx_unpcklps256_mask = 2833,
  CODE_FOR_unpcklps128_mask = 2834,
  CODE_FOR_vec_interleave_lowv4sf = 2835,
  CODE_FOR_avx_movshdup256 = 2836,
  CODE_FOR_avx_movshdup256_mask = 2837,
  CODE_FOR_sse3_movshdup = 2838,
  CODE_FOR_sse3_movshdup_mask = 2839,
  CODE_FOR_avx512f_movshdup512_mask = 2841,
  CODE_FOR_avx_movsldup256 = 2842,
  CODE_FOR_avx_movsldup256_mask = 2843,
  CODE_FOR_sse3_movsldup = 2844,
  CODE_FOR_sse3_movsldup_mask = 2845,
  CODE_FOR_avx512f_movsldup512_mask = 2847,
  CODE_FOR_avx_shufps256_1 = 2848,
  CODE_FOR_avx_shufps256_1_mask = 2849,
  CODE_FOR_sse_shufps_v4sf_mask = 2850,
  CODE_FOR_sse_shufps_v4si = 2851,
  CODE_FOR_sse_shufps_v4sf = 2852,
  CODE_FOR_sse_storehps = 2853,
  CODE_FOR_sse_loadhps = 2854,
  CODE_FOR_sse_storelps = 2855,
  CODE_FOR_sse_loadlps = 2856,
  CODE_FOR_sse_movss = 2857,
  CODE_FOR_avx2_vec_dupv8sf = 2858,
  CODE_FOR_avx2_vec_dupv4sf = 2859,
  CODE_FOR_avx2_vec_dupv8sf_1 = 2860,
  CODE_FOR_avx512f_vec_dupv16sf_1 = 2861,
  CODE_FOR_avx512f_vec_dupv8df_1 = 2862,
  CODE_FOR_vec_setv4si_0 = 2867,
  CODE_FOR_vec_setv4sf_0 = 2868,
  CODE_FOR_vec_setv8si_0 = 2870,
  CODE_FOR_vec_setv8sf_0 = 2871,
  CODE_FOR_vec_setv16si_0 = 2872,
  CODE_FOR_vec_setv16sf_0 = 2873,
  CODE_FOR_sse4_1_insertps = 2874,
  CODE_FOR_vec_setv2df_0 = 2875,
  CODE_FOR_avx512dq_vextractf64x2_1_mask = 2879,
  CODE_FOR_avx512dq_vextracti64x2_1_mask = 2880,
  CODE_FOR_avx512f_vextractf32x4_1_mask = 2883,
  CODE_FOR_avx512f_vextracti32x4_1_mask = 2884,
  CODE_FOR_vec_extract_lo_v8df_mask = 2887,
  CODE_FOR_vec_extract_lo_v8di_mask = 2888,
  CODE_FOR_vec_extract_lo_v8df = 2889,
  CODE_FOR_vec_extract_lo_v8di = 2890,
  CODE_FOR_vec_extract_hi_v8df_mask = 2891,
  CODE_FOR_vec_extract_hi_v8di_mask = 2892,
  CODE_FOR_vec_extract_hi_v8df = 2893,
  CODE_FOR_vec_extract_hi_v8di = 2894,
  CODE_FOR_vec_extract_hi_v16sf_mask = 2895,
  CODE_FOR_vec_extract_hi_v16si_mask = 2896,
  CODE_FOR_vec_extract_hi_v16sf = 2897,
  CODE_FOR_vec_extract_hi_v16si = 2898,
  CODE_FOR_vec_extract_lo_v16sf_mask = 2899,
  CODE_FOR_vec_extract_lo_v16si_mask = 2900,
  CODE_FOR_vec_extract_lo_v16sf = 2901,
  CODE_FOR_vec_extract_lo_v16si = 2902,
  CODE_FOR_vec_extract_lo_v4di_mask = 2903,
  CODE_FOR_vec_extract_lo_v4df_mask = 2904,
  CODE_FOR_vec_extract_lo_v4di = 2905,
  CODE_FOR_vec_extract_lo_v4df = 2906,
  CODE_FOR_vec_extract_hi_v4di_mask = 2907,
  CODE_FOR_vec_extract_hi_v4df_mask = 2908,
  CODE_FOR_vec_extract_hi_v4di = 2909,
  CODE_FOR_vec_extract_hi_v4df = 2910,
  CODE_FOR_vec_extract_lo_v8si_mask = 2911,
  CODE_FOR_vec_extract_lo_v8sf_mask = 2912,
  CODE_FOR_vec_extract_lo_v8si = 2913,
  CODE_FOR_vec_extract_lo_v8sf = 2914,
  CODE_FOR_vec_extract_hi_v8si_mask = 2915,
  CODE_FOR_vec_extract_hi_v8sf_mask = 2916,
  CODE_FOR_vec_extract_hi_v8si = 2917,
  CODE_FOR_vec_extract_hi_v8sf = 2918,
  CODE_FOR_vec_extract_lo_v32hi = 2919,
  CODE_FOR_vec_extract_hi_v32hi = 2920,
  CODE_FOR_vec_extract_lo_v16hi = 2921,
  CODE_FOR_vec_extract_hi_v16hi = 2922,
  CODE_FOR_vec_extract_lo_v64qi = 2923,
  CODE_FOR_vec_extract_hi_v64qi = 2924,
  CODE_FOR_vec_extract_lo_v32qi = 2925,
  CODE_FOR_vec_extract_hi_v32qi = 2926,
  CODE_FOR_avx512f_unpckhpd512_mask = 2928,
  CODE_FOR_avx_unpckhpd256 = 2929,
  CODE_FOR_avx_unpckhpd256_mask = 2930,
  CODE_FOR_avx512vl_unpckhpd128_mask = 2931,
  CODE_FOR_avx512vl_unpcklpd128_mask = 2937,
  CODE_FOR_avx512f_vmscalefv4sf = 2939,
  CODE_FOR_avx512f_vmscalefv4sf_round = 2940,
  CODE_FOR_avx512f_vmscalefv4sf_mask = 2941,
  CODE_FOR_avx512f_vmscalefv4sf_mask_round = 2942,
  CODE_FOR_avx512f_vmscalefv2df = 2943,
  CODE_FOR_avx512f_vmscalefv2df_round = 2944,
  CODE_FOR_avx512f_vmscalefv2df_mask = 2945,
  CODE_FOR_avx512f_vmscalefv2df_mask_round = 2946,
  CODE_FOR_avx512f_scalefv16sf = 2947,
  CODE_FOR_avx512f_scalefv16sf_round = 2948,
  CODE_FOR_avx512f_scalefv16sf_mask = 2949,
  CODE_FOR_avx512f_scalefv16sf_mask_round = 2950,
  CODE_FOR_avx512vl_scalefv8sf = 2951,
  CODE_FOR_avx512vl_scalefv8sf_round = 2952,
  CODE_FOR_avx512vl_scalefv8sf_mask = 2953,
  CODE_FOR_avx512vl_scalefv8sf_mask_round = 2954,
  CODE_FOR_avx512vl_scalefv4sf = 2955,
  CODE_FOR_avx512vl_scalefv4sf_round = 2956,
  CODE_FOR_avx512vl_scalefv4sf_mask = 2957,
  CODE_FOR_avx512vl_scalefv4sf_mask_round = 2958,
  CODE_FOR_avx512f_scalefv8df = 2959,
  CODE_FOR_avx512f_scalefv8df_round = 2960,
  CODE_FOR_avx512f_scalefv8df_mask = 2961,
  CODE_FOR_avx512f_scalefv8df_mask_round = 2962,
  CODE_FOR_avx512vl_scalefv4df = 2963,
  CODE_FOR_avx512vl_scalefv4df_round = 2964,
  CODE_FOR_avx512vl_scalefv4df_mask = 2965,
  CODE_FOR_avx512vl_scalefv4df_mask_round = 2966,
  CODE_FOR_avx512vl_scalefv2df = 2967,
  CODE_FOR_avx512vl_scalefv2df_round = 2968,
  CODE_FOR_avx512vl_scalefv2df_mask = 2969,
  CODE_FOR_avx512vl_scalefv2df_mask_round = 2970,
  CODE_FOR_avx512f_vternlogv16si = 2971,
  CODE_FOR_avx512f_vternlogv16si_maskz_1 = 2972,
  CODE_FOR_avx512vl_vternlogv8si = 2973,
  CODE_FOR_avx512vl_vternlogv8si_maskz_1 = 2974,
  CODE_FOR_avx512vl_vternlogv4si = 2975,
  CODE_FOR_avx512vl_vternlogv4si_maskz_1 = 2976,
  CODE_FOR_avx512f_vternlogv8di = 2977,
  CODE_FOR_avx512f_vternlogv8di_maskz_1 = 2978,
  CODE_FOR_avx512vl_vternlogv4di = 2979,
  CODE_FOR_avx512vl_vternlogv4di_maskz_1 = 2980,
  CODE_FOR_avx512vl_vternlogv2di = 2981,
  CODE_FOR_avx512vl_vternlogv2di_maskz_1 = 2982,
  CODE_FOR_avx512f_vternlogv16si_mask = 2983,
  CODE_FOR_avx512vl_vternlogv8si_mask = 2984,
  CODE_FOR_avx512vl_vternlogv4si_mask = 2985,
  CODE_FOR_avx512f_vternlogv8di_mask = 2986,
  CODE_FOR_avx512vl_vternlogv4di_mask = 2987,
  CODE_FOR_avx512vl_vternlogv2di_mask = 2988,
  CODE_FOR_avx512f_getexpv16sf = 2989,
  CODE_FOR_avx512f_getexpv16sf_round = 2990,
  CODE_FOR_avx512f_getexpv16sf_mask = 2991,
  CODE_FOR_avx512f_getexpv16sf_mask_round = 2992,
  CODE_FOR_avx512vl_getexpv8sf = 2993,
  CODE_FOR_avx512vl_getexpv8sf_round = 2994,
  CODE_FOR_avx512vl_getexpv8sf_mask = 2995,
  CODE_FOR_avx512vl_getexpv8sf_mask_round = 2996,
  CODE_FOR_avx512vl_getexpv4sf = 2997,
  CODE_FOR_avx512vl_getexpv4sf_round = 2998,
  CODE_FOR_avx512vl_getexpv4sf_mask = 2999,
  CODE_FOR_avx512vl_getexpv4sf_mask_round = 3000,
  CODE_FOR_avx512f_getexpv8df = 3001,
  CODE_FOR_avx512f_getexpv8df_round = 3002,
  CODE_FOR_avx512f_getexpv8df_mask = 3003,
  CODE_FOR_avx512f_getexpv8df_mask_round = 3004,
  CODE_FOR_avx512vl_getexpv4df = 3005,
  CODE_FOR_avx512vl_getexpv4df_round = 3006,
  CODE_FOR_avx512vl_getexpv4df_mask = 3007,
  CODE_FOR_avx512vl_getexpv4df_mask_round = 3008,
  CODE_FOR_avx512vl_getexpv2df = 3009,
  CODE_FOR_avx512vl_getexpv2df_round = 3010,
  CODE_FOR_avx512vl_getexpv2df_mask = 3011,
  CODE_FOR_avx512vl_getexpv2df_mask_round = 3012,
  CODE_FOR_avx512f_sgetexpv4sf = 3013,
  CODE_FOR_avx512f_sgetexpv4sf_mask = 3014,
  CODE_FOR_avx512f_sgetexpv4sf_round = 3015,
  CODE_FOR_avx512f_sgetexpv4sf_mask_round = 3016,
  CODE_FOR_avx512f_sgetexpv2df = 3017,
  CODE_FOR_avx512f_sgetexpv2df_mask = 3018,
  CODE_FOR_avx512f_sgetexpv2df_round = 3019,
  CODE_FOR_avx512f_sgetexpv2df_mask_round = 3020,
  CODE_FOR_avx512f_alignv16si_mask = 3022,
  CODE_FOR_avx512vl_alignv8si_mask = 3024,
  CODE_FOR_avx512vl_alignv4si_mask = 3026,
  CODE_FOR_avx512f_alignv8di_mask = 3028,
  CODE_FOR_avx512vl_alignv4di_mask = 3030,
  CODE_FOR_avx512vl_alignv2di_mask = 3032,
  CODE_FOR_avx512f_fixupimmv16sf = 3033,
  CODE_FOR_avx512f_fixupimmv16sf_round = 3034,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1 = 3035,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1_round = 3036,
  CODE_FOR_avx512vl_fixupimmv8sf = 3037,
  CODE_FOR_avx512vl_fixupimmv8sf_round = 3038,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1 = 3039,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1_round = 3040,
  CODE_FOR_avx512vl_fixupimmv4sf = 3041,
  CODE_FOR_avx512vl_fixupimmv4sf_round = 3042,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1 = 3043,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1_round = 3044,
  CODE_FOR_avx512f_fixupimmv8df = 3045,
  CODE_FOR_avx512f_fixupimmv8df_round = 3046,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1 = 3047,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1_round = 3048,
  CODE_FOR_avx512vl_fixupimmv4df = 3049,
  CODE_FOR_avx512vl_fixupimmv4df_round = 3050,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1 = 3051,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1_round = 3052,
  CODE_FOR_avx512vl_fixupimmv2df = 3053,
  CODE_FOR_avx512vl_fixupimmv2df_round = 3054,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1 = 3055,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1_round = 3056,
  CODE_FOR_avx512f_fixupimmv16sf_mask = 3057,
  CODE_FOR_avx512f_fixupimmv16sf_mask_round = 3058,
  CODE_FOR_avx512vl_fixupimmv8sf_mask = 3059,
  CODE_FOR_avx512vl_fixupimmv8sf_mask_round = 3060,
  CODE_FOR_avx512vl_fixupimmv4sf_mask = 3061,
  CODE_FOR_avx512vl_fixupimmv4sf_mask_round = 3062,
  CODE_FOR_avx512f_fixupimmv8df_mask = 3063,
  CODE_FOR_avx512f_fixupimmv8df_mask_round = 3064,
  CODE_FOR_avx512vl_fixupimmv4df_mask = 3065,
  CODE_FOR_avx512vl_fixupimmv4df_mask_round = 3066,
  CODE_FOR_avx512vl_fixupimmv2df_mask = 3067,
  CODE_FOR_avx512vl_fixupimmv2df_mask_round = 3068,
  CODE_FOR_avx512f_sfixupimmv4sf = 3069,
  CODE_FOR_avx512f_sfixupimmv4sf_round = 3070,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1 = 3071,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1_round = 3072,
  CODE_FOR_avx512f_sfixupimmv2df = 3073,
  CODE_FOR_avx512f_sfixupimmv2df_round = 3074,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1 = 3075,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1_round = 3076,
  CODE_FOR_avx512f_sfixupimmv4sf_mask = 3077,
  CODE_FOR_avx512f_sfixupimmv4sf_mask_round = 3078,
  CODE_FOR_avx512f_sfixupimmv2df_mask = 3079,
  CODE_FOR_avx512f_sfixupimmv2df_mask_round = 3080,
  CODE_FOR_avx512f_rndscalev16sf = 3081,
  CODE_FOR_avx512f_rndscalev16sf_round = 3082,
  CODE_FOR_avx512f_rndscalev16sf_mask = 3083,
  CODE_FOR_avx512f_rndscalev16sf_mask_round = 3084,
  CODE_FOR_avx512vl_rndscalev8sf = 3085,
  CODE_FOR_avx512vl_rndscalev8sf_round = 3086,
  CODE_FOR_avx512vl_rndscalev8sf_mask = 3087,
  CODE_FOR_avx512vl_rndscalev8sf_mask_round = 3088,
  CODE_FOR_avx512vl_rndscalev4sf = 3089,
  CODE_FOR_avx512vl_rndscalev4sf_round = 3090,
  CODE_FOR_avx512vl_rndscalev4sf_mask = 3091,
  CODE_FOR_avx512vl_rndscalev4sf_mask_round = 3092,
  CODE_FOR_avx512f_rndscalev8df = 3093,
  CODE_FOR_avx512f_rndscalev8df_round = 3094,
  CODE_FOR_avx512f_rndscalev8df_mask = 3095,
  CODE_FOR_avx512f_rndscalev8df_mask_round = 3096,
  CODE_FOR_avx512vl_rndscalev4df = 3097,
  CODE_FOR_avx512vl_rndscalev4df_round = 3098,
  CODE_FOR_avx512vl_rndscalev4df_mask = 3099,
  CODE_FOR_avx512vl_rndscalev4df_mask_round = 3100,
  CODE_FOR_avx512vl_rndscalev2df = 3101,
  CODE_FOR_avx512vl_rndscalev2df_round = 3102,
  CODE_FOR_avx512vl_rndscalev2df_mask = 3103,
  CODE_FOR_avx512vl_rndscalev2df_mask_round = 3104,
  CODE_FOR_avx512f_rndscalev4sf = 3105,
  CODE_FOR_avx512f_rndscalev4sf_mask = 3106,
  CODE_FOR_avx512f_rndscalev4sf_round = 3107,
  CODE_FOR_avx512f_rndscalev4sf_mask_round = 3108,
  CODE_FOR_avx512f_rndscalev2df = 3109,
  CODE_FOR_avx512f_rndscalev2df_mask = 3110,
  CODE_FOR_avx512f_rndscalev2df_round = 3111,
  CODE_FOR_avx512f_rndscalev2df_mask_round = 3112,
  CODE_FOR_avx512f_shufps512_1 = 3117,
  CODE_FOR_avx512f_shufps512_1_mask = 3118,
  CODE_FOR_avx512f_shufpd512_1 = 3119,
  CODE_FOR_avx512f_shufpd512_1_mask = 3120,
  CODE_FOR_avx_shufpd256_1 = 3121,
  CODE_FOR_avx_shufpd256_1_mask = 3122,
  CODE_FOR_sse2_shufpd_v2df_mask = 3123,
  CODE_FOR_avx2_interleave_highv4di = 3124,
  CODE_FOR_avx2_interleave_highv4di_mask = 3125,
  CODE_FOR_avx512f_interleave_highv8di_mask = 3127,
  CODE_FOR_vec_interleave_highv2di = 3128,
  CODE_FOR_vec_interleave_highv2di_mask = 3129,
  CODE_FOR_avx2_interleave_lowv4di = 3130,
  CODE_FOR_avx2_interleave_lowv4di_mask = 3131,
  CODE_FOR_avx512f_interleave_lowv8di_mask = 3133,
  CODE_FOR_vec_interleave_lowv2di = 3134,
  CODE_FOR_vec_interleave_lowv2di_mask = 3135,
  CODE_FOR_sse2_shufpd_v2di = 3136,
  CODE_FOR_sse2_shufpd_v2df = 3137,
  CODE_FOR_sse2_storehpd = 3138,
  CODE_FOR_sse2_storelpd = 3140,
  CODE_FOR_sse2_loadhpd = 3142,
  CODE_FOR_sse2_loadlpd = 3143,
  CODE_FOR_sse2_movsd = 3144,
  CODE_FOR_vec_dupv2df = 3145,
  CODE_FOR_vec_dupv2df_mask = 3146,
  CODE_FOR_vec_concatv2df = 3147,
  CODE_FOR_vec_setv8df_0 = 3148,
  CODE_FOR_vec_setv4df_0 = 3149,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask = 3162,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask = 3163,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask = 3164,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask = 3165,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask = 3166,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask = 3167,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask = 3168,
  CODE_FOR_avx512f_truncatev8div8si2_mask = 3169,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask = 3170,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask = 3171,
  CODE_FOR_avx512f_truncatev8div8hi2_mask = 3172,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask = 3173,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2 = 3174,
  CODE_FOR_avx512bw_truncatev32hiv32qi2 = 3175,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2 = 3176,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask = 3177,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask = 3178,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask = 3179,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask = 3189,
  CODE_FOR_avx512vl_truncatev4div4si2_mask = 3190,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask = 3191,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask = 3192,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask = 3193,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask = 3194,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask = 3195,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask = 3196,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask = 3197,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2 = 3198,
  CODE_FOR_avx512vl_truncatev4div4qi2 = 3199,
  CODE_FOR_avx512vl_us_truncatev4div4qi2 = 3200,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2 = 3201,
  CODE_FOR_avx512vl_truncatev2div2qi2 = 3202,
  CODE_FOR_avx512vl_us_truncatev2div2qi2 = 3203,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2 = 3204,
  CODE_FOR_avx512vl_truncatev8siv8qi2 = 3205,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2 = 3206,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2 = 3207,
  CODE_FOR_avx512vl_truncatev4siv4qi2 = 3208,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2 = 3209,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2 = 3210,
  CODE_FOR_avx512vl_truncatev8hiv8qi2 = 3211,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2 = 3212,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask = 3219,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask = 3220,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask = 3221,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask_store_2 = 3228,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask_store_2 = 3229,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask_store_2 = 3230,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask = 3243,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask = 3244,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask = 3245,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask = 3246,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask = 3247,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask = 3248,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask_store_2 = 3261,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask_store_2 = 3262,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask_store_2 = 3263,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask_store_2 = 3264,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask_store_2 = 3265,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask_store_2 = 3266,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask = 3279,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask = 3280,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask = 3281,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask = 3282,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask = 3283,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask = 3284,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask_store_2 = 3297,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask_store_2 = 3298,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask_store_2 = 3299,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask_store_2 = 3300,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask_store_2 = 3301,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask_store_2 = 3302,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2 = 3303,
  CODE_FOR_avx512vl_truncatev4div4hi2 = 3304,
  CODE_FOR_avx512vl_us_truncatev4div4hi2 = 3305,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2 = 3306,
  CODE_FOR_avx512vl_truncatev2div2hi2 = 3307,
  CODE_FOR_avx512vl_us_truncatev2div2hi2 = 3308,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2 = 3309,
  CODE_FOR_avx512vl_truncatev4siv4hi2 = 3310,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2 = 3311,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask = 3324,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask = 3325,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask = 3326,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask = 3327,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask = 3328,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask = 3329,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask_store_2 = 3342,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask_store_2 = 3343,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask_store_2 = 3344,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask_store_2 = 3345,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask_store_2 = 3346,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask_store_2 = 3347,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask = 3354,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask = 3355,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask = 3356,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask_store_2 = 3363,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask_store_2 = 3364,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask_store_2 = 3365,
  CODE_FOR_avx512vl_ss_truncatev2div2si2 = 3366,
  CODE_FOR_avx512vl_truncatev2div2si2 = 3367,
  CODE_FOR_avx512vl_us_truncatev2div2si2 = 3368,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask = 3375,
  CODE_FOR_avx512vl_truncatev2div2si2_mask = 3376,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask = 3377,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask_store_2 = 3384,
  CODE_FOR_avx512vl_truncatev2div2si2_mask_store_2 = 3385,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask_store_2 = 3386,
  CODE_FOR_avx512f_ss_truncatev8div16qi2 = 3387,
  CODE_FOR_avx512f_truncatev8div16qi2 = 3388,
  CODE_FOR_avx512f_us_truncatev8div16qi2 = 3389,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask = 3396,
  CODE_FOR_avx512f_truncatev8div16qi2_mask = 3397,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask = 3398,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask_store_2 = 3405,
  CODE_FOR_avx512f_truncatev8div16qi2_mask_store_2 = 3406,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask_store_2 = 3407,
  CODE_FOR_avx512bw_pmaddwd512v32hi = 3534,
  CODE_FOR_avx512bw_pmaddwd512v32hi_mask = 3535,
  CODE_FOR_avx512bw_pmaddwd512v16hi = 3536,
  CODE_FOR_avx512bw_pmaddwd512v16hi_mask = 3537,
  CODE_FOR_avx512bw_pmaddwd512v8hi = 3538,
  CODE_FOR_avx512bw_pmaddwd512v8hi_mask = 3539,
  CODE_FOR_avx512dq_mulv8di3 = 3542,
  CODE_FOR_avx512dq_mulv8di3_mask = 3543,
  CODE_FOR_avx512dq_mulv4di3 = 3544,
  CODE_FOR_avx512dq_mulv4di3_mask = 3545,
  CODE_FOR_avx512dq_mulv2di3 = 3546,
  CODE_FOR_avx512dq_mulv2di3_mask = 3547,
  CODE_FOR_ashrv16hi3_mask = 3555,
  CODE_FOR_ashrv8hi3_mask = 3557,
  CODE_FOR_ashrv8si3_mask = 3559,
  CODE_FOR_ashrv4si3_mask = 3561,
  CODE_FOR_ashrv2di3_mask = 3563,
  CODE_FOR_ashrv16hi3 = 3564,
  CODE_FOR_ashrv8hi3 = 3565,
  CODE_FOR_ashrv8si3 = 3566,
  CODE_FOR_ashrv4si3 = 3567,
  CODE_FOR_ashrv32hi3 = 3568,
  CODE_FOR_ashrv32hi3_mask = 3569,
  CODE_FOR_ashrv4di3 = 3570,
  CODE_FOR_ashrv4di3_mask = 3571,
  CODE_FOR_ashrv16si3 = 3572,
  CODE_FOR_ashrv16si3_mask = 3573,
  CODE_FOR_ashrv8di3 = 3574,
  CODE_FOR_ashrv8di3_mask = 3575,
  CODE_FOR_ashlv16hi3_mask = 3577,
  CODE_FOR_lshrv16hi3_mask = 3579,
  CODE_FOR_ashlv8hi3_mask = 3581,
  CODE_FOR_lshrv8hi3_mask = 3583,
  CODE_FOR_ashlv8si3_mask = 3585,
  CODE_FOR_lshrv8si3_mask = 3587,
  CODE_FOR_ashlv4si3_mask = 3589,
  CODE_FOR_lshrv4si3_mask = 3591,
  CODE_FOR_ashlv4di3_mask = 3593,
  CODE_FOR_lshrv4di3_mask = 3595,
  CODE_FOR_ashlv2di3_mask = 3597,
  CODE_FOR_lshrv2di3_mask = 3599,
  CODE_FOR_ashlv16hi3 = 3600,
  CODE_FOR_lshrv16hi3 = 3601,
  CODE_FOR_ashlv8hi3 = 3602,
  CODE_FOR_lshrv8hi3 = 3603,
  CODE_FOR_ashlv8si3 = 3604,
  CODE_FOR_lshrv8si3 = 3605,
  CODE_FOR_ashlv4si3 = 3606,
  CODE_FOR_lshrv4si3 = 3607,
  CODE_FOR_ashlv4di3 = 3608,
  CODE_FOR_lshrv4di3 = 3609,
  CODE_FOR_ashlv2di3 = 3610,
  CODE_FOR_lshrv2di3 = 3611,
  CODE_FOR_ashlv32hi3 = 3612,
  CODE_FOR_ashlv32hi3_mask = 3613,
  CODE_FOR_lshrv32hi3 = 3614,
  CODE_FOR_lshrv32hi3_mask = 3615,
  CODE_FOR_ashlv16si3 = 3616,
  CODE_FOR_ashlv16si3_mask = 3617,
  CODE_FOR_lshrv16si3 = 3618,
  CODE_FOR_lshrv16si3_mask = 3619,
  CODE_FOR_ashlv8di3 = 3620,
  CODE_FOR_ashlv8di3_mask = 3621,
  CODE_FOR_lshrv8di3 = 3622,
  CODE_FOR_lshrv8di3_mask = 3623,
  CODE_FOR_avx512bw_ashlv4ti3 = 3624,
  CODE_FOR_avx512bw_lshrv4ti3 = 3625,
  CODE_FOR_avx512bw_ashlv2ti3 = 3626,
  CODE_FOR_avx512bw_lshrv2ti3 = 3627,
  CODE_FOR_avx512bw_ashlv1ti3 = 3628,
  CODE_FOR_avx512bw_lshrv1ti3 = 3629,
  CODE_FOR_avx2_ashlv2ti3 = 3630,
  CODE_FOR_avx2_lshrv2ti3 = 3631,
  CODE_FOR_sse2_ashlv1ti3 = 3632,
  CODE_FOR_sse2_lshrv1ti3 = 3633,
  CODE_FOR_avx512f_rolvv16si = 3634,
  CODE_FOR_avx512f_rolvv16si_mask = 3635,
  CODE_FOR_avx512f_rorvv16si = 3636,
  CODE_FOR_avx512f_rorvv16si_mask = 3637,
  CODE_FOR_avx512vl_rolvv8si = 3638,
  CODE_FOR_avx512vl_rolvv8si_mask = 3639,
  CODE_FOR_avx512vl_rorvv8si = 3640,
  CODE_FOR_avx512vl_rorvv8si_mask = 3641,
  CODE_FOR_avx512vl_rolvv4si = 3642,
  CODE_FOR_avx512vl_rolvv4si_mask = 3643,
  CODE_FOR_avx512vl_rorvv4si = 3644,
  CODE_FOR_avx512vl_rorvv4si_mask = 3645,
  CODE_FOR_avx512f_rolvv8di = 3646,
  CODE_FOR_avx512f_rolvv8di_mask = 3647,
  CODE_FOR_avx512f_rorvv8di = 3648,
  CODE_FOR_avx512f_rorvv8di_mask = 3649,
  CODE_FOR_avx512vl_rolvv4di = 3650,
  CODE_FOR_avx512vl_rolvv4di_mask = 3651,
  CODE_FOR_avx512vl_rorvv4di = 3652,
  CODE_FOR_avx512vl_rorvv4di_mask = 3653,
  CODE_FOR_avx512vl_rolvv2di = 3654,
  CODE_FOR_avx512vl_rolvv2di_mask = 3655,
  CODE_FOR_avx512vl_rorvv2di = 3656,
  CODE_FOR_avx512vl_rorvv2di_mask = 3657,
  CODE_FOR_avx512f_rolv16si = 3658,
  CODE_FOR_avx512f_rolv16si_mask = 3659,
  CODE_FOR_avx512f_rorv16si = 3660,
  CODE_FOR_avx512f_rorv16si_mask = 3661,
  CODE_FOR_avx512vl_rolv8si = 3662,
  CODE_FOR_avx512vl_rolv8si_mask = 3663,
  CODE_FOR_avx512vl_rorv8si = 3664,
  CODE_FOR_avx512vl_rorv8si_mask = 3665,
  CODE_FOR_avx512vl_rolv4si = 3666,
  CODE_FOR_avx512vl_rolv4si_mask = 3667,
  CODE_FOR_avx512vl_rorv4si = 3668,
  CODE_FOR_avx512vl_rorv4si_mask = 3669,
  CODE_FOR_avx512f_rolv8di = 3670,
  CODE_FOR_avx512f_rolv8di_mask = 3671,
  CODE_FOR_avx512f_rorv8di = 3672,
  CODE_FOR_avx512f_rorv8di_mask = 3673,
  CODE_FOR_avx512vl_rolv4di = 3674,
  CODE_FOR_avx512vl_rolv4di_mask = 3675,
  CODE_FOR_avx512vl_rorv4di = 3676,
  CODE_FOR_avx512vl_rorv4di_mask = 3677,
  CODE_FOR_avx512vl_rolv2di = 3678,
  CODE_FOR_avx512vl_rolv2di_mask = 3679,
  CODE_FOR_avx512vl_rorv2di = 3680,
  CODE_FOR_avx512vl_rorv2di_mask = 3681,
  CODE_FOR_smaxv64qi3_mask = 3743,
  CODE_FOR_sminv64qi3_mask = 3745,
  CODE_FOR_umaxv64qi3_mask = 3747,
  CODE_FOR_uminv64qi3_mask = 3749,
  CODE_FOR_smaxv16qi3_mask = 3751,
  CODE_FOR_sminv16qi3_mask = 3753,
  CODE_FOR_umaxv16qi3_mask = 3755,
  CODE_FOR_uminv16qi3_mask = 3757,
  CODE_FOR_smaxv32qi3_mask = 3759,
  CODE_FOR_sminv32qi3_mask = 3761,
  CODE_FOR_umaxv32qi3_mask = 3763,
  CODE_FOR_uminv32qi3_mask = 3765,
  CODE_FOR_smaxv32hi3_mask = 3767,
  CODE_FOR_sminv32hi3_mask = 3769,
  CODE_FOR_umaxv32hi3_mask = 3771,
  CODE_FOR_uminv32hi3_mask = 3773,
  CODE_FOR_smaxv16hi3_mask = 3775,
  CODE_FOR_sminv16hi3_mask = 3777,
  CODE_FOR_umaxv16hi3_mask = 3779,
  CODE_FOR_uminv16hi3_mask = 3781,
  CODE_FOR_smaxv8hi3_mask = 3783,
  CODE_FOR_sminv8hi3_mask = 3785,
  CODE_FOR_umaxv8hi3_mask = 3787,
  CODE_FOR_uminv8hi3_mask = 3789,
  CODE_FOR_avx512bw_eqv64qi3_1 = 3838,
  CODE_FOR_avx512bw_eqv64qi3_mask_1 = 3839,
  CODE_FOR_avx512vl_eqv16qi3_1 = 3840,
  CODE_FOR_avx512vl_eqv16qi3_mask_1 = 3841,
  CODE_FOR_avx512vl_eqv32qi3_1 = 3842,
  CODE_FOR_avx512vl_eqv32qi3_mask_1 = 3843,
  CODE_FOR_avx512bw_eqv32hi3_1 = 3844,
  CODE_FOR_avx512bw_eqv32hi3_mask_1 = 3845,
  CODE_FOR_avx512vl_eqv16hi3_1 = 3846,
  CODE_FOR_avx512vl_eqv16hi3_mask_1 = 3847,
  CODE_FOR_avx512vl_eqv8hi3_1 = 3848,
  CODE_FOR_avx512vl_eqv8hi3_mask_1 = 3849,
  CODE_FOR_avx512f_eqv16si3_1 = 3850,
  CODE_FOR_avx512f_eqv16si3_mask_1 = 3851,
  CODE_FOR_avx512vl_eqv8si3_1 = 3852,
  CODE_FOR_avx512vl_eqv8si3_mask_1 = 3853,
  CODE_FOR_avx512vl_eqv4si3_1 = 3854,
  CODE_FOR_avx512vl_eqv4si3_mask_1 = 3855,
  CODE_FOR_avx512f_eqv8di3_1 = 3856,
  CODE_FOR_avx512f_eqv8di3_mask_1 = 3857,
  CODE_FOR_avx512vl_eqv4di3_1 = 3858,
  CODE_FOR_avx512vl_eqv4di3_mask_1 = 3859,
  CODE_FOR_avx512vl_eqv2di3_1 = 3860,
  CODE_FOR_avx512vl_eqv2di3_mask_1 = 3861,
  CODE_FOR_sse4_2_gtv2di3 = 3866,
  CODE_FOR_avx2_gtv32qi3 = 3867,
  CODE_FOR_avx2_gtv16hi3 = 3868,
  CODE_FOR_avx2_gtv8si3 = 3869,
  CODE_FOR_avx2_gtv4di3 = 3870,
  CODE_FOR_avx512f_gtv16si3 = 3879,
  CODE_FOR_avx512f_gtv16si3_mask = 3880,
  CODE_FOR_avx512vl_gtv8si3 = 3881,
  CODE_FOR_avx512vl_gtv8si3_mask = 3882,
  CODE_FOR_avx512vl_gtv4si3 = 3883,
  CODE_FOR_avx512vl_gtv4si3_mask = 3884,
  CODE_FOR_avx512f_gtv8di3 = 3885,
  CODE_FOR_avx512f_gtv8di3_mask = 3886,
  CODE_FOR_avx512vl_gtv4di3 = 3887,
  CODE_FOR_avx512vl_gtv4di3_mask = 3888,
  CODE_FOR_avx512vl_gtv2di3 = 3889,
  CODE_FOR_avx512vl_gtv2di3_mask = 3890,
  CODE_FOR_avx512bw_gtv64qi3 = 3891,
  CODE_FOR_avx512bw_gtv64qi3_mask = 3892,
  CODE_FOR_avx512vl_gtv16qi3 = 3893,
  CODE_FOR_avx512vl_gtv16qi3_mask = 3894,
  CODE_FOR_avx512vl_gtv32qi3 = 3895,
  CODE_FOR_avx512vl_gtv32qi3_mask = 3896,
  CODE_FOR_avx512bw_gtv32hi3 = 3897,
  CODE_FOR_avx512bw_gtv32hi3_mask = 3898,
  CODE_FOR_avx512vl_gtv16hi3 = 3899,
  CODE_FOR_avx512vl_gtv16hi3_mask = 3900,
  CODE_FOR_avx512vl_gtv8hi3 = 3901,
  CODE_FOR_avx512vl_gtv8hi3_mask = 3902,
  CODE_FOR_sse2_gtv16qi3 = 3903,
  CODE_FOR_sse2_gtv8hi3 = 3904,
  CODE_FOR_sse2_gtv4si3 = 3905,
  CODE_FOR_one_cmplv16si2_mask = 3907,
  CODE_FOR_one_cmplv8di2_mask = 3909,
   CODE_FOR_one_cmplv64qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv32qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv16qi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv32hi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv16hi2_mask = CODE_FOR_nothing,
   CODE_FOR_one_cmplv8hi2_mask = CODE_FOR_nothing,
  CODE_FOR_one_cmplv8si2_mask = 3917,
  CODE_FOR_one_cmplv4si2_mask = 3919,
  CODE_FOR_one_cmplv4di2_mask = 3921,
  CODE_FOR_one_cmplv2di2_mask = 3923,
  CODE_FOR_andv16si3_mask = 3943,
  CODE_FOR_iorv16si3_mask = 3945,
  CODE_FOR_xorv16si3_mask = 3947,
  CODE_FOR_andv8si3_mask = 3949,
  CODE_FOR_iorv8si3_mask = 3951,
  CODE_FOR_xorv8si3_mask = 3953,
  CODE_FOR_andv4si3_mask = 3955,
  CODE_FOR_iorv4si3_mask = 3957,
  CODE_FOR_xorv4si3_mask = 3959,
  CODE_FOR_andv8di3_mask = 3961,
  CODE_FOR_iorv8di3_mask = 3963,
  CODE_FOR_xorv8di3_mask = 3965,
  CODE_FOR_andv4di3_mask = 3967,
  CODE_FOR_iorv4di3_mask = 3969,
  CODE_FOR_xorv4di3_mask = 3971,
  CODE_FOR_andv2di3_mask = 3973,
  CODE_FOR_iorv2di3_mask = 3975,
  CODE_FOR_xorv2di3_mask = 3977,
  CODE_FOR_avx512bw_testmv64qi3 = 3996,
  CODE_FOR_avx512bw_testmv64qi3_mask = 3997,
  CODE_FOR_avx512vl_testmv32qi3 = 3998,
  CODE_FOR_avx512vl_testmv32qi3_mask = 3999,
  CODE_FOR_avx512vl_testmv16qi3 = 4000,
  CODE_FOR_avx512vl_testmv16qi3_mask = 4001,
  CODE_FOR_avx512bw_testmv32hi3 = 4002,
  CODE_FOR_avx512bw_testmv32hi3_mask = 4003,
  CODE_FOR_avx512vl_testmv16hi3 = 4004,
  CODE_FOR_avx512vl_testmv16hi3_mask = 4005,
  CODE_FOR_avx512vl_testmv8hi3 = 4006,
  CODE_FOR_avx512vl_testmv8hi3_mask = 4007,
  CODE_FOR_avx512f_testmv16si3 = 4008,
  CODE_FOR_avx512f_testmv16si3_mask = 4009,
  CODE_FOR_avx512vl_testmv8si3 = 4010,
  CODE_FOR_avx512vl_testmv8si3_mask = 4011,
  CODE_FOR_avx512vl_testmv4si3 = 4012,
  CODE_FOR_avx512vl_testmv4si3_mask = 4013,
  CODE_FOR_avx512f_testmv8di3 = 4014,
  CODE_FOR_avx512f_testmv8di3_mask = 4015,
  CODE_FOR_avx512vl_testmv4di3 = 4016,
  CODE_FOR_avx512vl_testmv4di3_mask = 4017,
  CODE_FOR_avx512vl_testmv2di3 = 4018,
  CODE_FOR_avx512vl_testmv2di3_mask = 4019,
  CODE_FOR_avx512bw_testnmv64qi3 = 4020,
  CODE_FOR_avx512bw_testnmv64qi3_mask = 4021,
  CODE_FOR_avx512vl_testnmv32qi3 = 4022,
  CODE_FOR_avx512vl_testnmv32qi3_mask = 4023,
  CODE_FOR_avx512vl_testnmv16qi3 = 4024,
  CODE_FOR_avx512vl_testnmv16qi3_mask = 4025,
  CODE_FOR_avx512bw_testnmv32hi3 = 4026,
  CODE_FOR_avx512bw_testnmv32hi3_mask = 4027,
  CODE_FOR_avx512vl_testnmv16hi3 = 4028,
  CODE_FOR_avx512vl_testnmv16hi3_mask = 4029,
  CODE_FOR_avx512vl_testnmv8hi3 = 4030,
  CODE_FOR_avx512vl_testnmv8hi3_mask = 4031,
  CODE_FOR_avx512f_testnmv16si3 = 4032,
  CODE_FOR_avx512f_testnmv16si3_mask = 4033,
  CODE_FOR_avx512vl_testnmv8si3 = 4034,
  CODE_FOR_avx512vl_testnmv8si3_mask = 4035,
  CODE_FOR_avx512vl_testnmv4si3 = 4036,
  CODE_FOR_avx512vl_testnmv4si3_mask = 4037,
  CODE_FOR_avx512f_testnmv8di3 = 4038,
  CODE_FOR_avx512f_testnmv8di3_mask = 4039,
  CODE_FOR_avx512vl_testnmv4di3 = 4040,
  CODE_FOR_avx512vl_testnmv4di3_mask = 4041,
  CODE_FOR_avx512vl_testnmv2di3 = 4042,
  CODE_FOR_avx512vl_testnmv2di3_mask = 4043,
  CODE_FOR_avx512bw_packsswb = 4188,
  CODE_FOR_avx512bw_packsswb_mask = 4189,
  CODE_FOR_avx2_packsswb = 4190,
  CODE_FOR_avx2_packsswb_mask = 4191,
  CODE_FOR_sse2_packsswb = 4192,
  CODE_FOR_sse2_packsswb_mask = 4193,
  CODE_FOR_avx512bw_packssdw = 4194,
  CODE_FOR_avx512bw_packssdw_mask = 4195,
  CODE_FOR_avx2_packssdw = 4196,
  CODE_FOR_avx2_packssdw_mask = 4197,
  CODE_FOR_sse2_packssdw = 4198,
  CODE_FOR_sse2_packssdw_mask = 4199,
  CODE_FOR_avx512bw_packuswb = 4200,
  CODE_FOR_avx512bw_packuswb_mask = 4201,
  CODE_FOR_avx2_packuswb = 4202,
  CODE_FOR_avx2_packuswb_mask = 4203,
  CODE_FOR_sse2_packuswb = 4204,
  CODE_FOR_sse2_packuswb_mask = 4205,
  CODE_FOR_avx512bw_interleave_highv64qi = 4206,
  CODE_FOR_avx512bw_interleave_highv64qi_mask = 4207,
  CODE_FOR_avx2_interleave_highv32qi = 4208,
  CODE_FOR_avx2_interleave_highv32qi_mask = 4209,
  CODE_FOR_vec_interleave_highv16qi = 4210,
  CODE_FOR_vec_interleave_highv16qi_mask = 4211,
  CODE_FOR_avx512bw_interleave_lowv64qi = 4212,
  CODE_FOR_avx512bw_interleave_lowv64qi_mask = 4213,
  CODE_FOR_avx2_interleave_lowv32qi = 4214,
  CODE_FOR_avx2_interleave_lowv32qi_mask = 4215,
  CODE_FOR_vec_interleave_lowv16qi = 4216,
  CODE_FOR_vec_interleave_lowv16qi_mask = 4217,
  CODE_FOR_avx512bw_interleave_highv32hi = 4218,
  CODE_FOR_avx512bw_interleave_highv32hi_mask = 4219,
  CODE_FOR_avx2_interleave_highv16hi = 4220,
  CODE_FOR_avx2_interleave_highv16hi_mask = 4221,
  CODE_FOR_vec_interleave_highv8hi = 4222,
  CODE_FOR_vec_interleave_highv8hi_mask = 4223,
  CODE_FOR_avx512bw_interleave_lowv32hi_mask = 4225,
  CODE_FOR_avx2_interleave_lowv16hi = 4226,
  CODE_FOR_avx2_interleave_lowv16hi_mask = 4227,
  CODE_FOR_vec_interleave_lowv8hi = 4228,
  CODE_FOR_vec_interleave_lowv8hi_mask = 4229,
  CODE_FOR_avx2_interleave_highv8si = 4230,
  CODE_FOR_avx2_interleave_highv8si_mask = 4231,
  CODE_FOR_avx512f_interleave_highv16si_mask = 4233,
  CODE_FOR_vec_interleave_highv4si = 4234,
  CODE_FOR_vec_interleave_highv4si_mask = 4235,
  CODE_FOR_avx2_interleave_lowv8si = 4236,
  CODE_FOR_avx2_interleave_lowv8si_mask = 4237,
  CODE_FOR_avx512f_interleave_lowv16si_mask = 4239,
  CODE_FOR_vec_interleave_lowv4si = 4240,
  CODE_FOR_vec_interleave_lowv4si_mask = 4241,
  CODE_FOR_sse4_1_pinsrb = 4242,
  CODE_FOR_sse2_pinsrw = 4243,
  CODE_FOR_sse4_1_pinsrd = 4244,
  CODE_FOR_sse4_1_pinsrq = 4245,
  CODE_FOR_avx512dq_vinsertf64x2_1_mask = 4251,
  CODE_FOR_avx512dq_vinserti64x2_1_mask = 4253,
  CODE_FOR_avx512f_vinsertf32x4_1_mask = 4255,
  CODE_FOR_avx512f_vinserti32x4_1_mask = 4257,
  CODE_FOR_vec_set_lo_v16sf = 4258,
  CODE_FOR_vec_set_lo_v16sf_mask = 4259,
  CODE_FOR_vec_set_lo_v16si = 4260,
  CODE_FOR_vec_set_lo_v16si_mask = 4261,
  CODE_FOR_vec_set_hi_v16sf = 4262,
  CODE_FOR_vec_set_hi_v16sf_mask = 4263,
  CODE_FOR_vec_set_hi_v16si = 4264,
  CODE_FOR_vec_set_hi_v16si_mask = 4265,
  CODE_FOR_vec_set_lo_v8df = 4266,
  CODE_FOR_vec_set_lo_v8df_mask = 4267,
  CODE_FOR_vec_set_lo_v8di = 4268,
  CODE_FOR_vec_set_lo_v8di_mask = 4269,
  CODE_FOR_vec_set_hi_v8df = 4270,
  CODE_FOR_vec_set_hi_v8df_mask = 4271,
  CODE_FOR_vec_set_hi_v8di = 4272,
  CODE_FOR_vec_set_hi_v8di_mask = 4273,
  CODE_FOR_avx512dq_shuf_i64x2_1_mask = 4275,
  CODE_FOR_avx512dq_shuf_f64x2_1_mask = 4277,
  CODE_FOR_avx512f_shuf_f64x2_1 = 4278,
  CODE_FOR_avx512f_shuf_f64x2_1_mask = 4279,
  CODE_FOR_avx512f_shuf_i64x2_1 = 4280,
  CODE_FOR_avx512f_shuf_i64x2_1_mask = 4281,
  CODE_FOR_avx512vl_shuf_i32x4_1 = 4282,
  CODE_FOR_avx512vl_shuf_i32x4_1_mask = 4283,
  CODE_FOR_avx512vl_shuf_f32x4_1 = 4284,
  CODE_FOR_avx512vl_shuf_f32x4_1_mask = 4285,
  CODE_FOR_avx512f_shuf_f32x4_1 = 4286,
  CODE_FOR_avx512f_shuf_f32x4_1_mask = 4287,
  CODE_FOR_avx512f_shuf_i32x4_1 = 4288,
  CODE_FOR_avx512f_shuf_i32x4_1_mask = 4289,
  CODE_FOR_avx512f_pshufd_1 = 4290,
  CODE_FOR_avx512f_pshufd_1_mask = 4291,
  CODE_FOR_avx2_pshufd_1 = 4292,
  CODE_FOR_avx2_pshufd_1_mask = 4293,
  CODE_FOR_sse2_pshufd_1 = 4294,
  CODE_FOR_sse2_pshufd_1_mask = 4295,
  CODE_FOR_avx512bw_pshuflwv32hi_mask = 4297,
  CODE_FOR_avx2_pshuflw_1 = 4298,
  CODE_FOR_avx2_pshuflw_1_mask = 4299,
  CODE_FOR_sse2_pshuflw_1 = 4300,
  CODE_FOR_sse2_pshuflw_1_mask = 4301,
  CODE_FOR_avx512bw_pshufhwv32hi_mask = 4303,
  CODE_FOR_avx2_pshufhw_1 = 4304,
  CODE_FOR_avx2_pshufhw_1_mask = 4305,
  CODE_FOR_sse2_pshufhw_1 = 4306,
  CODE_FOR_sse2_pshufhw_1_mask = 4307,
  CODE_FOR_sse2_loadld = 4308,
  CODE_FOR_vec_concatv2di = 4334,
  CODE_FOR_vec_setv8di_0 = 4336,
  CODE_FOR_vec_setv4di_0 = 4337,
  CODE_FOR_avx512f_psadbw = 4350,
  CODE_FOR_avx2_psadbw = 4351,
  CODE_FOR_sse2_psadbw = 4352,
  CODE_FOR_avx_movmskps256 = 4353,
  CODE_FOR_sse_movmskps = 4354,
  CODE_FOR_avx_movmskpd256 = 4355,
  CODE_FOR_sse2_movmskpd = 4356,
  CODE_FOR_avx2_pmovmskb = 4389,
  CODE_FOR_sse2_pmovmskb = 4390,
  CODE_FOR_sse_ldmxcsr = 4402,
  CODE_FOR_sse_stmxcsr = 4403,
  CODE_FOR_sse2_clflush = 4404,
  CODE_FOR_sse3_mwait = 4405,
  CODE_FOR_sse3_monitor_si = 4406,
  CODE_FOR_sse3_monitor_di = 4407,
  CODE_FOR_avx2_phaddwv16hi3 = 4408,
  CODE_FOR_avx2_phaddswv16hi3 = 4409,
  CODE_FOR_avx2_phsubwv16hi3 = 4410,
  CODE_FOR_avx2_phsubswv16hi3 = 4411,
  CODE_FOR_ssse3_phaddwv8hi3 = 4412,
  CODE_FOR_ssse3_phaddswv8hi3 = 4413,
  CODE_FOR_ssse3_phsubwv8hi3 = 4414,
  CODE_FOR_ssse3_phsubswv8hi3 = 4415,
  CODE_FOR_ssse3_phaddwv4hi3 = 4416,
  CODE_FOR_ssse3_phaddswv4hi3 = 4417,
  CODE_FOR_ssse3_phsubwv4hi3 = 4418,
  CODE_FOR_ssse3_phsubswv4hi3 = 4419,
  CODE_FOR_avx2_phadddv8si3 = 4420,
  CODE_FOR_avx2_phsubdv8si3 = 4421,
  CODE_FOR_ssse3_phadddv4si3 = 4422,
  CODE_FOR_ssse3_phsubdv4si3 = 4423,
  CODE_FOR_ssse3_phadddv2si3 = 4424,
  CODE_FOR_ssse3_phsubdv2si3 = 4425,
  CODE_FOR_avx2_pmaddubsw256 = 4426,
  CODE_FOR_avx512bw_pmaddubsw512v8hi = 4427,
  CODE_FOR_avx512bw_pmaddubsw512v8hi_mask = 4428,
  CODE_FOR_avx512bw_pmaddubsw512v16hi = 4429,
  CODE_FOR_avx512bw_pmaddubsw512v16hi_mask = 4430,
  CODE_FOR_avx512bw_pmaddubsw512v32hi = 4431,
  CODE_FOR_avx512bw_pmaddubsw512v32hi_mask = 4432,
  CODE_FOR_avx512bw_umulhrswv32hi3 = 4433,
  CODE_FOR_avx512bw_umulhrswv32hi3_mask = 4434,
  CODE_FOR_ssse3_pmaddubsw128 = 4435,
  CODE_FOR_ssse3_pmaddubsw = 4436,
  CODE_FOR_avx512bw_pshufbv64qi3 = 4444,
  CODE_FOR_avx512bw_pshufbv64qi3_mask = 4445,
  CODE_FOR_avx2_pshufbv32qi3 = 4446,
  CODE_FOR_avx2_pshufbv32qi3_mask = 4447,
  CODE_FOR_ssse3_pshufbv16qi3 = 4448,
  CODE_FOR_ssse3_pshufbv16qi3_mask = 4449,
  CODE_FOR_avx2_psignv32qi3 = 4451,
  CODE_FOR_ssse3_psignv16qi3 = 4452,
  CODE_FOR_avx2_psignv16hi3 = 4453,
  CODE_FOR_ssse3_psignv8hi3 = 4454,
  CODE_FOR_avx2_psignv8si3 = 4455,
  CODE_FOR_ssse3_psignv4si3 = 4456,
  CODE_FOR_ssse3_psignv8qi3 = 4457,
  CODE_FOR_ssse3_psignv4hi3 = 4458,
  CODE_FOR_ssse3_psignv2si3 = 4459,
  CODE_FOR_avx512bw_palignrv64qi_mask = 4460,
  CODE_FOR_avx2_palignrv32qi_mask = 4461,
  CODE_FOR_ssse3_palignrv16qi_mask = 4462,
  CODE_FOR_avx512bw_palignrv4ti = 4463,
  CODE_FOR_avx2_palignrv2ti = 4464,
  CODE_FOR_ssse3_palignrti = 4465,
  CODE_FOR_ssse3_palignrdi = 4466,
  CODE_FOR_absv16si2_mask = 4479,
  CODE_FOR_absv8si2_mask = 4480,
  CODE_FOR_absv4si2_mask = 4481,
  CODE_FOR_absv8di2_mask = 4482,
  CODE_FOR_absv4di2_mask = 4483,
  CODE_FOR_absv2di2_mask = 4484,
  CODE_FOR_absv64qi2_mask = 4485,
  CODE_FOR_absv16qi2_mask = 4486,
  CODE_FOR_absv32qi2_mask = 4487,
  CODE_FOR_absv32hi2_mask = 4488,
  CODE_FOR_absv16hi2_mask = 4489,
  CODE_FOR_absv8hi2_mask = 4490,
  CODE_FOR_ssse3_absv8qi2 = 4491,
  CODE_FOR_ssse3_absv4hi2 = 4492,
  CODE_FOR_ssse3_absv2si2 = 4493,
  CODE_FOR_sse4a_movntsf = 4494,
  CODE_FOR_sse4a_movntdf = 4495,
  CODE_FOR_sse4a_vmmovntv4sf = 4496,
  CODE_FOR_sse4a_vmmovntv2df = 4497,
  CODE_FOR_sse4a_extrqi = 4498,
  CODE_FOR_sse4a_extrq = 4499,
  CODE_FOR_sse4a_insertqi = 4500,
  CODE_FOR_sse4a_insertq = 4501,
  CODE_FOR_avx_blendps256 = 4502,
  CODE_FOR_sse4_1_blendps = 4503,
  CODE_FOR_avx_blendpd256 = 4504,
  CODE_FOR_sse4_1_blendpd = 4505,
  CODE_FOR_avx_blendvps256 = 4506,
  CODE_FOR_sse4_1_blendvps = 4507,
  CODE_FOR_avx_blendvpd256 = 4508,
  CODE_FOR_sse4_1_blendvpd = 4509,
  CODE_FOR_sse4_1_blendvss = 4510,
  CODE_FOR_sse4_1_blendvsd = 4511,
  CODE_FOR_avx_dpps256 = 4520,
  CODE_FOR_sse4_1_dpps = 4521,
  CODE_FOR_avx_dppd256 = 4522,
  CODE_FOR_sse4_1_dppd = 4523,
  CODE_FOR_avx512f_movntdqa = 4524,
  CODE_FOR_avx2_movntdqa = 4525,
  CODE_FOR_sse4_1_movntdqa = 4526,
  CODE_FOR_avx2_mpsadbw = 4527,
  CODE_FOR_sse4_1_mpsadbw = 4528,
  CODE_FOR_avx512bw_packusdw = 4529,
  CODE_FOR_avx512bw_packusdw_mask = 4530,
  CODE_FOR_avx2_packusdw = 4531,
  CODE_FOR_avx2_packusdw_mask = 4532,
  CODE_FOR_sse4_1_packusdw = 4533,
  CODE_FOR_sse4_1_packusdw_mask = 4534,
  CODE_FOR_avx2_pblendvb = 4535,
  CODE_FOR_sse4_1_pblendvb = 4536,
  CODE_FOR_sse4_1_pblendw = 4539,
  CODE_FOR_avx2_pblenddv8si = 4541,
  CODE_FOR_avx2_pblenddv4si = 4542,
  CODE_FOR_sse4_1_phminposuw = 4543,
  CODE_FOR_avx2_sign_extendv16qiv16hi2 = 4544,
  CODE_FOR_avx2_sign_extendv16qiv16hi2_mask = 4545,
  CODE_FOR_avx2_zero_extendv16qiv16hi2 = 4546,
  CODE_FOR_avx2_zero_extendv16qiv16hi2_mask = 4547,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2 = 4549,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2_mask = 4550,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2 = 4551,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2_mask = 4552,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2 = 4554,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2_mask = 4555,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2 = 4556,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2_mask = 4557,
  CODE_FOR_avx512f_sign_extendv16qiv16si2_mask = 4568,
  CODE_FOR_avx512f_zero_extendv16qiv16si2_mask = 4570,
  CODE_FOR_avx2_sign_extendv8qiv8si2 = 4571,
  CODE_FOR_avx2_sign_extendv8qiv8si2_mask = 4572,
  CODE_FOR_avx2_zero_extendv8qiv8si2 = 4573,
  CODE_FOR_avx2_zero_extendv8qiv8si2_mask = 4574,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2 = 4583,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2_mask = 4584,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2 = 4585,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2_mask = 4586,
  CODE_FOR_avx512f_sign_extendv16hiv16si2 = 4595,
  CODE_FOR_avx512f_sign_extendv16hiv16si2_mask = 4596,
  CODE_FOR_avx512f_zero_extendv16hiv16si2 = 4597,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_mask = 4598,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_1 = 4599,
  CODE_FOR_avx2_sign_extendv8hiv8si2 = 4600,
  CODE_FOR_avx2_sign_extendv8hiv8si2_mask = 4601,
  CODE_FOR_avx2_zero_extendv8hiv8si2 = 4602,
  CODE_FOR_avx2_zero_extendv8hiv8si2_mask = 4603,
  CODE_FOR_avx2_zero_extendv8hiv8si2_1 = 4604,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2 = 4605,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2_mask = 4606,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2 = 4607,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2_mask = 4608,
  CODE_FOR_avx512f_sign_extendv8qiv8di2 = 4618,
  CODE_FOR_avx512f_sign_extendv8qiv8di2_mask = 4619,
  CODE_FOR_avx512f_zero_extendv8qiv8di2 = 4620,
  CODE_FOR_avx512f_zero_extendv8qiv8di2_mask = 4621,
  CODE_FOR_avx2_sign_extendv4qiv4di2 = 4630,
  CODE_FOR_avx2_sign_extendv4qiv4di2_mask = 4631,
  CODE_FOR_avx2_zero_extendv4qiv4di2 = 4632,
  CODE_FOR_avx2_zero_extendv4qiv4di2_mask = 4633,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2 = 4642,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2_mask = 4643,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2 = 4644,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2_mask = 4645,
  CODE_FOR_avx512f_sign_extendv8hiv8di2 = 4646,
  CODE_FOR_avx512f_sign_extendv8hiv8di2_mask = 4647,
  CODE_FOR_avx512f_zero_extendv8hiv8di2 = 4648,
  CODE_FOR_avx512f_zero_extendv8hiv8di2_mask = 4649,
  CODE_FOR_avx2_sign_extendv4hiv4di2 = 4650,
  CODE_FOR_avx2_sign_extendv4hiv4di2_mask = 4651,
  CODE_FOR_avx2_zero_extendv4hiv4di2 = 4652,
  CODE_FOR_avx2_zero_extendv4hiv4di2_mask = 4653,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2 = 4662,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2_mask = 4663,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2 = 4664,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2_mask = 4665,
  CODE_FOR_avx512f_sign_extendv8siv8di2 = 4674,
  CODE_FOR_avx512f_sign_extendv8siv8di2_mask = 4675,
  CODE_FOR_avx512f_zero_extendv8siv8di2 = 4676,
  CODE_FOR_avx512f_zero_extendv8siv8di2_mask = 4677,
  CODE_FOR_avx2_sign_extendv4siv4di2 = 4679,
  CODE_FOR_avx2_sign_extendv4siv4di2_mask = 4680,
  CODE_FOR_avx2_zero_extendv4siv4di2 = 4681,
  CODE_FOR_avx2_zero_extendv4siv4di2_mask = 4682,
  CODE_FOR_sse4_1_sign_extendv2siv2di2 = 4684,
  CODE_FOR_sse4_1_sign_extendv2siv2di2_mask = 4685,
  CODE_FOR_sse4_1_zero_extendv2siv2di2 = 4686,
  CODE_FOR_sse4_1_zero_extendv2siv2di2_mask = 4687,
  CODE_FOR_avx_vtestps256 = 4697,
  CODE_FOR_avx_vtestps = 4698,
  CODE_FOR_avx_vtestpd256 = 4699,
  CODE_FOR_avx_vtestpd = 4700,
  CODE_FOR_sse4_1_ptestv16qi = 4701,
  CODE_FOR_sse4_1_ptestv8hi = 4702,
  CODE_FOR_sse4_1_ptestv4si = 4703,
  CODE_FOR_sse4_1_ptestv2di = 4704,
  CODE_FOR_sse4_1_ptestv4sf = 4705,
  CODE_FOR_sse4_1_ptestv2df = 4706,
  CODE_FOR_avx_ptestv32qi = 4707,
  CODE_FOR_avx_ptestv16hi = 4708,
  CODE_FOR_avx_ptestv8si = 4709,
  CODE_FOR_avx_ptestv4di = 4710,
  CODE_FOR_avx_ptestv8sf = 4711,
  CODE_FOR_avx_ptestv4df = 4712,
  CODE_FOR_ptesttf2 = 4713,
  CODE_FOR_avx_roundps256 = 4714,
  CODE_FOR_sse4_1_roundps = 4715,
  CODE_FOR_avx_roundpd256 = 4716,
  CODE_FOR_sse4_1_roundpd = 4717,
  CODE_FOR_sse4_1_roundss = 4718,
  CODE_FOR_sse4_1_roundsd = 4719,
  CODE_FOR_sse4_2_pcmpestr = 4722,
  CODE_FOR_sse4_2_pcmpestri = 4723,
  CODE_FOR_sse4_2_pcmpestrm = 4724,
  CODE_FOR_sse4_2_pcmpestr_cconly = 4725,
  CODE_FOR_sse4_2_pcmpistr = 4726,
  CODE_FOR_sse4_2_pcmpistri = 4727,
  CODE_FOR_sse4_2_pcmpistrm = 4728,
  CODE_FOR_sse4_2_pcmpistr_cconly = 4729,
  CODE_FOR_avx512er_exp2v16sf = 4746,
  CODE_FOR_avx512er_exp2v16sf_round = 4747,
  CODE_FOR_avx512er_exp2v16sf_mask = 4748,
  CODE_FOR_avx512er_exp2v16sf_mask_round = 4749,
  CODE_FOR_avx512er_exp2v8df = 4750,
  CODE_FOR_avx512er_exp2v8df_round = 4751,
  CODE_FOR_avx512er_exp2v8df_mask = 4752,
  CODE_FOR_avx512er_exp2v8df_mask_round = 4753,
  CODE_FOR_avx512er_rcp28v16sf_mask = 4756,
  CODE_FOR_avx512er_rcp28v16sf_mask_round = 4757,
  CODE_FOR_avx512er_rcp28v8df_mask = 4760,
  CODE_FOR_avx512er_rcp28v8df_mask_round = 4761,
  CODE_FOR_avx512er_vmrcp28v4sf = 4762,
  CODE_FOR_avx512er_vmrcp28v4sf_round = 4763,
  CODE_FOR_avx512er_vmrcp28v4sf_mask = 4764,
  CODE_FOR_avx512er_vmrcp28v4sf_mask_round = 4765,
  CODE_FOR_avx512er_vmrcp28v2df = 4766,
  CODE_FOR_avx512er_vmrcp28v2df_round = 4767,
  CODE_FOR_avx512er_vmrcp28v2df_mask = 4768,
  CODE_FOR_avx512er_vmrcp28v2df_mask_round = 4769,
  CODE_FOR_avx512er_rsqrt28v16sf_mask = 4772,
  CODE_FOR_avx512er_rsqrt28v16sf_mask_round = 4773,
  CODE_FOR_avx512er_rsqrt28v8df_mask = 4776,
  CODE_FOR_avx512er_rsqrt28v8df_mask_round = 4777,
  CODE_FOR_avx512er_vmrsqrt28v4sf = 4778,
  CODE_FOR_avx512er_vmrsqrt28v4sf_round = 4779,
  CODE_FOR_avx512er_vmrsqrt28v4sf_mask = 4780,
  CODE_FOR_avx512er_vmrsqrt28v4sf_mask_round = 4781,
  CODE_FOR_avx512er_vmrsqrt28v2df = 4782,
  CODE_FOR_avx512er_vmrsqrt28v2df_round = 4783,
  CODE_FOR_avx512er_vmrsqrt28v2df_mask = 4784,
  CODE_FOR_avx512er_vmrsqrt28v2df_mask_round = 4785,
  CODE_FOR_xop_pmacsww = 4786,
  CODE_FOR_xop_pmacssww = 4787,
  CODE_FOR_xop_pmacsdd = 4788,
  CODE_FOR_xop_pmacssdd = 4789,
  CODE_FOR_xop_pmacsdql = 4790,
  CODE_FOR_xop_pmacssdql = 4791,
  CODE_FOR_xop_pmacsdqh = 4792,
  CODE_FOR_xop_pmacssdqh = 4793,
  CODE_FOR_xop_pmacswd = 4794,
  CODE_FOR_xop_pmacsswd = 4795,
  CODE_FOR_xop_pmadcswd = 4796,
  CODE_FOR_xop_pmadcsswd = 4797,
  CODE_FOR_xop_pcmov_v32qi256 = 4798,
  CODE_FOR_xop_pcmov_v16qi = 4799,
  CODE_FOR_xop_pcmov_v16hi256 = 4800,
  CODE_FOR_xop_pcmov_v8hi = 4801,
  CODE_FOR_xop_pcmov_v8si256 = 4802,
  CODE_FOR_xop_pcmov_v4si = 4803,
  CODE_FOR_xop_pcmov_v4di256 = 4804,
  CODE_FOR_xop_pcmov_v2di = 4805,
  CODE_FOR_xop_pcmov_v8sf256 = 4806,
  CODE_FOR_xop_pcmov_v4sf = 4807,
  CODE_FOR_xop_pcmov_v4df256 = 4808,
  CODE_FOR_xop_pcmov_v2df = 4809,
  CODE_FOR_xop_phaddbw = 4810,
  CODE_FOR_xop_phaddubw = 4811,
  CODE_FOR_xop_phaddbd = 4812,
  CODE_FOR_xop_phaddubd = 4813,
  CODE_FOR_xop_phaddbq = 4814,
  CODE_FOR_xop_phaddubq = 4815,
  CODE_FOR_xop_phaddwd = 4816,
  CODE_FOR_xop_phadduwd = 4817,
  CODE_FOR_xop_phaddwq = 4818,
  CODE_FOR_xop_phadduwq = 4819,
  CODE_FOR_xop_phadddq = 4820,
  CODE_FOR_xop_phaddudq = 4821,
  CODE_FOR_xop_phsubbw = 4822,
  CODE_FOR_xop_phsubwd = 4823,
  CODE_FOR_xop_phsubdq = 4824,
  CODE_FOR_xop_pperm = 4825,
  CODE_FOR_xop_pperm_pack_v2di_v4si = 4826,
  CODE_FOR_xop_pperm_pack_v4si_v8hi = 4827,
  CODE_FOR_xop_pperm_pack_v8hi_v16qi = 4828,
  CODE_FOR_xop_rotlv16qi3 = 4829,
  CODE_FOR_xop_rotlv8hi3 = 4830,
  CODE_FOR_xop_rotlv4si3 = 4831,
  CODE_FOR_xop_rotlv2di3 = 4832,
  CODE_FOR_xop_rotrv16qi3 = 4833,
  CODE_FOR_xop_rotrv8hi3 = 4834,
  CODE_FOR_xop_rotrv4si3 = 4835,
  CODE_FOR_xop_rotrv2di3 = 4836,
  CODE_FOR_xop_vrotlv16qi3 = 4837,
  CODE_FOR_xop_vrotlv8hi3 = 4838,
  CODE_FOR_xop_vrotlv4si3 = 4839,
  CODE_FOR_xop_vrotlv2di3 = 4840,
  CODE_FOR_xop_shav16qi3 = 4841,
  CODE_FOR_xop_shav8hi3 = 4842,
  CODE_FOR_xop_shav4si3 = 4843,
  CODE_FOR_xop_shav2di3 = 4844,
  CODE_FOR_xop_shlv16qi3 = 4845,
  CODE_FOR_xop_shlv8hi3 = 4846,
  CODE_FOR_xop_shlv4si3 = 4847,
  CODE_FOR_xop_shlv2di3 = 4848,
  CODE_FOR_xop_frczsf2 = 4849,
  CODE_FOR_xop_frczdf2 = 4850,
  CODE_FOR_xop_frczv4sf2 = 4851,
  CODE_FOR_xop_frczv2df2 = 4852,
  CODE_FOR_xop_frczv8sf2 = 4853,
  CODE_FOR_xop_frczv4df2 = 4854,
  CODE_FOR_xop_maskcmpv16qi3 = 4857,
  CODE_FOR_xop_maskcmpv8hi3 = 4858,
  CODE_FOR_xop_maskcmpv4si3 = 4859,
  CODE_FOR_xop_maskcmpv2di3 = 4860,
  CODE_FOR_xop_maskcmp_unsv16qi3 = 4861,
  CODE_FOR_xop_maskcmp_unsv8hi3 = 4862,
  CODE_FOR_xop_maskcmp_unsv4si3 = 4863,
  CODE_FOR_xop_maskcmp_unsv2di3 = 4864,
  CODE_FOR_xop_maskcmp_uns2v16qi3 = 4865,
  CODE_FOR_xop_maskcmp_uns2v8hi3 = 4866,
  CODE_FOR_xop_maskcmp_uns2v4si3 = 4867,
  CODE_FOR_xop_maskcmp_uns2v2di3 = 4868,
  CODE_FOR_xop_pcom_tfv16qi3 = 4869,
  CODE_FOR_xop_pcom_tfv8hi3 = 4870,
  CODE_FOR_xop_pcom_tfv4si3 = 4871,
  CODE_FOR_xop_pcom_tfv2di3 = 4872,
  CODE_FOR_xop_vpermil2v8sf3 = 4873,
  CODE_FOR_xop_vpermil2v4sf3 = 4874,
  CODE_FOR_xop_vpermil2v4df3 = 4875,
  CODE_FOR_xop_vpermil2v2df3 = 4876,
  CODE_FOR_aesenc = 4877,
  CODE_FOR_aesenclast = 4878,
  CODE_FOR_aesdec = 4879,
  CODE_FOR_aesdeclast = 4880,
  CODE_FOR_aesimc = 4881,
  CODE_FOR_aeskeygenassist = 4882,
  CODE_FOR_pclmulqdq = 4883,
  CODE_FOR_avx2_pbroadcastv16si = 4887,
  CODE_FOR_avx2_pbroadcastv8di = 4888,
  CODE_FOR_avx2_pbroadcastv64qi = 4889,
  CODE_FOR_avx2_pbroadcastv32qi = 4890,
  CODE_FOR_avx2_pbroadcastv16qi = 4891,
  CODE_FOR_avx2_pbroadcastv32hi = 4892,
  CODE_FOR_avx2_pbroadcastv16hi = 4893,
  CODE_FOR_avx2_pbroadcastv8hi = 4894,
  CODE_FOR_avx2_pbroadcastv8si = 4895,
  CODE_FOR_avx2_pbroadcastv4si = 4896,
  CODE_FOR_avx2_pbroadcastv4di = 4897,
  CODE_FOR_avx2_pbroadcastv2di = 4898,
  CODE_FOR_avx2_pbroadcastv32qi_1 = 4899,
  CODE_FOR_avx2_pbroadcastv16hi_1 = 4900,
  CODE_FOR_avx2_pbroadcastv8si_1 = 4901,
  CODE_FOR_avx2_pbroadcastv4di_1 = 4902,
  CODE_FOR_avx2_permvarv8si = 4903,
  CODE_FOR_avx2_permvarv8si_mask = 4904,
  CODE_FOR_avx2_permvarv8sf = 4905,
  CODE_FOR_avx2_permvarv8sf_mask = 4906,
  CODE_FOR_avx512f_permvarv16si = 4907,
  CODE_FOR_avx512f_permvarv16si_mask = 4908,
  CODE_FOR_avx512f_permvarv16sf = 4909,
  CODE_FOR_avx512f_permvarv16sf_mask = 4910,
  CODE_FOR_avx512f_permvarv8di = 4911,
  CODE_FOR_avx512f_permvarv8di_mask = 4912,
  CODE_FOR_avx512f_permvarv8df = 4913,
  CODE_FOR_avx512f_permvarv8df_mask = 4914,
  CODE_FOR_avx2_permvarv4di = 4915,
  CODE_FOR_avx2_permvarv4di_mask = 4916,
  CODE_FOR_avx2_permvarv4df = 4917,
  CODE_FOR_avx2_permvarv4df_mask = 4918,
  CODE_FOR_avx512bw_permvarv64qi = 4919,
  CODE_FOR_avx512bw_permvarv64qi_mask = 4920,
  CODE_FOR_avx512vl_permvarv16qi = 4921,
  CODE_FOR_avx512vl_permvarv16qi_mask = 4922,
  CODE_FOR_avx512vl_permvarv32qi = 4923,
  CODE_FOR_avx512vl_permvarv32qi_mask = 4924,
  CODE_FOR_avx512vl_permvarv8hi = 4925,
  CODE_FOR_avx512vl_permvarv8hi_mask = 4926,
  CODE_FOR_avx512vl_permvarv16hi = 4927,
  CODE_FOR_avx512vl_permvarv16hi_mask = 4928,
  CODE_FOR_avx512bw_permvarv32hi = 4929,
  CODE_FOR_avx512bw_permvarv32hi_mask = 4930,
  CODE_FOR_avx2_permv4di_1 = 4946,
  CODE_FOR_avx2_permv4di_1_mask = 4947,
  CODE_FOR_avx2_permv4df_1 = 4948,
  CODE_FOR_avx2_permv4df_1_mask = 4949,
  CODE_FOR_avx512f_permv8df_1 = 4950,
  CODE_FOR_avx512f_permv8df_1_mask = 4951,
  CODE_FOR_avx512f_permv8di_1 = 4952,
  CODE_FOR_avx512f_permv8di_1_mask = 4953,
  CODE_FOR_avx2_permv2ti = 4954,
  CODE_FOR_avx2_vec_dupv4df = 4955,
  CODE_FOR_avx512f_vec_dupv16si_1 = 4956,
  CODE_FOR_avx512f_vec_dupv8di_1 = 4957,
  CODE_FOR_avx512bw_vec_dupv32hi_1 = 4958,
  CODE_FOR_avx512bw_vec_dupv64qi_1 = 4959,
  CODE_FOR_avx512f_vec_dupv16si = 4960,
  CODE_FOR_avx512f_vec_dupv16si_mask = 4961,
  CODE_FOR_avx512vl_vec_dupv8si = 4962,
  CODE_FOR_avx512vl_vec_dupv8si_mask = 4963,
  CODE_FOR_avx512vl_vec_dupv4si = 4964,
  CODE_FOR_avx512vl_vec_dupv4si_mask = 4965,
  CODE_FOR_avx512f_vec_dupv8di = 4966,
  CODE_FOR_avx512f_vec_dupv8di_mask = 4967,
  CODE_FOR_avx512vl_vec_dupv4di = 4968,
  CODE_FOR_avx512vl_vec_dupv4di_mask = 4969,
  CODE_FOR_avx512vl_vec_dupv2di = 4970,
  CODE_FOR_avx512vl_vec_dupv2di_mask = 4971,
  CODE_FOR_avx512f_vec_dupv16sf = 4972,
  CODE_FOR_avx512f_vec_dupv16sf_mask = 4973,
  CODE_FOR_avx512vl_vec_dupv8sf = 4974,
  CODE_FOR_avx512vl_vec_dupv8sf_mask = 4975,
  CODE_FOR_avx512vl_vec_dupv4sf = 4976,
  CODE_FOR_avx512vl_vec_dupv4sf_mask = 4977,
  CODE_FOR_avx512f_vec_dupv8df = 4978,
  CODE_FOR_avx512f_vec_dupv8df_mask = 4979,
  CODE_FOR_avx512vl_vec_dupv4df = 4980,
  CODE_FOR_avx512vl_vec_dupv4df_mask = 4981,
  CODE_FOR_avx512vl_vec_dupv2df = 4982,
  CODE_FOR_avx512vl_vec_dupv2df_mask = 4983,
  CODE_FOR_avx512bw_vec_dupv64qi = 4984,
  CODE_FOR_avx512bw_vec_dupv64qi_mask = 4985,
  CODE_FOR_avx512vl_vec_dupv16qi = 4986,
  CODE_FOR_avx512vl_vec_dupv16qi_mask = 4987,
  CODE_FOR_avx512vl_vec_dupv32qi = 4988,
  CODE_FOR_avx512vl_vec_dupv32qi_mask = 4989,
  CODE_FOR_avx512bw_vec_dupv32hi = 4990,
  CODE_FOR_avx512bw_vec_dupv32hi_mask = 4991,
  CODE_FOR_avx512vl_vec_dupv16hi = 4992,
  CODE_FOR_avx512vl_vec_dupv16hi_mask = 4993,
  CODE_FOR_avx512vl_vec_dupv8hi = 4994,
  CODE_FOR_avx512vl_vec_dupv8hi_mask = 4995,
  CODE_FOR_avx512f_broadcastv16sf_mask = 4997,
  CODE_FOR_avx512f_broadcastv16si_mask = 4999,
  CODE_FOR_avx512f_broadcastv8df_mask = 5001,
  CODE_FOR_avx512f_broadcastv8di_mask = 5003,
  CODE_FOR_avx512bw_vec_dup_gprv64qi_mask = 5005,
  CODE_FOR_avx512vl_vec_dup_gprv16qi_mask = 5007,
  CODE_FOR_avx512vl_vec_dup_gprv32qi_mask = 5009,
  CODE_FOR_avx512bw_vec_dup_gprv32hi_mask = 5011,
  CODE_FOR_avx512vl_vec_dup_gprv16hi_mask = 5013,
  CODE_FOR_avx512vl_vec_dup_gprv8hi_mask = 5015,
  CODE_FOR_avx512f_vec_dup_gprv16si_mask = 5017,
  CODE_FOR_avx512vl_vec_dup_gprv8si_mask = 5019,
  CODE_FOR_avx512vl_vec_dup_gprv4si_mask = 5021,
  CODE_FOR_avx512f_vec_dup_gprv8di_mask = 5023,
  CODE_FOR_avx512vl_vec_dup_gprv4di_mask = 5025,
  CODE_FOR_avx512vl_vec_dup_gprv2di_mask = 5027,
  CODE_FOR_avx512f_vec_dup_gprv16sf_mask = 5029,
  CODE_FOR_avx512vl_vec_dup_gprv8sf_mask = 5031,
  CODE_FOR_avx512vl_vec_dup_gprv4sf_mask = 5033,
  CODE_FOR_avx512f_vec_dup_gprv8df_mask = 5035,
  CODE_FOR_avx512vl_vec_dup_gprv4df_mask = 5037,
  CODE_FOR_avx512vl_vec_dup_gprv2df_mask = 5039,
  CODE_FOR_vec_dupv4sf = 5040,
  CODE_FOR_avx2_vbroadcasti128_v32qi = 5043,
  CODE_FOR_avx2_vbroadcasti128_v16hi = 5044,
  CODE_FOR_avx2_vbroadcasti128_v8si = 5045,
  CODE_FOR_avx2_vbroadcasti128_v4di = 5046,
  CODE_FOR_vec_dupv8si = 5053,
  CODE_FOR_vec_dupv8sf = 5054,
  CODE_FOR_vec_dupv4di = 5055,
  CODE_FOR_vec_dupv4df = 5056,
  CODE_FOR_avx_vbroadcastf128_v32qi = 5057,
  CODE_FOR_avx_vbroadcastf128_v16hi = 5058,
  CODE_FOR_avx_vbroadcastf128_v8si = 5059,
  CODE_FOR_avx_vbroadcastf128_v4di = 5060,
  CODE_FOR_avx_vbroadcastf128_v8sf = 5061,
  CODE_FOR_avx_vbroadcastf128_v4df = 5062,
  CODE_FOR_avx512dq_broadcastv16si_mask = 5064,
  CODE_FOR_avx512dq_broadcastv8si_mask = 5066,
  CODE_FOR_avx512dq_broadcastv4si_mask = 5068,
  CODE_FOR_avx512dq_broadcastv16sf_mask = 5070,
  CODE_FOR_avx512dq_broadcastv8sf_mask = 5072,
  CODE_FOR_avx512vl_broadcastv8si_mask_1 = 5074,
  CODE_FOR_avx512vl_broadcastv8sf_mask_1 = 5076,
  CODE_FOR_avx512dq_broadcastv16sf_mask_1 = 5078,
  CODE_FOR_avx512dq_broadcastv16si_mask_1 = 5080,
  CODE_FOR_avx512dq_broadcastv8di_mask_1 = 5082,
  CODE_FOR_avx512dq_broadcastv8df_mask_1 = 5084,
  CODE_FOR_avx512dq_broadcastv4di_mask_1 = 5086,
  CODE_FOR_avx512dq_broadcastv4df_mask_1 = 5088,
  CODE_FOR_avx512cd_maskb_vec_dupv8di = 5089,
  CODE_FOR_avx512cd_maskb_vec_dupv4di = 5090,
  CODE_FOR_avx512cd_maskb_vec_dupv2di = 5091,
  CODE_FOR_avx512cd_maskw_vec_dupv16si = 5092,
  CODE_FOR_avx512cd_maskw_vec_dupv8si = 5093,
  CODE_FOR_avx512cd_maskw_vec_dupv4si = 5094,
  CODE_FOR_avx512f_vpermilvarv16sf3 = 5095,
  CODE_FOR_avx512f_vpermilvarv16sf3_mask = 5096,
  CODE_FOR_avx_vpermilvarv8sf3 = 5097,
  CODE_FOR_avx_vpermilvarv8sf3_mask = 5098,
  CODE_FOR_avx_vpermilvarv4sf3 = 5099,
  CODE_FOR_avx_vpermilvarv4sf3_mask = 5100,
  CODE_FOR_avx512f_vpermilvarv8df3 = 5101,
  CODE_FOR_avx512f_vpermilvarv8df3_mask = 5102,
  CODE_FOR_avx_vpermilvarv4df3 = 5103,
  CODE_FOR_avx_vpermilvarv4df3_mask = 5104,
  CODE_FOR_avx_vpermilvarv2df3 = 5105,
  CODE_FOR_avx_vpermilvarv2df3_mask = 5106,
  CODE_FOR_avx512f_vpermt2varv16si3 = 5125,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz_1 = 5126,
  CODE_FOR_avx512f_vpermt2varv16sf3 = 5127,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz_1 = 5128,
  CODE_FOR_avx512f_vpermt2varv8di3 = 5129,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz_1 = 5130,
  CODE_FOR_avx512f_vpermt2varv8df3 = 5131,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz_1 = 5132,
  CODE_FOR_avx512vl_vpermt2varv8si3 = 5133,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz_1 = 5134,
  CODE_FOR_avx512vl_vpermt2varv8sf3 = 5135,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz_1 = 5136,
  CODE_FOR_avx512vl_vpermt2varv4di3 = 5137,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz_1 = 5138,
  CODE_FOR_avx512vl_vpermt2varv4df3 = 5139,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz_1 = 5140,
  CODE_FOR_avx512vl_vpermt2varv4si3 = 5141,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz_1 = 5142,
  CODE_FOR_avx512vl_vpermt2varv4sf3 = 5143,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz_1 = 5144,
  CODE_FOR_avx512vl_vpermt2varv2di3 = 5145,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz_1 = 5146,
  CODE_FOR_avx512vl_vpermt2varv2df3 = 5147,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz_1 = 5148,
  CODE_FOR_avx512bw_vpermt2varv32hi3 = 5149,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz_1 = 5150,
  CODE_FOR_avx512vl_vpermt2varv16hi3 = 5151,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz_1 = 5152,
  CODE_FOR_avx512vl_vpermt2varv8hi3 = 5153,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz_1 = 5154,
  CODE_FOR_avx512bw_vpermt2varv64qi3 = 5155,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz_1 = 5156,
  CODE_FOR_avx512vl_vpermt2varv32qi3 = 5157,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz_1 = 5158,
  CODE_FOR_avx512vl_vpermt2varv16qi3 = 5159,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz_1 = 5160,
  CODE_FOR_avx512f_vpermt2varv16si3_mask = 5161,
  CODE_FOR_avx512f_vpermt2varv16sf3_mask = 5162,
  CODE_FOR_avx512f_vpermt2varv8di3_mask = 5163,
  CODE_FOR_avx512f_vpermt2varv8df3_mask = 5164,
  CODE_FOR_avx512vl_vpermt2varv8si3_mask = 5165,
  CODE_FOR_avx512vl_vpermt2varv8sf3_mask = 5166,
  CODE_FOR_avx512vl_vpermt2varv4di3_mask = 5167,
  CODE_FOR_avx512vl_vpermt2varv4df3_mask = 5168,
  CODE_FOR_avx512vl_vpermt2varv4si3_mask = 5169,
  CODE_FOR_avx512vl_vpermt2varv4sf3_mask = 5170,
  CODE_FOR_avx512vl_vpermt2varv2di3_mask = 5171,
  CODE_FOR_avx512vl_vpermt2varv2df3_mask = 5172,
  CODE_FOR_avx512bw_vpermt2varv32hi3_mask = 5173,
  CODE_FOR_avx512vl_vpermt2varv16hi3_mask = 5174,
  CODE_FOR_avx512vl_vpermt2varv8hi3_mask = 5175,
  CODE_FOR_avx512bw_vpermt2varv64qi3_mask = 5176,
  CODE_FOR_avx512vl_vpermt2varv32qi3_mask = 5177,
  CODE_FOR_avx512vl_vpermt2varv16qi3_mask = 5178,
  CODE_FOR_vec_set_lo_v4di = 5191,
  CODE_FOR_vec_set_lo_v4di_mask = 5192,
  CODE_FOR_vec_set_lo_v4df = 5193,
  CODE_FOR_vec_set_lo_v4df_mask = 5194,
  CODE_FOR_vec_set_hi_v4di = 5195,
  CODE_FOR_vec_set_hi_v4di_mask = 5196,
  CODE_FOR_vec_set_hi_v4df = 5197,
  CODE_FOR_vec_set_hi_v4df_mask = 5198,
  CODE_FOR_vec_set_lo_v8si = 5199,
  CODE_FOR_vec_set_lo_v8si_mask = 5200,
  CODE_FOR_vec_set_lo_v8sf = 5201,
  CODE_FOR_vec_set_lo_v8sf_mask = 5202,
  CODE_FOR_vec_set_hi_v8si = 5203,
  CODE_FOR_vec_set_hi_v8si_mask = 5204,
  CODE_FOR_vec_set_hi_v8sf = 5205,
  CODE_FOR_vec_set_hi_v8sf_mask = 5206,
  CODE_FOR_vec_set_lo_v16hi = 5207,
  CODE_FOR_vec_set_hi_v16hi = 5208,
  CODE_FOR_vec_set_lo_v32qi = 5209,
  CODE_FOR_vec_set_hi_v32qi = 5210,
  CODE_FOR_avx_maskloadps = 5211,
  CODE_FOR_avx_maskloadpd = 5212,
  CODE_FOR_avx_maskloadps256 = 5213,
  CODE_FOR_avx_maskloadpd256 = 5214,
  CODE_FOR_avx2_maskloadd = 5215,
  CODE_FOR_avx2_maskloadq = 5216,
  CODE_FOR_avx2_maskloadd256 = 5217,
  CODE_FOR_avx2_maskloadq256 = 5218,
  CODE_FOR_avx_maskstoreps = 5219,
  CODE_FOR_avx_maskstorepd = 5220,
  CODE_FOR_avx_maskstoreps256 = 5221,
  CODE_FOR_avx_maskstorepd256 = 5222,
  CODE_FOR_avx2_maskstored = 5223,
  CODE_FOR_avx2_maskstoreq = 5224,
  CODE_FOR_avx2_maskstored256 = 5225,
  CODE_FOR_avx2_maskstoreq256 = 5226,
  CODE_FOR_avx_si256_si = 5227,
  CODE_FOR_avx_ps256_ps = 5228,
  CODE_FOR_avx_pd256_pd = 5229,
  CODE_FOR_avx2_ashrvv4si = 5230,
  CODE_FOR_avx2_ashrvv4si_mask = 5231,
  CODE_FOR_avx2_ashrvv8si = 5232,
  CODE_FOR_avx2_ashrvv8si_mask = 5233,
  CODE_FOR_avx512f_ashrvv16si = 5234,
  CODE_FOR_avx512f_ashrvv16si_mask = 5235,
  CODE_FOR_avx2_ashrvv2di = 5236,
  CODE_FOR_avx2_ashrvv2di_mask = 5237,
  CODE_FOR_avx2_ashrvv4di = 5238,
  CODE_FOR_avx2_ashrvv4di_mask = 5239,
  CODE_FOR_avx512f_ashrvv8di = 5240,
  CODE_FOR_avx512f_ashrvv8di_mask = 5241,
  CODE_FOR_avx512vl_ashrvv8hi = 5242,
  CODE_FOR_avx512vl_ashrvv8hi_mask = 5243,
  CODE_FOR_avx512vl_ashrvv16hi = 5244,
  CODE_FOR_avx512vl_ashrvv16hi_mask = 5245,
  CODE_FOR_avx512bw_ashrvv32hi = 5246,
  CODE_FOR_avx512bw_ashrvv32hi_mask = 5247,
  CODE_FOR_avx512f_ashlvv16si = 5248,
  CODE_FOR_avx512f_ashlvv16si_mask = 5249,
  CODE_FOR_avx512f_lshrvv16si = 5250,
  CODE_FOR_avx512f_lshrvv16si_mask = 5251,
  CODE_FOR_avx2_ashlvv8si = 5252,
  CODE_FOR_avx2_ashlvv8si_mask = 5253,
  CODE_FOR_avx2_lshrvv8si = 5254,
  CODE_FOR_avx2_lshrvv8si_mask = 5255,
  CODE_FOR_avx2_ashlvv4si = 5256,
  CODE_FOR_avx2_ashlvv4si_mask = 5257,
  CODE_FOR_avx2_lshrvv4si = 5258,
  CODE_FOR_avx2_lshrvv4si_mask = 5259,
  CODE_FOR_avx512f_ashlvv8di = 5260,
  CODE_FOR_avx512f_ashlvv8di_mask = 5261,
  CODE_FOR_avx512f_lshrvv8di = 5262,
  CODE_FOR_avx512f_lshrvv8di_mask = 5263,
  CODE_FOR_avx2_ashlvv4di = 5264,
  CODE_FOR_avx2_ashlvv4di_mask = 5265,
  CODE_FOR_avx2_lshrvv4di = 5266,
  CODE_FOR_avx2_lshrvv4di_mask = 5267,
  CODE_FOR_avx2_ashlvv2di = 5268,
  CODE_FOR_avx2_ashlvv2di_mask = 5269,
  CODE_FOR_avx2_lshrvv2di = 5270,
  CODE_FOR_avx2_lshrvv2di_mask = 5271,
  CODE_FOR_avx512vl_ashlvv8hi = 5272,
  CODE_FOR_avx512vl_ashlvv8hi_mask = 5273,
  CODE_FOR_avx512vl_lshrvv8hi = 5274,
  CODE_FOR_avx512vl_lshrvv8hi_mask = 5275,
  CODE_FOR_avx512vl_ashlvv16hi = 5276,
  CODE_FOR_avx512vl_ashlvv16hi_mask = 5277,
  CODE_FOR_avx512vl_lshrvv16hi = 5278,
  CODE_FOR_avx512vl_lshrvv16hi_mask = 5279,
  CODE_FOR_avx512bw_ashlvv32hi = 5280,
  CODE_FOR_avx512bw_ashlvv32hi_mask = 5281,
  CODE_FOR_avx512bw_lshrvv32hi = 5282,
  CODE_FOR_avx512bw_lshrvv32hi_mask = 5283,
  CODE_FOR_avx_vec_concatv32qi = 5284,
  CODE_FOR_avx_vec_concatv16hi = 5285,
  CODE_FOR_avx_vec_concatv8si = 5286,
  CODE_FOR_avx_vec_concatv4di = 5287,
  CODE_FOR_avx_vec_concatv8sf = 5288,
  CODE_FOR_avx_vec_concatv4df = 5289,
  CODE_FOR_avx_vec_concatv64qi = 5290,
  CODE_FOR_avx_vec_concatv32hi = 5291,
  CODE_FOR_avx_vec_concatv16si = 5292,
  CODE_FOR_avx_vec_concatv8di = 5293,
  CODE_FOR_avx_vec_concatv16sf = 5294,
  CODE_FOR_avx_vec_concatv8df = 5295,
  CODE_FOR_vcvtph2ps = 5296,
  CODE_FOR_vcvtph2ps_mask = 5297,
  CODE_FOR_vcvtph2ps256 = 5300,
  CODE_FOR_vcvtph2ps256_mask = 5301,
  CODE_FOR_avx512f_vcvtph2ps512_mask = 5304,
  CODE_FOR_avx512f_vcvtph2ps512_mask_round = 5305,
  CODE_FOR_vcvtps2ph256 = 5309,
  CODE_FOR_vcvtps2ph256_mask = 5310,
  CODE_FOR_avx512f_vcvtps2ph512_mask = 5314,
  CODE_FOR_avx512f_compressv16si_mask = 5533,
  CODE_FOR_avx512f_compressv16sf_mask = 5534,
  CODE_FOR_avx512f_compressv8di_mask = 5535,
  CODE_FOR_avx512f_compressv8df_mask = 5536,
  CODE_FOR_avx512vl_compressv8si_mask = 5537,
  CODE_FOR_avx512vl_compressv8sf_mask = 5538,
  CODE_FOR_avx512vl_compressv4di_mask = 5539,
  CODE_FOR_avx512vl_compressv4df_mask = 5540,
  CODE_FOR_avx512vl_compressv4si_mask = 5541,
  CODE_FOR_avx512vl_compressv4sf_mask = 5542,
  CODE_FOR_avx512vl_compressv2di_mask = 5543,
  CODE_FOR_avx512vl_compressv2df_mask = 5544,
  CODE_FOR_compressv64qi_mask = 5545,
  CODE_FOR_compressv16qi_mask = 5546,
  CODE_FOR_compressv32qi_mask = 5547,
  CODE_FOR_compressv32hi_mask = 5548,
  CODE_FOR_compressv16hi_mask = 5549,
  CODE_FOR_compressv8hi_mask = 5550,
  CODE_FOR_avx512f_compressstorev16si_mask = 5551,
  CODE_FOR_avx512f_compressstorev16sf_mask = 5552,
  CODE_FOR_avx512f_compressstorev8di_mask = 5553,
  CODE_FOR_avx512f_compressstorev8df_mask = 5554,
  CODE_FOR_avx512vl_compressstorev8si_mask = 5555,
  CODE_FOR_avx512vl_compressstorev8sf_mask = 5556,
  CODE_FOR_avx512vl_compressstorev4di_mask = 5557,
  CODE_FOR_avx512vl_compressstorev4df_mask = 5558,
  CODE_FOR_avx512vl_compressstorev4si_mask = 5559,
  CODE_FOR_avx512vl_compressstorev4sf_mask = 5560,
  CODE_FOR_avx512vl_compressstorev2di_mask = 5561,
  CODE_FOR_avx512vl_compressstorev2df_mask = 5562,
  CODE_FOR_compressstorev64qi_mask = 5563,
  CODE_FOR_compressstorev16qi_mask = 5564,
  CODE_FOR_compressstorev32qi_mask = 5565,
  CODE_FOR_compressstorev32hi_mask = 5566,
  CODE_FOR_compressstorev16hi_mask = 5567,
  CODE_FOR_compressstorev8hi_mask = 5568,
  CODE_FOR_avx512f_expandv16si_mask = 5569,
  CODE_FOR_avx512f_expandv16sf_mask = 5570,
  CODE_FOR_avx512f_expandv8di_mask = 5571,
  CODE_FOR_avx512f_expandv8df_mask = 5572,
  CODE_FOR_avx512vl_expandv8si_mask = 5573,
  CODE_FOR_avx512vl_expandv8sf_mask = 5574,
  CODE_FOR_avx512vl_expandv4di_mask = 5575,
  CODE_FOR_avx512vl_expandv4df_mask = 5576,
  CODE_FOR_avx512vl_expandv4si_mask = 5577,
  CODE_FOR_avx512vl_expandv4sf_mask = 5578,
  CODE_FOR_avx512vl_expandv2di_mask = 5579,
  CODE_FOR_avx512vl_expandv2df_mask = 5580,
  CODE_FOR_expandv64qi_mask = 5581,
  CODE_FOR_expandv16qi_mask = 5582,
  CODE_FOR_expandv32qi_mask = 5583,
  CODE_FOR_expandv32hi_mask = 5584,
  CODE_FOR_expandv16hi_mask = 5585,
  CODE_FOR_expandv8hi_mask = 5586,
  CODE_FOR_avx512dq_rangepv16sf = 5587,
  CODE_FOR_avx512dq_rangepv16sf_round = 5588,
  CODE_FOR_avx512dq_rangepv16sf_mask = 5589,
  CODE_FOR_avx512dq_rangepv16sf_mask_round = 5590,
  CODE_FOR_avx512dq_rangepv8sf = 5591,
   CODE_FOR_avx512dq_rangepv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8sf_mask = 5592,
   CODE_FOR_avx512dq_rangepv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf = 5593,
   CODE_FOR_avx512dq_rangepv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf_mask = 5594,
   CODE_FOR_avx512dq_rangepv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8df = 5595,
  CODE_FOR_avx512dq_rangepv8df_round = 5596,
  CODE_FOR_avx512dq_rangepv8df_mask = 5597,
  CODE_FOR_avx512dq_rangepv8df_mask_round = 5598,
  CODE_FOR_avx512dq_rangepv4df = 5599,
   CODE_FOR_avx512dq_rangepv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4df_mask = 5600,
   CODE_FOR_avx512dq_rangepv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df = 5601,
   CODE_FOR_avx512dq_rangepv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df_mask = 5602,
   CODE_FOR_avx512dq_rangepv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangesv4sf = 5603,
  CODE_FOR_avx512dq_rangesv4sf_mask = 5604,
  CODE_FOR_avx512dq_rangesv4sf_round = 5605,
  CODE_FOR_avx512dq_rangesv4sf_mask_round = 5606,
  CODE_FOR_avx512dq_rangesv2df = 5607,
  CODE_FOR_avx512dq_rangesv2df_mask = 5608,
  CODE_FOR_avx512dq_rangesv2df_round = 5609,
  CODE_FOR_avx512dq_rangesv2df_mask_round = 5610,
  CODE_FOR_avx512dq_fpclassv16sf = 5611,
  CODE_FOR_avx512dq_fpclassv16sf_mask = 5612,
  CODE_FOR_avx512dq_fpclassv8sf = 5613,
  CODE_FOR_avx512dq_fpclassv8sf_mask = 5614,
  CODE_FOR_avx512dq_fpclassv4sf = 5615,
  CODE_FOR_avx512dq_fpclassv4sf_mask = 5616,
  CODE_FOR_avx512dq_fpclassv8df = 5617,
  CODE_FOR_avx512dq_fpclassv8df_mask = 5618,
  CODE_FOR_avx512dq_fpclassv4df = 5619,
  CODE_FOR_avx512dq_fpclassv4df_mask = 5620,
  CODE_FOR_avx512dq_fpclassv2df = 5621,
  CODE_FOR_avx512dq_fpclassv2df_mask = 5622,
  CODE_FOR_avx512dq_vmfpclassv4sf = 5623,
  CODE_FOR_avx512dq_vmfpclassv4sf_mask = 5624,
  CODE_FOR_avx512dq_vmfpclassv2df = 5625,
  CODE_FOR_avx512dq_vmfpclassv2df_mask = 5626,
  CODE_FOR_avx512f_getmantv16sf = 5627,
  CODE_FOR_avx512f_getmantv16sf_round = 5628,
  CODE_FOR_avx512f_getmantv16sf_mask = 5629,
  CODE_FOR_avx512f_getmantv16sf_mask_round = 5630,
  CODE_FOR_avx512vl_getmantv8sf = 5631,
  CODE_FOR_avx512vl_getmantv8sf_round = 5632,
  CODE_FOR_avx512vl_getmantv8sf_mask = 5633,
  CODE_FOR_avx512vl_getmantv8sf_mask_round = 5634,
  CODE_FOR_avx512vl_getmantv4sf = 5635,
  CODE_FOR_avx512vl_getmantv4sf_round = 5636,
  CODE_FOR_avx512vl_getmantv4sf_mask = 5637,
  CODE_FOR_avx512vl_getmantv4sf_mask_round = 5638,
  CODE_FOR_avx512f_getmantv8df = 5639,
  CODE_FOR_avx512f_getmantv8df_round = 5640,
  CODE_FOR_avx512f_getmantv8df_mask = 5641,
  CODE_FOR_avx512f_getmantv8df_mask_round = 5642,
  CODE_FOR_avx512vl_getmantv4df = 5643,
  CODE_FOR_avx512vl_getmantv4df_round = 5644,
  CODE_FOR_avx512vl_getmantv4df_mask = 5645,
  CODE_FOR_avx512vl_getmantv4df_mask_round = 5646,
  CODE_FOR_avx512vl_getmantv2df = 5647,
  CODE_FOR_avx512vl_getmantv2df_round = 5648,
  CODE_FOR_avx512vl_getmantv2df_mask = 5649,
  CODE_FOR_avx512vl_getmantv2df_mask_round = 5650,
  CODE_FOR_avx512f_vgetmantv4sf = 5651,
  CODE_FOR_avx512f_vgetmantv4sf_mask = 5652,
  CODE_FOR_avx512f_vgetmantv4sf_round = 5653,
  CODE_FOR_avx512f_vgetmantv4sf_mask_round = 5654,
  CODE_FOR_avx512f_vgetmantv2df = 5655,
  CODE_FOR_avx512f_vgetmantv2df_mask = 5656,
  CODE_FOR_avx512f_vgetmantv2df_round = 5657,
  CODE_FOR_avx512f_vgetmantv2df_mask_round = 5658,
  CODE_FOR_avx512bw_dbpsadbwv8hi_mask = 5660,
  CODE_FOR_avx512bw_dbpsadbwv16hi_mask = 5662,
  CODE_FOR_avx512bw_dbpsadbwv32hi_mask = 5664,
  CODE_FOR_clzv16si2 = 5665,
  CODE_FOR_clzv16si2_mask = 5666,
  CODE_FOR_clzv8si2 = 5667,
  CODE_FOR_clzv8si2_mask = 5668,
  CODE_FOR_clzv4si2 = 5669,
  CODE_FOR_clzv4si2_mask = 5670,
  CODE_FOR_clzv8di2 = 5671,
  CODE_FOR_clzv8di2_mask = 5672,
  CODE_FOR_clzv4di2 = 5673,
  CODE_FOR_clzv4di2_mask = 5674,
  CODE_FOR_clzv2di2 = 5675,
  CODE_FOR_clzv2di2_mask = 5676,
  CODE_FOR_conflictv16si_mask = 5678,
  CODE_FOR_conflictv8si_mask = 5680,
  CODE_FOR_conflictv4si_mask = 5682,
  CODE_FOR_conflictv8di_mask = 5684,
  CODE_FOR_conflictv4di_mask = 5686,
  CODE_FOR_conflictv2di_mask = 5688,
  CODE_FOR_sha1msg1 = 5689,
  CODE_FOR_sha1msg2 = 5690,
  CODE_FOR_sha1nexte = 5691,
  CODE_FOR_sha1rnds4 = 5692,
  CODE_FOR_sha256msg1 = 5693,
  CODE_FOR_sha256msg2 = 5694,
  CODE_FOR_sha256rnds2 = 5695,
  CODE_FOR_avx512f_si512_si = 5696,
  CODE_FOR_avx512f_ps512_ps = 5697,
  CODE_FOR_avx512f_pd512_pd = 5698,
  CODE_FOR_avx512f_si512_256si = 5699,
  CODE_FOR_avx512f_ps512_256ps = 5700,
  CODE_FOR_avx512f_pd512_256pd = 5701,
  CODE_FOR_vpamdd52luqv8di = 5702,
  CODE_FOR_vpamdd52luqv8di_maskz_1 = 5703,
  CODE_FOR_vpamdd52huqv8di = 5704,
  CODE_FOR_vpamdd52huqv8di_maskz_1 = 5705,
  CODE_FOR_vpamdd52luqv4di = 5706,
  CODE_FOR_vpamdd52luqv4di_maskz_1 = 5707,
  CODE_FOR_vpamdd52huqv4di = 5708,
  CODE_FOR_vpamdd52huqv4di_maskz_1 = 5709,
  CODE_FOR_vpamdd52luqv2di = 5710,
  CODE_FOR_vpamdd52luqv2di_maskz_1 = 5711,
  CODE_FOR_vpamdd52huqv2di = 5712,
  CODE_FOR_vpamdd52huqv2di_maskz_1 = 5713,
  CODE_FOR_vpamdd52luqv8di_mask = 5714,
  CODE_FOR_vpamdd52huqv8di_mask = 5715,
  CODE_FOR_vpamdd52luqv4di_mask = 5716,
  CODE_FOR_vpamdd52huqv4di_mask = 5717,
  CODE_FOR_vpamdd52luqv2di_mask = 5718,
  CODE_FOR_vpamdd52huqv2di_mask = 5719,
  CODE_FOR_vpmultishiftqbv64qi = 5720,
  CODE_FOR_vpmultishiftqbv64qi_mask = 5721,
  CODE_FOR_vpmultishiftqbv16qi = 5722,
  CODE_FOR_vpmultishiftqbv16qi_mask = 5723,
  CODE_FOR_vpmultishiftqbv32qi = 5724,
  CODE_FOR_vpmultishiftqbv32qi_mask = 5725,
  CODE_FOR_avx5124fmaddps_4fmaddps = 5728,
  CODE_FOR_avx5124fmaddps_4fmaddps_mask = 5729,
  CODE_FOR_avx5124fmaddps_4fmaddps_maskz = 5730,
  CODE_FOR_avx5124fmaddps_4fmaddss = 5731,
  CODE_FOR_avx5124fmaddps_4fmaddss_mask = 5732,
  CODE_FOR_avx5124fmaddps_4fmaddss_maskz = 5733,
  CODE_FOR_avx5124fmaddps_4fnmaddps = 5734,
  CODE_FOR_avx5124fmaddps_4fnmaddps_mask = 5735,
  CODE_FOR_avx5124fmaddps_4fnmaddps_maskz = 5736,
  CODE_FOR_avx5124fmaddps_4fnmaddss = 5737,
  CODE_FOR_avx5124fmaddps_4fnmaddss_mask = 5738,
  CODE_FOR_avx5124fmaddps_4fnmaddss_maskz = 5739,
  CODE_FOR_avx5124vnniw_vp4dpwssd = 5740,
  CODE_FOR_avx5124vnniw_vp4dpwssd_mask = 5741,
  CODE_FOR_avx5124vnniw_vp4dpwssd_maskz = 5742,
  CODE_FOR_avx5124vnniw_vp4dpwssds = 5743,
  CODE_FOR_avx5124vnniw_vp4dpwssds_mask = 5744,
  CODE_FOR_avx5124vnniw_vp4dpwssds_maskz = 5745,
  CODE_FOR_vpopcountv16si = 5746,
  CODE_FOR_vpopcountv16si_mask = 5747,
  CODE_FOR_vpopcountv8si = 5748,
  CODE_FOR_vpopcountv8si_mask = 5749,
  CODE_FOR_vpopcountv4si = 5750,
  CODE_FOR_vpopcountv4si_mask = 5751,
  CODE_FOR_vpopcountv8di = 5752,
  CODE_FOR_vpopcountv8di_mask = 5753,
  CODE_FOR_vpopcountv4di = 5754,
  CODE_FOR_vpopcountv4di_mask = 5755,
  CODE_FOR_vpopcountv2di = 5756,
  CODE_FOR_vpopcountv2di_mask = 5757,
  CODE_FOR_vpopcountv64qi = 5766,
  CODE_FOR_vpopcountv64qi_mask = 5767,
  CODE_FOR_vpopcountv16qi = 5768,
  CODE_FOR_vpopcountv16qi_mask = 5769,
  CODE_FOR_vpopcountv32qi = 5770,
  CODE_FOR_vpopcountv32qi_mask = 5771,
  CODE_FOR_vpopcountv32hi = 5772,
  CODE_FOR_vpopcountv32hi_mask = 5773,
  CODE_FOR_vpopcountv16hi = 5774,
  CODE_FOR_vpopcountv16hi_mask = 5775,
  CODE_FOR_vpopcountv8hi = 5776,
  CODE_FOR_vpopcountv8hi_mask = 5777,
  CODE_FOR_vgf2p8affineinvqb_v64qi = 5778,
  CODE_FOR_vgf2p8affineinvqb_v64qi_mask = 5779,
  CODE_FOR_vgf2p8affineinvqb_v32qi = 5780,
  CODE_FOR_vgf2p8affineinvqb_v32qi_mask = 5781,
  CODE_FOR_vgf2p8affineinvqb_v16qi = 5782,
  CODE_FOR_vgf2p8affineinvqb_v16qi_mask = 5783,
  CODE_FOR_vgf2p8affineqb_v64qi = 5784,
  CODE_FOR_vgf2p8affineqb_v64qi_mask = 5785,
  CODE_FOR_vgf2p8affineqb_v32qi = 5786,
  CODE_FOR_vgf2p8affineqb_v32qi_mask = 5787,
  CODE_FOR_vgf2p8affineqb_v16qi = 5788,
  CODE_FOR_vgf2p8affineqb_v16qi_mask = 5789,
  CODE_FOR_vgf2p8mulb_v64qi = 5790,
  CODE_FOR_vgf2p8mulb_v64qi_mask = 5791,
  CODE_FOR_vgf2p8mulb_v32qi = 5792,
  CODE_FOR_vgf2p8mulb_v32qi_mask = 5793,
  CODE_FOR_vgf2p8mulb_v16qi = 5794,
  CODE_FOR_vgf2p8mulb_v16qi_mask = 5795,
  CODE_FOR_vpshrd_v32hi = 5796,
  CODE_FOR_vpshrd_v32hi_mask = 5797,
  CODE_FOR_vpshrd_v16si = 5798,
  CODE_FOR_vpshrd_v16si_mask = 5799,
  CODE_FOR_vpshrd_v8di = 5800,
  CODE_FOR_vpshrd_v8di_mask = 5801,
  CODE_FOR_vpshrd_v16hi = 5802,
  CODE_FOR_vpshrd_v16hi_mask = 5803,
  CODE_FOR_vpshrd_v8si = 5804,
  CODE_FOR_vpshrd_v8si_mask = 5805,
  CODE_FOR_vpshrd_v4di = 5806,
  CODE_FOR_vpshrd_v4di_mask = 5807,
  CODE_FOR_vpshrd_v8hi = 5808,
  CODE_FOR_vpshrd_v8hi_mask = 5809,
  CODE_FOR_vpshrd_v4si = 5810,
  CODE_FOR_vpshrd_v4si_mask = 5811,
  CODE_FOR_vpshrd_v2di = 5812,
  CODE_FOR_vpshrd_v2di_mask = 5813,
  CODE_FOR_vpshld_v32hi = 5814,
  CODE_FOR_vpshld_v32hi_mask = 5815,
  CODE_FOR_vpshld_v16si = 5816,
  CODE_FOR_vpshld_v16si_mask = 5817,
  CODE_FOR_vpshld_v8di = 5818,
  CODE_FOR_vpshld_v8di_mask = 5819,
  CODE_FOR_vpshld_v16hi = 5820,
  CODE_FOR_vpshld_v16hi_mask = 5821,
  CODE_FOR_vpshld_v8si = 5822,
  CODE_FOR_vpshld_v8si_mask = 5823,
  CODE_FOR_vpshld_v4di = 5824,
  CODE_FOR_vpshld_v4di_mask = 5825,
  CODE_FOR_vpshld_v8hi = 5826,
  CODE_FOR_vpshld_v8hi_mask = 5827,
  CODE_FOR_vpshld_v4si = 5828,
  CODE_FOR_vpshld_v4si_mask = 5829,
  CODE_FOR_vpshld_v2di = 5830,
  CODE_FOR_vpshld_v2di_mask = 5831,
  CODE_FOR_vpshrdv_v32hi = 5832,
  CODE_FOR_vpshrdv_v16si = 5833,
  CODE_FOR_vpshrdv_v8di = 5834,
  CODE_FOR_vpshrdv_v16hi = 5835,
  CODE_FOR_vpshrdv_v8si = 5836,
  CODE_FOR_vpshrdv_v4di = 5837,
  CODE_FOR_vpshrdv_v8hi = 5838,
  CODE_FOR_vpshrdv_v4si = 5839,
  CODE_FOR_vpshrdv_v2di = 5840,
  CODE_FOR_vpshrdv_v32hi_mask = 5841,
  CODE_FOR_vpshrdv_v16si_mask = 5842,
  CODE_FOR_vpshrdv_v8di_mask = 5843,
  CODE_FOR_vpshrdv_v16hi_mask = 5844,
  CODE_FOR_vpshrdv_v8si_mask = 5845,
  CODE_FOR_vpshrdv_v4di_mask = 5846,
  CODE_FOR_vpshrdv_v8hi_mask = 5847,
  CODE_FOR_vpshrdv_v4si_mask = 5848,
  CODE_FOR_vpshrdv_v2di_mask = 5849,
  CODE_FOR_vpshrdv_v32hi_maskz_1 = 5850,
  CODE_FOR_vpshrdv_v16si_maskz_1 = 5851,
  CODE_FOR_vpshrdv_v8di_maskz_1 = 5852,
  CODE_FOR_vpshrdv_v16hi_maskz_1 = 5853,
  CODE_FOR_vpshrdv_v8si_maskz_1 = 5854,
  CODE_FOR_vpshrdv_v4di_maskz_1 = 5855,
  CODE_FOR_vpshrdv_v8hi_maskz_1 = 5856,
  CODE_FOR_vpshrdv_v4si_maskz_1 = 5857,
  CODE_FOR_vpshrdv_v2di_maskz_1 = 5858,
  CODE_FOR_vpshldv_v32hi = 5859,
  CODE_FOR_vpshldv_v16si = 5860,
  CODE_FOR_vpshldv_v8di = 5861,
  CODE_FOR_vpshldv_v16hi = 5862,
  CODE_FOR_vpshldv_v8si = 5863,
  CODE_FOR_vpshldv_v4di = 5864,
  CODE_FOR_vpshldv_v8hi = 5865,
  CODE_FOR_vpshldv_v4si = 5866,
  CODE_FOR_vpshldv_v2di = 5867,
  CODE_FOR_vpshldv_v32hi_mask = 5868,
  CODE_FOR_vpshldv_v16si_mask = 5869,
  CODE_FOR_vpshldv_v8di_mask = 5870,
  CODE_FOR_vpshldv_v16hi_mask = 5871,
  CODE_FOR_vpshldv_v8si_mask = 5872,
  CODE_FOR_vpshldv_v4di_mask = 5873,
  CODE_FOR_vpshldv_v8hi_mask = 5874,
  CODE_FOR_vpshldv_v4si_mask = 5875,
  CODE_FOR_vpshldv_v2di_mask = 5876,
  CODE_FOR_vpshldv_v32hi_maskz_1 = 5877,
  CODE_FOR_vpshldv_v16si_maskz_1 = 5878,
  CODE_FOR_vpshldv_v8di_maskz_1 = 5879,
  CODE_FOR_vpshldv_v16hi_maskz_1 = 5880,
  CODE_FOR_vpshldv_v8si_maskz_1 = 5881,
  CODE_FOR_vpshldv_v4di_maskz_1 = 5882,
  CODE_FOR_vpshldv_v8hi_maskz_1 = 5883,
  CODE_FOR_vpshldv_v4si_maskz_1 = 5884,
  CODE_FOR_vpshldv_v2di_maskz_1 = 5885,
  CODE_FOR_vpdpbusd_v16si = 5886,
  CODE_FOR_vpdpbusd_v8si = 5887,
  CODE_FOR_vpdpbusd_v4si = 5888,
  CODE_FOR_vpdpbusd_v16si_mask = 5889,
  CODE_FOR_vpdpbusd_v8si_mask = 5890,
  CODE_FOR_vpdpbusd_v4si_mask = 5891,
  CODE_FOR_vpdpbusd_v16si_maskz_1 = 5892,
  CODE_FOR_vpdpbusd_v8si_maskz_1 = 5893,
  CODE_FOR_vpdpbusd_v4si_maskz_1 = 5894,
  CODE_FOR_vpdpbusds_v16si = 5895,
  CODE_FOR_vpdpbusds_v8si = 5896,
  CODE_FOR_vpdpbusds_v4si = 5897,
  CODE_FOR_vpdpbusds_v16si_mask = 5898,
  CODE_FOR_vpdpbusds_v8si_mask = 5899,
  CODE_FOR_vpdpbusds_v4si_mask = 5900,
  CODE_FOR_vpdpbusds_v16si_maskz_1 = 5901,
  CODE_FOR_vpdpbusds_v8si_maskz_1 = 5902,
  CODE_FOR_vpdpbusds_v4si_maskz_1 = 5903,
  CODE_FOR_vpdpwssd_v16si = 5904,
  CODE_FOR_vpdpwssd_v8si = 5905,
  CODE_FOR_vpdpwssd_v4si = 5906,
  CODE_FOR_vpdpwssd_v16si_mask = 5907,
  CODE_FOR_vpdpwssd_v8si_mask = 5908,
  CODE_FOR_vpdpwssd_v4si_mask = 5909,
  CODE_FOR_vpdpwssd_v16si_maskz_1 = 5910,
  CODE_FOR_vpdpwssd_v8si_maskz_1 = 5911,
  CODE_FOR_vpdpwssd_v4si_maskz_1 = 5912,
  CODE_FOR_vpdpwssds_v16si = 5913,
  CODE_FOR_vpdpwssds_v8si = 5914,
  CODE_FOR_vpdpwssds_v4si = 5915,
  CODE_FOR_vpdpwssds_v16si_mask = 5916,
  CODE_FOR_vpdpwssds_v8si_mask = 5917,
  CODE_FOR_vpdpwssds_v4si_mask = 5918,
  CODE_FOR_vpdpwssds_v16si_maskz_1 = 5919,
  CODE_FOR_vpdpwssds_v8si_maskz_1 = 5920,
  CODE_FOR_vpdpwssds_v4si_maskz_1 = 5921,
  CODE_FOR_vaesdec_v32qi = 5922,
  CODE_FOR_vaesdec_v16qi = 5923,
  CODE_FOR_vaesdec_v64qi = 5924,
  CODE_FOR_vaesdeclast_v32qi = 5925,
  CODE_FOR_vaesdeclast_v16qi = 5926,
  CODE_FOR_vaesdeclast_v64qi = 5927,
  CODE_FOR_vaesenc_v32qi = 5928,
  CODE_FOR_vaesenc_v16qi = 5929,
  CODE_FOR_vaesenc_v64qi = 5930,
  CODE_FOR_vaesenclast_v32qi = 5931,
  CODE_FOR_vaesenclast_v16qi = 5932,
  CODE_FOR_vaesenclast_v64qi = 5933,
  CODE_FOR_vpclmulqdq_v8di = 5934,
  CODE_FOR_vpclmulqdq_v4di = 5935,
  CODE_FOR_vpclmulqdq_v2di = 5936,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi = 5937,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi_mask = 5938,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi = 5939,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi_mask = 5940,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi = 5941,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi_mask = 5942,
  CODE_FOR_avx512vp2intersect_2intersectv8di = 5945,
  CODE_FOR_avx512vp2intersect_2intersectv4di = 5946,
  CODE_FOR_avx512vp2intersect_2intersectv2di = 5947,
  CODE_FOR_avx512vp2intersect_2intersectv8si = 5948,
  CODE_FOR_avx512vp2intersect_2intersectv4si = 5949,
  CODE_FOR_avx512vp2intersect_2intersectv16si = 5950,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32hi = 5951,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32hi_mask = 5952,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16hi = 5953,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16hi_mask = 5954,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8hi = 5955,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8hi_mask = 5956,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf = 5957,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf_mask = 5958,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf = 5959,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf_mask = 5960,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf = 5961,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_mask = 5962,
  CODE_FOR_avx512f_dpbf16ps_v16sf = 5963,
  CODE_FOR_avx512f_dpbf16ps_v16sf_maskz_1 = 5964,
  CODE_FOR_avx512f_dpbf16ps_v8sf = 5965,
  CODE_FOR_avx512f_dpbf16ps_v8sf_maskz_1 = 5966,
  CODE_FOR_avx512f_dpbf16ps_v4sf = 5967,
  CODE_FOR_avx512f_dpbf16ps_v4sf_maskz_1 = 5968,
  CODE_FOR_avx512f_dpbf16ps_v16sf_mask = 5969,
  CODE_FOR_avx512f_dpbf16ps_v8sf_mask = 5970,
  CODE_FOR_avx512f_dpbf16ps_v4sf_mask = 5971,
  CODE_FOR_loadiwkey = 5972,
  CODE_FOR_aesdec128klu8 = 5975,
  CODE_FOR_aesdec256klu8 = 5976,
  CODE_FOR_aesenc128klu8 = 5977,
  CODE_FOR_aesenc256klu8 = 5978,
  CODE_FOR_mfence_sse2 = 5985,
  CODE_FOR_mfence_nosse = 5986,
  CODE_FOR_atomic_loaddi_fpu = 5987,
  CODE_FOR_atomic_storeqi_1 = 5988,
  CODE_FOR_atomic_storehi_1 = 5989,
  CODE_FOR_atomic_storesi_1 = 5990,
  CODE_FOR_atomic_storedi_1 = 5991,
  CODE_FOR_atomic_storedi_fpu = 5992,
  CODE_FOR_loaddi_via_fpu = 5993,
  CODE_FOR_storedi_via_fpu = 5994,
  CODE_FOR_loaddi_via_sse = 5995,
  CODE_FOR_storedi_via_sse = 5996,
  CODE_FOR_atomic_compare_and_swapdi_doubleword = 5997,
  CODE_FOR_atomic_compare_and_swapti_doubleword = 5998,
  CODE_FOR_atomic_compare_and_swapqi_1 = 5999,
  CODE_FOR_atomic_compare_and_swaphi_1 = 6000,
  CODE_FOR_atomic_compare_and_swapsi_1 = 6001,
  CODE_FOR_atomic_compare_and_swapdi_1 = 6002,
  CODE_FOR_atomic_fetch_addqi = 6003,
  CODE_FOR_atomic_fetch_addhi = 6004,
  CODE_FOR_atomic_fetch_addsi = 6005,
  CODE_FOR_atomic_fetch_adddi = 6006,
  CODE_FOR_atomic_exchangeqi = 6011,
  CODE_FOR_atomic_exchangehi = 6012,
  CODE_FOR_atomic_exchangesi = 6013,
  CODE_FOR_atomic_exchangedi = 6014,
  CODE_FOR_atomic_addqi = 6015,
  CODE_FOR_atomic_addhi = 6016,
  CODE_FOR_atomic_addsi = 6017,
  CODE_FOR_atomic_adddi = 6018,
  CODE_FOR_atomic_subqi = 6019,
  CODE_FOR_atomic_subhi = 6020,
  CODE_FOR_atomic_subsi = 6021,
  CODE_FOR_atomic_subdi = 6022,
  CODE_FOR_atomic_andqi = 6023,
  CODE_FOR_atomic_orqi = 6024,
  CODE_FOR_atomic_xorqi = 6025,
  CODE_FOR_atomic_andhi = 6026,
  CODE_FOR_atomic_orhi = 6027,
  CODE_FOR_atomic_xorhi = 6028,
  CODE_FOR_atomic_andsi = 6029,
  CODE_FOR_atomic_orsi = 6030,
  CODE_FOR_atomic_xorsi = 6031,
  CODE_FOR_atomic_anddi = 6032,
  CODE_FOR_atomic_ordi = 6033,
  CODE_FOR_atomic_xordi = 6034,
  CODE_FOR_atomic_bit_test_and_sethi_1 = 6035,
  CODE_FOR_atomic_bit_test_and_setsi_1 = 6036,
  CODE_FOR_atomic_bit_test_and_setdi_1 = 6037,
  CODE_FOR_atomic_bit_test_and_complementhi_1 = 6038,
  CODE_FOR_atomic_bit_test_and_complementsi_1 = 6039,
  CODE_FOR_atomic_bit_test_and_complementdi_1 = 6040,
  CODE_FOR_atomic_bit_test_and_resethi_1 = 6041,
  CODE_FOR_atomic_bit_test_and_resetsi_1 = 6042,
  CODE_FOR_atomic_bit_test_and_resetdi_1 = 6043,
  CODE_FOR_cbranchqi4 = 6044,
  CODE_FOR_cbranchhi4 = 6045,
  CODE_FOR_cbranchsi4 = 6046,
  CODE_FOR_cbranchdi4 = 6047,
  CODE_FOR_cbranchti4 = 6048,
  CODE_FOR_cstoreqi4 = 6049,
  CODE_FOR_cstorehi4 = 6050,
  CODE_FOR_cstoresi4 = 6051,
  CODE_FOR_cstoredi4 = 6052,
  CODE_FOR_cmpsi_1 = 6053,
  CODE_FOR_cmpdi_1 = 6054,
  CODE_FOR_cmpqi_ext_3 = 6055,
  CODE_FOR_cbranchxf4 = 6056,
  CODE_FOR_cstorexf4 = 6057,
  CODE_FOR_cbranchsf4 = 6058,
  CODE_FOR_cbranchdf4 = 6059,
  CODE_FOR_cstoresf4 = 6060,
  CODE_FOR_cstoredf4 = 6061,
  CODE_FOR_cbranchcc4 = 6062,
  CODE_FOR_cstorecc4 = 6063,
  CODE_FOR_reload_noff_store = 6064,
  CODE_FOR_reload_noff_load = 6065,
  CODE_FOR_movxi = 6066,
  CODE_FOR_movoi = 6067,
  CODE_FOR_movti = 6068,
  CODE_FOR_movcdi = 6069,
  CODE_FOR_movqi = 6070,
  CODE_FOR_movhi = 6071,
  CODE_FOR_movsi = 6072,
  CODE_FOR_movdi = 6073,
  CODE_FOR_movstrictqi = 6074,
  CODE_FOR_movstricthi = 6075,
  CODE_FOR_extvhi = 6076,
  CODE_FOR_extvsi = 6077,
  CODE_FOR_extzvhi = 6078,
  CODE_FOR_extzvsi = 6079,
  CODE_FOR_extzvdi = 6080,
  CODE_FOR_insvhi = 6081,
  CODE_FOR_insvsi = 6082,
  CODE_FOR_insvdi = 6083,
  CODE_FOR_movtf = 6084,
  CODE_FOR_movsf = 6085,
  CODE_FOR_movdf = 6086,
  CODE_FOR_movxf = 6087,
  CODE_FOR_zero_extendsidi2 = 6088,
  CODE_FOR_zero_extendqisi2 = 6089,
  CODE_FOR_zero_extendhisi2 = 6090,
  CODE_FOR_zero_extendqihi2 = 6091,
  CODE_FOR_extendsidi2 = 6092,
  CODE_FOR_extendsfdf2 = 6093,
  CODE_FOR_extendsfxf2 = 6094,
  CODE_FOR_extenddfxf2 = 6095,
  CODE_FOR_fix_truncxfdi2 = 6096,
  CODE_FOR_fix_truncsfdi2 = 6097,
  CODE_FOR_fix_truncdfdi2 = 6098,
  CODE_FOR_fix_truncxfsi2 = 6099,
  CODE_FOR_fix_truncsfsi2 = 6100,
  CODE_FOR_fix_truncdfsi2 = 6101,
  CODE_FOR_fix_truncsfhi2 = 6102,
  CODE_FOR_fix_truncdfhi2 = 6103,
  CODE_FOR_fix_truncxfhi2 = 6104,
  CODE_FOR_fixuns_truncsfsi2 = 6105,
  CODE_FOR_fixuns_truncdfsi2 = 6106,
  CODE_FOR_fixuns_truncsfhi2 = 6107,
  CODE_FOR_fixuns_truncdfhi2 = 6108,
  CODE_FOR_floatsisf2 = 6109,
  CODE_FOR_floatdisf2 = 6110,
  CODE_FOR_floatsidf2 = 6111,
  CODE_FOR_floatdidf2 = 6112,
  CODE_FOR_floatunsqisf2 = 6113,
  CODE_FOR_floatunshisf2 = 6114,
  CODE_FOR_floatunsqidf2 = 6115,
  CODE_FOR_floatunshidf2 = 6116,
  CODE_FOR_floatunssisf2 = 6117,
  CODE_FOR_floatunssidf2 = 6118,
  CODE_FOR_floatunssixf2 = 6119,
  CODE_FOR_floatunsdisf2 = 6120,
  CODE_FOR_floatunsdidf2 = 6121,
  CODE_FOR_addqi3 = 6122,
  CODE_FOR_addhi3 = 6123,
  CODE_FOR_addsi3 = 6124,
  CODE_FOR_adddi3 = 6125,
  CODE_FOR_addti3 = 6126,
  CODE_FOR_addqi_ext_1 = 6127,
  CODE_FOR_addvqi4 = 6128,
  CODE_FOR_addvhi4 = 6129,
  CODE_FOR_addvsi4 = 6130,
  CODE_FOR_addvdi4 = 6131,
  CODE_FOR_addvti4 = 6132,
  CODE_FOR_uaddvqi4 = 6133,
  CODE_FOR_uaddvhi4 = 6134,
  CODE_FOR_uaddvsi4 = 6135,
  CODE_FOR_uaddvdi4 = 6136,
  CODE_FOR_uaddvti4 = 6137,
  CODE_FOR_subqi3 = 6138,
  CODE_FOR_subhi3 = 6139,
  CODE_FOR_subsi3 = 6140,
  CODE_FOR_subdi3 = 6141,
  CODE_FOR_subti3 = 6142,
  CODE_FOR_subvqi4 = 6143,
  CODE_FOR_subvhi4 = 6144,
  CODE_FOR_subvsi4 = 6145,
  CODE_FOR_subvdi4 = 6146,
  CODE_FOR_subvti4 = 6147,
  CODE_FOR_usubvqi4 = 6148,
  CODE_FOR_usubvhi4 = 6149,
  CODE_FOR_usubvsi4 = 6150,
  CODE_FOR_usubvdi4 = 6151,
  CODE_FOR_addcarrysi_0 = 6152,
  CODE_FOR_addcarrydi_0 = 6153,
  CODE_FOR_subborrowsi_0 = 6154,
  CODE_FOR_subborrowdi_0 = 6155,
  CODE_FOR_addqi3_cconly_overflow = 6156,
  CODE_FOR_addxf3 = 6157,
  CODE_FOR_subxf3 = 6158,
  CODE_FOR_addsf3 = 6159,
  CODE_FOR_subsf3 = 6160,
  CODE_FOR_adddf3 = 6161,
  CODE_FOR_subdf3 = 6162,
  CODE_FOR_mulhi3 = 6163,
  CODE_FOR_mulsi3 = 6164,
  CODE_FOR_muldi3 = 6165,
  CODE_FOR_mulqi3 = 6166,
  CODE_FOR_mulvhi4 = 6167,
  CODE_FOR_mulvsi4 = 6168,
  CODE_FOR_mulvdi4 = 6169,
  CODE_FOR_umulvhi4 = 6170,
  CODE_FOR_umulvsi4 = 6171,
  CODE_FOR_umulvdi4 = 6172,
  CODE_FOR_mulvqi4 = 6173,
  CODE_FOR_umulvqi4 = 6174,
  CODE_FOR_mulsidi3 = 6175,
  CODE_FOR_umulsidi3 = 6176,
  CODE_FOR_mulditi3 = 6177,
  CODE_FOR_umulditi3 = 6178,
  CODE_FOR_mulqihi3 = 6179,
  CODE_FOR_umulqihi3 = 6180,
  CODE_FOR_smulsi3_highpart = 6181,
  CODE_FOR_umulsi3_highpart = 6182,
  CODE_FOR_smuldi3_highpart = 6183,
  CODE_FOR_umuldi3_highpart = 6184,
  CODE_FOR_mulxf3 = 6185,
  CODE_FOR_mulsf3 = 6186,
  CODE_FOR_muldf3 = 6187,
  CODE_FOR_divxf3 = 6188,
  CODE_FOR_divsf3 = 6189,
  CODE_FOR_divdf3 = 6190,
  CODE_FOR_divmodhi4 = 6191,
  CODE_FOR_udivmodhi4 = 6192,
  CODE_FOR_divmodsi4 = 6193,
  CODE_FOR_udivmodsi4 = 6194,
  CODE_FOR_divmoddi4 = 6195,
  CODE_FOR_udivmoddi4 = 6196,
  CODE_FOR_divmodqi4 = 6197,
  CODE_FOR_udivmodqi4 = 6198,
  CODE_FOR_testsi_ccno_1 = 6199,
  CODE_FOR_testdi_ccno_1 = 6200,
  CODE_FOR_testqi_ccz_1 = 6201,
  CODE_FOR_testqi_ext_1_ccno = 6202,
  CODE_FOR_andqi3 = 6203,
  CODE_FOR_andhi3 = 6204,
  CODE_FOR_andsi3 = 6205,
  CODE_FOR_anddi3 = 6206,
  CODE_FOR_andqi_ext_1 = 6207,
  CODE_FOR_iorqi3 = 6208,
  CODE_FOR_xorqi3 = 6209,
  CODE_FOR_iorhi3 = 6210,
  CODE_FOR_xorhi3 = 6211,
  CODE_FOR_iorsi3 = 6212,
  CODE_FOR_xorsi3 = 6213,
  CODE_FOR_iordi3 = 6214,
  CODE_FOR_xordi3 = 6215,
  CODE_FOR_xorqi_ext_1_cc = 6216,
  CODE_FOR_negqi2 = 6217,
  CODE_FOR_neghi2 = 6218,
  CODE_FOR_negsi2 = 6219,
  CODE_FOR_negdi2 = 6220,
  CODE_FOR_negti2 = 6221,
  CODE_FOR_negvqi3 = 6222,
  CODE_FOR_negvhi3 = 6223,
  CODE_FOR_negvsi3 = 6224,
  CODE_FOR_negvdi3 = 6225,
  CODE_FOR_absqi2 = 6226,
  CODE_FOR_abshi2 = 6227,
  CODE_FOR_abssi2 = 6228,
  CODE_FOR_absdi2 = 6229,
  CODE_FOR_absti2 = 6230,
  CODE_FOR_abstf2 = 6231,
  CODE_FOR_negtf2 = 6232,
  CODE_FOR_abssf2 = 6233,
  CODE_FOR_negsf2 = 6234,
  CODE_FOR_absdf2 = 6235,
  CODE_FOR_negdf2 = 6236,
  CODE_FOR_absxf2 = 6237,
  CODE_FOR_negxf2 = 6238,
  CODE_FOR_copysignsf3 = 6239,
  CODE_FOR_copysigndf3 = 6240,
  CODE_FOR_copysigntf3 = 6241,
  CODE_FOR_xorsignsf3 = 6242,
  CODE_FOR_xorsigndf3 = 6243,
  CODE_FOR_one_cmplqi2 = 6244,
  CODE_FOR_one_cmplhi2 = 6245,
  CODE_FOR_one_cmplsi2 = 6246,
  CODE_FOR_one_cmpldi2 = 6247,
  CODE_FOR_ashlqi3 = 6248,
  CODE_FOR_ashlhi3 = 6249,
  CODE_FOR_ashlsi3 = 6250,
  CODE_FOR_ashldi3 = 6251,
  CODE_FOR_ashlti3 = 6252,
  CODE_FOR_x86_shiftsi_adj_1 = 6253,
  CODE_FOR_x86_shiftdi_adj_1 = 6254,
  CODE_FOR_x86_shiftsi_adj_2 = 6255,
  CODE_FOR_x86_shiftdi_adj_2 = 6256,
  CODE_FOR_lshrqi3 = 6257,
  CODE_FOR_ashrqi3 = 6258,
  CODE_FOR_lshrhi3 = 6259,
  CODE_FOR_ashrhi3 = 6260,
  CODE_FOR_lshrsi3 = 6261,
  CODE_FOR_ashrsi3 = 6262,
  CODE_FOR_lshrdi3 = 6263,
  CODE_FOR_ashrdi3 = 6264,
  CODE_FOR_lshrti3 = 6265,
  CODE_FOR_ashrti3 = 6266,
  CODE_FOR_x86_shiftsi_adj_3 = 6267,
  CODE_FOR_x86_shiftdi_adj_3 = 6268,
  CODE_FOR_rotlti3 = 6269,
  CODE_FOR_rotrti3 = 6270,
  CODE_FOR_rotldi3 = 6271,
  CODE_FOR_rotrdi3 = 6272,
  CODE_FOR_rotlqi3 = 6273,
  CODE_FOR_rotrqi3 = 6274,
  CODE_FOR_rotlhi3 = 6275,
  CODE_FOR_rotrhi3 = 6276,
  CODE_FOR_rotlsi3 = 6277,
  CODE_FOR_rotrsi3 = 6278,
  CODE_FOR_indirect_jump = 6279,
  CODE_FOR_tablejump = 6280,
  CODE_FOR_call = 6281,
  CODE_FOR_sibcall = 6282,
  CODE_FOR_call_pop = 6283,
  CODE_FOR_call_value = 6284,
  CODE_FOR_sibcall_value = 6285,
  CODE_FOR_call_value_pop = 6286,
  CODE_FOR_untyped_call = 6287,
  CODE_FOR_memory_blockage = 6288,
  CODE_FOR_return = 6289,
  CODE_FOR_simple_return = 6290,
  CODE_FOR_simple_return_indirect_internal = 6291,
  CODE_FOR_prologue = 6292,
  CODE_FOR_set_got = 6293,
  CODE_FOR_set_got_labelled = 6294,
  CODE_FOR_epilogue = 6295,
  CODE_FOR_sibcall_epilogue = 6296,
  CODE_FOR_eh_return = 6297,
  CODE_FOR_leave_si = 6298,
  CODE_FOR_leave_di = 6299,
  CODE_FOR_split_stack_prologue = 6300,
  CODE_FOR_split_stack_space_check = 6301,
  CODE_FOR_ffssi2 = 6302,
  CODE_FOR_ffsdi2 = 6303,
  CODE_FOR_clzsi2 = 6304,
  CODE_FOR_clzdi2 = 6305,
  CODE_FOR_bmi2_bzhi_si3 = 6306,
  CODE_FOR_bmi2_bzhi_di3 = 6307,
  CODE_FOR_bswapdi2 = 6308,
  CODE_FOR_bswapsi2 = 6309,
  CODE_FOR_bswaphi2 = 6310,
  CODE_FOR_paritydi2 = 6311,
  CODE_FOR_paritysi2 = 6312,
  CODE_FOR_parityhi2 = 6313,
  CODE_FOR_parityqi2 = 6314,
  CODE_FOR_tls_global_dynamic_32 = 6315,
  CODE_FOR_tls_global_dynamic_64_si = 6316,
  CODE_FOR_tls_global_dynamic_64_di = 6317,
  CODE_FOR_tls_local_dynamic_base_32 = 6318,
  CODE_FOR_tls_local_dynamic_base_64_si = 6319,
  CODE_FOR_tls_local_dynamic_base_64_di = 6320,
  CODE_FOR_get_thread_pointersi = 6321,
  CODE_FOR_get_thread_pointerdi = 6322,
  CODE_FOR_tls_dynamic_gnu2_32 = 6323,
  CODE_FOR_tls_dynamic_gnu2_64_si = 6324,
  CODE_FOR_tls_dynamic_gnu2_64_di = 6325,
  CODE_FOR_rsqrtsf2 = 6326,
  CODE_FOR_sqrtsf2 = 6327,
  CODE_FOR_sqrtdf2 = 6328,
  CODE_FOR_hypotsf3 = 6329,
  CODE_FOR_hypotdf3 = 6330,
  CODE_FOR_fmodxf3 = 6331,
  CODE_FOR_fmodsf3 = 6332,
  CODE_FOR_fmoddf3 = 6333,
  CODE_FOR_remainderxf3 = 6334,
  CODE_FOR_remaindersf3 = 6335,
  CODE_FOR_remainderdf3 = 6336,
  CODE_FOR_sinsf2 = 6337,
  CODE_FOR_cossf2 = 6338,
  CODE_FOR_sindf2 = 6339,
  CODE_FOR_cosdf2 = 6340,
  CODE_FOR_sincossf3 = 6341,
  CODE_FOR_sincosdf3 = 6342,
  CODE_FOR_tanxf2 = 6343,
  CODE_FOR_tansf2 = 6344,
  CODE_FOR_tandf2 = 6345,
  CODE_FOR_atan2sf3 = 6346,
  CODE_FOR_atan2df3 = 6347,
  CODE_FOR_atanxf2 = 6348,
  CODE_FOR_atansf2 = 6349,
  CODE_FOR_atandf2 = 6350,
  CODE_FOR_asinxf2 = 6351,
  CODE_FOR_asinsf2 = 6352,
  CODE_FOR_asindf2 = 6353,
  CODE_FOR_acosxf2 = 6354,
  CODE_FOR_acossf2 = 6355,
  CODE_FOR_acosdf2 = 6356,
  CODE_FOR_sinhxf2 = 6357,
  CODE_FOR_sinhsf2 = 6358,
  CODE_FOR_sinhdf2 = 6359,
  CODE_FOR_coshxf2 = 6360,
  CODE_FOR_coshsf2 = 6361,
  CODE_FOR_coshdf2 = 6362,
  CODE_FOR_tanhxf2 = 6363,
  CODE_FOR_tanhsf2 = 6364,
  CODE_FOR_tanhdf2 = 6365,
  CODE_FOR_asinhxf2 = 6366,
  CODE_FOR_asinhsf2 = 6367,
  CODE_FOR_asinhdf2 = 6368,
  CODE_FOR_acoshxf2 = 6369,
  CODE_FOR_acoshsf2 = 6370,
  CODE_FOR_acoshdf2 = 6371,
  CODE_FOR_atanhxf2 = 6372,
  CODE_FOR_atanhsf2 = 6373,
  CODE_FOR_atanhdf2 = 6374,
  CODE_FOR_logxf2 = 6375,
  CODE_FOR_logsf2 = 6376,
  CODE_FOR_logdf2 = 6377,
  CODE_FOR_log10xf2 = 6378,
  CODE_FOR_log10sf2 = 6379,
  CODE_FOR_log10df2 = 6380,
  CODE_FOR_log2xf2 = 6381,
  CODE_FOR_log2sf2 = 6382,
  CODE_FOR_log2df2 = 6383,
  CODE_FOR_log1pxf2 = 6384,
  CODE_FOR_log1psf2 = 6385,
  CODE_FOR_log1pdf2 = 6386,
  CODE_FOR_logbxf2 = 6387,
  CODE_FOR_logbsf2 = 6388,
  CODE_FOR_logbdf2 = 6389,
  CODE_FOR_ilogbxf2 = 6390,
  CODE_FOR_ilogbsf2 = 6391,
  CODE_FOR_ilogbdf2 = 6392,
  CODE_FOR_expNcorexf3 = 6393,
  CODE_FOR_expxf2 = 6394,
  CODE_FOR_expsf2 = 6395,
  CODE_FOR_expdf2 = 6396,
  CODE_FOR_exp10xf2 = 6397,
  CODE_FOR_exp10sf2 = 6398,
  CODE_FOR_exp10df2 = 6399,
  CODE_FOR_exp2xf2 = 6400,
  CODE_FOR_exp2sf2 = 6401,
  CODE_FOR_exp2df2 = 6402,
  CODE_FOR_expm1xf2 = 6403,
  CODE_FOR_expm1sf2 = 6404,
  CODE_FOR_expm1df2 = 6405,
  CODE_FOR_ldexpxf3 = 6406,
  CODE_FOR_ldexpsf3 = 6407,
  CODE_FOR_ldexpdf3 = 6408,
  CODE_FOR_scalbxf3 = 6409,
  CODE_FOR_scalbsf3 = 6410,
  CODE_FOR_scalbdf3 = 6411,
  CODE_FOR_significandxf2 = 6412,
  CODE_FOR_significandsf2 = 6413,
  CODE_FOR_significanddf2 = 6414,
  CODE_FOR_rintsf2 = 6415,
  CODE_FOR_rintdf2 = 6416,
  CODE_FOR_nearbyintxf2 = 6417,
  CODE_FOR_nearbyintsf2 = 6418,
  CODE_FOR_nearbyintdf2 = 6419,
  CODE_FOR_roundsf2 = 6420,
  CODE_FOR_rounddf2 = 6421,
  CODE_FOR_roundxf2 = 6422,
  CODE_FOR_lrintsfsi2 = 6423,
  CODE_FOR_lrintsfdi2 = 6424,
  CODE_FOR_lrintdfsi2 = 6425,
  CODE_FOR_lrintdfdi2 = 6426,
  CODE_FOR_lroundsfhi2 = 6427,
  CODE_FOR_lrounddfhi2 = 6428,
  CODE_FOR_lroundxfhi2 = 6429,
  CODE_FOR_lroundsfsi2 = 6430,
  CODE_FOR_lrounddfsi2 = 6431,
  CODE_FOR_lroundxfsi2 = 6432,
  CODE_FOR_lroundsfdi2 = 6433,
  CODE_FOR_lrounddfdi2 = 6434,
  CODE_FOR_lroundxfdi2 = 6435,
  CODE_FOR_roundevenxf2 = 6436,
  CODE_FOR_floorxf2 = 6437,
  CODE_FOR_ceilxf2 = 6438,
  CODE_FOR_btruncxf2 = 6439,
  CODE_FOR_roundevensf2 = 6440,
  CODE_FOR_floorsf2 = 6441,
  CODE_FOR_ceilsf2 = 6442,
  CODE_FOR_btruncsf2 = 6443,
  CODE_FOR_roundevendf2 = 6444,
  CODE_FOR_floordf2 = 6445,
  CODE_FOR_ceildf2 = 6446,
  CODE_FOR_btruncdf2 = 6447,
  CODE_FOR_lfloorxfhi2 = 6448,
  CODE_FOR_lceilxfhi2 = 6449,
  CODE_FOR_lfloorxfsi2 = 6450,
  CODE_FOR_lceilxfsi2 = 6451,
  CODE_FOR_lfloorxfdi2 = 6452,
  CODE_FOR_lceilxfdi2 = 6453,
  CODE_FOR_lfloorsfsi2 = 6454,
  CODE_FOR_lceilsfsi2 = 6455,
  CODE_FOR_lfloorsfdi2 = 6456,
  CODE_FOR_lceilsfdi2 = 6457,
  CODE_FOR_lfloordfsi2 = 6458,
  CODE_FOR_lceildfsi2 = 6459,
  CODE_FOR_lfloordfdi2 = 6460,
  CODE_FOR_lceildfdi2 = 6461,
  CODE_FOR_signbittf2 = 6462,
  CODE_FOR_signbitxf2 = 6463,
  CODE_FOR_signbitdf2 = 6464,
  CODE_FOR_signbitsf2 = 6465,
  CODE_FOR_cpymemsi = 6466,
  CODE_FOR_cpymemdi = 6467,
  CODE_FOR_strmov = 6468,
  CODE_FOR_strmov_singleop = 6469,
  CODE_FOR_rep_mov = 6470,
  CODE_FOR_setmemsi = 6471,
  CODE_FOR_setmemdi = 6472,
  CODE_FOR_strset = 6473,
  CODE_FOR_strset_singleop = 6474,
  CODE_FOR_rep_stos = 6475,
  CODE_FOR_cmpmemsi = 6476,
  CODE_FOR_cmpstrnsi = 6477,
  CODE_FOR_cmpintqi = 6478,
  CODE_FOR_cmpstrnqi_nz_1 = 6479,
  CODE_FOR_cmpstrnqi_1 = 6480,
  CODE_FOR_strlensi = 6481,
  CODE_FOR_strlendi = 6482,
  CODE_FOR_strlenqi_1 = 6483,
  CODE_FOR_movqicc = 6484,
  CODE_FOR_movhicc = 6485,
  CODE_FOR_movsicc = 6486,
  CODE_FOR_movdicc = 6487,
  CODE_FOR_x86_movsicc_0_m1 = 6488,
  CODE_FOR_x86_movdicc_0_m1 = 6489,
  CODE_FOR_movsfcc = 6490,
  CODE_FOR_movdfcc = 6491,
  CODE_FOR_movxfcc = 6492,
  CODE_FOR_addqicc = 6493,
  CODE_FOR_addhicc = 6494,
  CODE_FOR_addsicc = 6495,
  CODE_FOR_adddicc = 6496,
  CODE_FOR_smaxqi3 = 6497,
  CODE_FOR_sminqi3 = 6498,
  CODE_FOR_umaxqi3 = 6499,
  CODE_FOR_uminqi3 = 6500,
  CODE_FOR_smaxhi3 = 6501,
  CODE_FOR_sminhi3 = 6502,
  CODE_FOR_umaxhi3 = 6503,
  CODE_FOR_uminhi3 = 6504,
  CODE_FOR_smaxsi3 = 6505,
  CODE_FOR_sminsi3 = 6506,
  CODE_FOR_umaxsi3 = 6507,
  CODE_FOR_uminsi3 = 6508,
  CODE_FOR_smaxdi3 = 6509,
  CODE_FOR_smindi3 = 6510,
  CODE_FOR_umaxdi3 = 6511,
  CODE_FOR_umindi3 = 6512,
  CODE_FOR_smaxti3 = 6513,
  CODE_FOR_sminti3 = 6514,
  CODE_FOR_umaxti3 = 6515,
  CODE_FOR_uminti3 = 6516,
  CODE_FOR_allocate_stack = 6517,
  CODE_FOR_probe_stack = 6518,
  CODE_FOR_builtin_setjmp_receiver = 6519,
  CODE_FOR_save_stack_nonlocal = 6520,
  CODE_FOR_restore_stack_nonlocal = 6521,
  CODE_FOR_prefetch = 6522,
  CODE_FOR_stack_protect_set = 6523,
  CODE_FOR_stack_protect_test = 6524,
  CODE_FOR_pause = 6525,
  CODE_FOR_xbegin = 6526,
  CODE_FOR_xtest = 6527,
  CODE_FOR_rdpkru = 6528,
  CODE_FOR_wrpkru = 6529,
  CODE_FOR_movv8qi = 6530,
  CODE_FOR_movv4hi = 6531,
  CODE_FOR_movv2si = 6532,
  CODE_FOR_movv1di = 6533,
  CODE_FOR_movv2sf = 6534,
  CODE_FOR_movmisalignv8qi = 6535,
  CODE_FOR_movmisalignv4hi = 6536,
  CODE_FOR_movmisalignv2si = 6537,
  CODE_FOR_movmisalignv1di = 6538,
  CODE_FOR_movmisalignv2sf = 6539,
  CODE_FOR_absv2sf2 = 6540,
  CODE_FOR_negv2sf2 = 6541,
  CODE_FOR_mmx_addv2sf3 = 6542,
  CODE_FOR_addv2sf3 = 6543,
  CODE_FOR_mmx_subv2sf3 = 6544,
  CODE_FOR_mmx_subrv2sf3 = 6545,
  CODE_FOR_subv2sf3 = 6546,
  CODE_FOR_mmx_mulv2sf3 = 6547,
  CODE_FOR_mulv2sf3 = 6548,
  CODE_FOR_mmx_smaxv2sf3 = 6549,
  CODE_FOR_mmx_sminv2sf3 = 6550,
  CODE_FOR_smaxv2sf3 = 6551,
  CODE_FOR_sminv2sf3 = 6552,
  CODE_FOR_mmx_haddv2sf3 = 6553,
  CODE_FOR_mmx_haddsubv2sf3 = 6554,
  CODE_FOR_mmx_eqv2sf3 = 6555,
  CODE_FOR_copysignv2sf3 = 6556,
  CODE_FOR_xorsignv2sf3 = 6557,
  CODE_FOR_signbitv2sf2 = 6558,
  CODE_FOR_fix_truncv2sfv2si2 = 6559,
  CODE_FOR_floatv2siv2sf2 = 6560,
  CODE_FOR_vec_setv2sf = 6561,
  CODE_FOR_vec_extractv2sfsf = 6562,
  CODE_FOR_vec_initv2sfsf = 6563,
  CODE_FOR_mmx_addv8qi3 = 6564,
  CODE_FOR_mmx_subv8qi3 = 6565,
  CODE_FOR_mmx_addv4hi3 = 6566,
  CODE_FOR_mmx_subv4hi3 = 6567,
  CODE_FOR_mmx_addv2si3 = 6568,
  CODE_FOR_mmx_subv2si3 = 6569,
  CODE_FOR_mmx_addv1di3 = 6570,
  CODE_FOR_mmx_subv1di3 = 6571,
  CODE_FOR_addv8qi3 = 6572,
  CODE_FOR_subv8qi3 = 6573,
  CODE_FOR_addv4hi3 = 6574,
  CODE_FOR_subv4hi3 = 6575,
  CODE_FOR_addv2si3 = 6576,
  CODE_FOR_subv2si3 = 6577,
  CODE_FOR_mmx_ssaddv8qi3 = 6578,
  CODE_FOR_mmx_usaddv8qi3 = 6579,
  CODE_FOR_mmx_sssubv8qi3 = 6580,
  CODE_FOR_mmx_ussubv8qi3 = 6581,
  CODE_FOR_mmx_ssaddv4hi3 = 6582,
  CODE_FOR_mmx_usaddv4hi3 = 6583,
  CODE_FOR_mmx_sssubv4hi3 = 6584,
  CODE_FOR_mmx_ussubv4hi3 = 6585,
  CODE_FOR_mmx_mulv4hi3 = 6586,
  CODE_FOR_mulv4hi3 = 6587,
  CODE_FOR_mmx_smulv4hi3_highpart = 6588,
  CODE_FOR_mmx_umulv4hi3_highpart = 6589,
  CODE_FOR_mmx_pmaddwd = 6590,
  CODE_FOR_mmx_pmulhrwv4hi3 = 6591,
  CODE_FOR_sse2_umulv1siv1di3 = 6592,
  CODE_FOR_mmx_smaxv4hi3 = 6593,
  CODE_FOR_mmx_sminv4hi3 = 6594,
  CODE_FOR_smaxv4hi3 = 6595,
  CODE_FOR_sminv4hi3 = 6596,
  CODE_FOR_mmx_umaxv8qi3 = 6597,
  CODE_FOR_mmx_uminv8qi3 = 6598,
  CODE_FOR_umaxv8qi3 = 6599,
  CODE_FOR_uminv8qi3 = 6600,
  CODE_FOR_ashrv4hi3 = 6601,
  CODE_FOR_ashrv2si3 = 6602,
  CODE_FOR_ashlv4hi3 = 6603,
  CODE_FOR_lshrv4hi3 = 6604,
  CODE_FOR_ashlv2si3 = 6605,
  CODE_FOR_lshrv2si3 = 6606,
  CODE_FOR_mmx_eqv8qi3 = 6607,
  CODE_FOR_mmx_eqv4hi3 = 6608,
  CODE_FOR_mmx_eqv2si3 = 6609,
  CODE_FOR_one_cmplv8qi2 = 6610,
  CODE_FOR_one_cmplv4hi2 = 6611,
  CODE_FOR_one_cmplv2si2 = 6612,
  CODE_FOR_mmx_andv8qi3 = 6613,
  CODE_FOR_mmx_iorv8qi3 = 6614,
  CODE_FOR_mmx_xorv8qi3 = 6615,
  CODE_FOR_mmx_andv4hi3 = 6616,
  CODE_FOR_mmx_iorv4hi3 = 6617,
  CODE_FOR_mmx_xorv4hi3 = 6618,
  CODE_FOR_mmx_andv2si3 = 6619,
  CODE_FOR_mmx_iorv2si3 = 6620,
  CODE_FOR_mmx_xorv2si3 = 6621,
  CODE_FOR_andv8qi3 = 6622,
  CODE_FOR_iorv8qi3 = 6623,
  CODE_FOR_xorv8qi3 = 6624,
  CODE_FOR_andv4hi3 = 6625,
  CODE_FOR_iorv4hi3 = 6626,
  CODE_FOR_xorv4hi3 = 6627,
  CODE_FOR_andv2si3 = 6628,
  CODE_FOR_iorv2si3 = 6629,
  CODE_FOR_xorv2si3 = 6630,
  CODE_FOR_mmx_pinsrw = 6631,
  CODE_FOR_mmx_pshufw = 6632,
  CODE_FOR_vec_setv2si = 6633,
  CODE_FOR_vec_extractv2sisi = 6634,
  CODE_FOR_vec_initv2sisi = 6635,
  CODE_FOR_vec_setv4hi = 6636,
  CODE_FOR_vec_extractv4hihi = 6637,
  CODE_FOR_vec_initv4hihi = 6638,
  CODE_FOR_vec_setv8qi = 6639,
  CODE_FOR_vec_extractv8qiqi = 6640,
  CODE_FOR_vec_initv8qiqi = 6641,
  CODE_FOR_mmx_uavgv8qi3 = 6642,
  CODE_FOR_mmx_uavgv4hi3 = 6643,
  CODE_FOR_uavgv8qi3_ceil = 6644,
  CODE_FOR_uavgv4hi3_ceil = 6645,
  CODE_FOR_reduc_plus_scal_v8qi = 6646,
  CODE_FOR_usadv8qi = 6647,
  CODE_FOR_mmx_maskmovq = 6648,
  CODE_FOR_mmx_emms = 6649,
  CODE_FOR_mmx_femms = 6650,
  CODE_FOR_movv64qi = 6651,
  CODE_FOR_movv32qi = 6652,
  CODE_FOR_movv16qi = 6653,
  CODE_FOR_movv32hi = 6654,
  CODE_FOR_movv16hi = 6655,
  CODE_FOR_movv8hi = 6656,
  CODE_FOR_movv16si = 6657,
  CODE_FOR_movv8si = 6658,
  CODE_FOR_movv4si = 6659,
  CODE_FOR_movv8di = 6660,
  CODE_FOR_movv4di = 6661,
  CODE_FOR_movv2di = 6662,
  CODE_FOR_movv4ti = 6663,
  CODE_FOR_movv2ti = 6664,
  CODE_FOR_movv1ti = 6665,
  CODE_FOR_movv16sf = 6666,
  CODE_FOR_movv8sf = 6667,
  CODE_FOR_movv4sf = 6668,
  CODE_FOR_movv8df = 6669,
  CODE_FOR_movv4df = 6670,
  CODE_FOR_movv2df = 6671,
  CODE_FOR_avx512f_loadv16si_mask = 6672,
  CODE_FOR_avx512vl_loadv8si_mask = 6673,
  CODE_FOR_avx512vl_loadv4si_mask = 6674,
  CODE_FOR_avx512f_loadv8di_mask = 6675,
  CODE_FOR_avx512vl_loadv4di_mask = 6676,
  CODE_FOR_avx512vl_loadv2di_mask = 6677,
  CODE_FOR_avx512f_loadv16sf_mask = 6678,
  CODE_FOR_avx512vl_loadv8sf_mask = 6679,
  CODE_FOR_avx512vl_loadv4sf_mask = 6680,
  CODE_FOR_avx512f_loadv8df_mask = 6681,
  CODE_FOR_avx512vl_loadv4df_mask = 6682,
  CODE_FOR_avx512vl_loadv2df_mask = 6683,
  CODE_FOR_avx512bw_loadv64qi_mask = 6684,
  CODE_FOR_avx512vl_loadv16qi_mask = 6685,
  CODE_FOR_avx512vl_loadv32qi_mask = 6686,
  CODE_FOR_avx512bw_loadv32hi_mask = 6687,
  CODE_FOR_avx512vl_loadv16hi_mask = 6688,
  CODE_FOR_avx512vl_loadv8hi_mask = 6689,
  CODE_FOR_avx512f_loadsf_mask = 6690,
  CODE_FOR_avx512f_loaddf_mask = 6691,
  CODE_FOR_movmisalignv64qi = 6692,
  CODE_FOR_movmisalignv32qi = 6693,
  CODE_FOR_movmisalignv16qi = 6694,
  CODE_FOR_movmisalignv32hi = 6695,
  CODE_FOR_movmisalignv16hi = 6696,
  CODE_FOR_movmisalignv8hi = 6697,
  CODE_FOR_movmisalignv16si = 6698,
  CODE_FOR_movmisalignv8si = 6699,
  CODE_FOR_movmisalignv4si = 6700,
  CODE_FOR_movmisalignv8di = 6701,
  CODE_FOR_movmisalignv4di = 6702,
  CODE_FOR_movmisalignv2di = 6703,
  CODE_FOR_movmisalignv4ti = 6704,
  CODE_FOR_movmisalignv2ti = 6705,
  CODE_FOR_movmisalignv1ti = 6706,
  CODE_FOR_movmisalignv16sf = 6707,
  CODE_FOR_movmisalignv8sf = 6708,
  CODE_FOR_movmisalignv4sf = 6709,
  CODE_FOR_movmisalignv8df = 6710,
  CODE_FOR_movmisalignv4df = 6711,
  CODE_FOR_movmisalignv2df = 6712,
  CODE_FOR_storentdi = 6713,
  CODE_FOR_storentsi = 6714,
  CODE_FOR_storentsf = 6715,
  CODE_FOR_storentdf = 6716,
  CODE_FOR_storentv8di = 6717,
  CODE_FOR_storentv4di = 6718,
  CODE_FOR_storentv2di = 6719,
  CODE_FOR_storentv16sf = 6720,
  CODE_FOR_storentv8sf = 6721,
  CODE_FOR_storentv4sf = 6722,
  CODE_FOR_storentv8df = 6723,
  CODE_FOR_storentv4df = 6724,
  CODE_FOR_storentv2df = 6725,
  CODE_FOR_kmovb = 6726,
  CODE_FOR_kmovw = 6727,
  CODE_FOR_kmovd = 6728,
  CODE_FOR_kmovq = 6729,
  CODE_FOR_absv16sf2 = 6730,
  CODE_FOR_negv16sf2 = 6731,
  CODE_FOR_absv8sf2 = 6732,
  CODE_FOR_negv8sf2 = 6733,
  CODE_FOR_absv4sf2 = 6734,
  CODE_FOR_negv4sf2 = 6735,
  CODE_FOR_absv8df2 = 6736,
  CODE_FOR_negv8df2 = 6737,
  CODE_FOR_absv4df2 = 6738,
  CODE_FOR_negv4df2 = 6739,
  CODE_FOR_absv2df2 = 6740,
  CODE_FOR_negv2df2 = 6741,
  CODE_FOR_addv16sf3 = 6742,
  CODE_FOR_addv16sf3_round = 6743,
  CODE_FOR_addv16sf3_mask = 6744,
  CODE_FOR_addv16sf3_mask_round = 6745,
  CODE_FOR_subv16sf3 = 6746,
  CODE_FOR_subv16sf3_round = 6747,
  CODE_FOR_subv16sf3_mask = 6748,
  CODE_FOR_subv16sf3_mask_round = 6749,
  CODE_FOR_addv8sf3 = 6750,
   CODE_FOR_addv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8sf3_mask = 6751,
   CODE_FOR_addv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3 = 6752,
   CODE_FOR_subv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3_mask = 6753,
   CODE_FOR_subv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3 = 6754,
   CODE_FOR_addv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3_mask = 6755,
   CODE_FOR_addv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3 = 6756,
   CODE_FOR_subv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3_mask = 6757,
   CODE_FOR_subv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8df3 = 6758,
  CODE_FOR_addv8df3_round = 6759,
  CODE_FOR_addv8df3_mask = 6760,
  CODE_FOR_addv8df3_mask_round = 6761,
  CODE_FOR_subv8df3 = 6762,
  CODE_FOR_subv8df3_round = 6763,
  CODE_FOR_subv8df3_mask = 6764,
  CODE_FOR_subv8df3_mask_round = 6765,
  CODE_FOR_addv4df3 = 6766,
   CODE_FOR_addv4df3_round = CODE_FOR_nothing,
  CODE_FOR_addv4df3_mask = 6767,
   CODE_FOR_addv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3 = 6768,
   CODE_FOR_subv4df3_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3_mask = 6769,
   CODE_FOR_subv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3 = 6770,
   CODE_FOR_addv2df3_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3_mask = 6771,
   CODE_FOR_addv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3 = 6772,
   CODE_FOR_subv2df3_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3_mask = 6773,
   CODE_FOR_subv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv16sf3 = 6774,
  CODE_FOR_mulv16sf3_round = 6775,
  CODE_FOR_mulv16sf3_mask = 6776,
  CODE_FOR_mulv16sf3_mask_round = 6777,
  CODE_FOR_mulv8sf3 = 6778,
   CODE_FOR_mulv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8sf3_mask = 6779,
   CODE_FOR_mulv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3 = 6780,
   CODE_FOR_mulv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3_mask = 6781,
   CODE_FOR_mulv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8df3 = 6782,
  CODE_FOR_mulv8df3_round = 6783,
  CODE_FOR_mulv8df3_mask = 6784,
  CODE_FOR_mulv8df3_mask_round = 6785,
  CODE_FOR_mulv4df3 = 6786,
   CODE_FOR_mulv4df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4df3_mask = 6787,
   CODE_FOR_mulv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3 = 6788,
   CODE_FOR_mulv2df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3_mask = 6789,
   CODE_FOR_mulv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_divv8df3 = 6790,
  CODE_FOR_divv4df3 = 6791,
  CODE_FOR_divv2df3 = 6792,
  CODE_FOR_divv16sf3 = 6793,
  CODE_FOR_divv8sf3 = 6794,
  CODE_FOR_divv4sf3 = 6795,
  CODE_FOR_sqrtv8df2 = 6796,
  CODE_FOR_sqrtv4df2 = 6797,
  CODE_FOR_sqrtv2df2 = 6798,
  CODE_FOR_sqrtv16sf2 = 6799,
  CODE_FOR_sqrtv8sf2 = 6800,
  CODE_FOR_sqrtv4sf2 = 6801,
  CODE_FOR_rsqrtv16sf2 = 6802,
  CODE_FOR_rsqrtv8sf2 = 6803,
  CODE_FOR_rsqrtv4sf2 = 6804,
  CODE_FOR_smaxv16sf3 = 6805,
  CODE_FOR_smaxv16sf3_round = 6806,
  CODE_FOR_smaxv16sf3_mask = 6807,
  CODE_FOR_smaxv16sf3_mask_round = 6808,
  CODE_FOR_sminv16sf3 = 6809,
  CODE_FOR_sminv16sf3_round = 6810,
  CODE_FOR_sminv16sf3_mask = 6811,
  CODE_FOR_sminv16sf3_mask_round = 6812,
  CODE_FOR_smaxv8sf3 = 6813,
   CODE_FOR_smaxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8sf3_mask = 6814,
   CODE_FOR_smaxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3 = 6815,
   CODE_FOR_sminv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3_mask = 6816,
   CODE_FOR_sminv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3 = 6817,
   CODE_FOR_smaxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3_mask = 6818,
   CODE_FOR_smaxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3 = 6819,
   CODE_FOR_sminv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3_mask = 6820,
   CODE_FOR_sminv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8df3 = 6821,
  CODE_FOR_smaxv8df3_round = 6822,
  CODE_FOR_smaxv8df3_mask = 6823,
  CODE_FOR_smaxv8df3_mask_round = 6824,
  CODE_FOR_sminv8df3 = 6825,
  CODE_FOR_sminv8df3_round = 6826,
  CODE_FOR_sminv8df3_mask = 6827,
  CODE_FOR_sminv8df3_mask_round = 6828,
  CODE_FOR_smaxv4df3 = 6829,
   CODE_FOR_smaxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4df3_mask = 6830,
   CODE_FOR_smaxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3 = 6831,
   CODE_FOR_sminv4df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3_mask = 6832,
   CODE_FOR_sminv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3 = 6833,
   CODE_FOR_smaxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3_mask = 6834,
   CODE_FOR_smaxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3 = 6835,
   CODE_FOR_sminv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3_mask = 6836,
   CODE_FOR_sminv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse3_haddv2df3 = 6837,
  CODE_FOR_reduc_plus_scal_v2df = 6838,
  CODE_FOR_reduc_plus_scal_v4sf = 6839,
  CODE_FOR_reduc_plus_scal_v16qi = 6840,
  CODE_FOR_reduc_plus_scal_v4df = 6841,
  CODE_FOR_reduc_plus_scal_v8sf = 6842,
  CODE_FOR_reduc_plus_scal_v8df = 6843,
  CODE_FOR_reduc_plus_scal_v16sf = 6844,
  CODE_FOR_reduc_plus_scal_v32qi = 6845,
  CODE_FOR_reduc_plus_scal_v64qi = 6846,
  CODE_FOR_reduc_smax_scal_v4sf = 6847,
  CODE_FOR_reduc_smin_scal_v4sf = 6848,
  CODE_FOR_reduc_smax_scal_v2df = 6849,
  CODE_FOR_reduc_smin_scal_v2df = 6850,
  CODE_FOR_reduc_smax_scal_v4si = 6851,
  CODE_FOR_reduc_smin_scal_v4si = 6852,
  CODE_FOR_reduc_smax_scal_v8hi = 6853,
  CODE_FOR_reduc_smin_scal_v8hi = 6854,
  CODE_FOR_reduc_smax_scal_v16qi = 6855,
  CODE_FOR_reduc_smin_scal_v16qi = 6856,
  CODE_FOR_reduc_smax_scal_v2di = 6857,
  CODE_FOR_reduc_smin_scal_v2di = 6858,
  CODE_FOR_reduc_smax_scal_v32qi = 6859,
  CODE_FOR_reduc_smin_scal_v32qi = 6860,
  CODE_FOR_reduc_smax_scal_v16hi = 6861,
  CODE_FOR_reduc_smin_scal_v16hi = 6862,
  CODE_FOR_reduc_smax_scal_v8si = 6863,
  CODE_FOR_reduc_smin_scal_v8si = 6864,
  CODE_FOR_reduc_smax_scal_v4di = 6865,
  CODE_FOR_reduc_smin_scal_v4di = 6866,
  CODE_FOR_reduc_smax_scal_v8sf = 6867,
  CODE_FOR_reduc_smin_scal_v8sf = 6868,
  CODE_FOR_reduc_smax_scal_v4df = 6869,
  CODE_FOR_reduc_smin_scal_v4df = 6870,
  CODE_FOR_reduc_smax_scal_v64qi = 6871,
  CODE_FOR_reduc_smin_scal_v64qi = 6872,
  CODE_FOR_reduc_smax_scal_v32hi = 6873,
  CODE_FOR_reduc_smin_scal_v32hi = 6874,
  CODE_FOR_reduc_smax_scal_v16si = 6875,
  CODE_FOR_reduc_smin_scal_v16si = 6876,
  CODE_FOR_reduc_smax_scal_v8di = 6877,
  CODE_FOR_reduc_smin_scal_v8di = 6878,
  CODE_FOR_reduc_smax_scal_v16sf = 6879,
  CODE_FOR_reduc_smin_scal_v16sf = 6880,
  CODE_FOR_reduc_smax_scal_v8df = 6881,
  CODE_FOR_reduc_smin_scal_v8df = 6882,
  CODE_FOR_reduc_umax_scal_v16si = 6883,
  CODE_FOR_reduc_umin_scal_v16si = 6884,
  CODE_FOR_reduc_umax_scal_v8di = 6885,
  CODE_FOR_reduc_umin_scal_v8di = 6886,
  CODE_FOR_reduc_umax_scal_v32hi = 6887,
  CODE_FOR_reduc_umin_scal_v32hi = 6888,
  CODE_FOR_reduc_umax_scal_v64qi = 6889,
  CODE_FOR_reduc_umin_scal_v64qi = 6890,
  CODE_FOR_reduc_umax_scal_v32qi = 6891,
  CODE_FOR_reduc_umin_scal_v32qi = 6892,
  CODE_FOR_reduc_umax_scal_v16hi = 6893,
  CODE_FOR_reduc_umin_scal_v16hi = 6894,
  CODE_FOR_reduc_umax_scal_v8si = 6895,
  CODE_FOR_reduc_umin_scal_v8si = 6896,
  CODE_FOR_reduc_umax_scal_v4di = 6897,
  CODE_FOR_reduc_umin_scal_v4di = 6898,
  CODE_FOR_reduc_umin_scal_v8hi = 6899,
  CODE_FOR_vec_cmpv16sihi = 6900,
  CODE_FOR_vec_cmpv8siqi = 6901,
  CODE_FOR_vec_cmpv4siqi = 6902,
  CODE_FOR_vec_cmpv8diqi = 6903,
  CODE_FOR_vec_cmpv4diqi = 6904,
  CODE_FOR_vec_cmpv2diqi = 6905,
  CODE_FOR_vec_cmpv16sfhi = 6906,
  CODE_FOR_vec_cmpv8sfqi = 6907,
  CODE_FOR_vec_cmpv4sfqi = 6908,
  CODE_FOR_vec_cmpv8dfqi = 6909,
  CODE_FOR_vec_cmpv4dfqi = 6910,
  CODE_FOR_vec_cmpv2dfqi = 6911,
  CODE_FOR_vec_cmpv64qidi = 6912,
  CODE_FOR_vec_cmpv16qihi = 6913,
  CODE_FOR_vec_cmpv32qisi = 6914,
  CODE_FOR_vec_cmpv32hisi = 6915,
  CODE_FOR_vec_cmpv16hihi = 6916,
  CODE_FOR_vec_cmpv8hiqi = 6917,
  CODE_FOR_vec_cmpv32qiv32qi = 6918,
  CODE_FOR_vec_cmpv16hiv16hi = 6919,
  CODE_FOR_vec_cmpv8siv8si = 6920,
  CODE_FOR_vec_cmpv4div4di = 6921,
  CODE_FOR_vec_cmpv16qiv16qi = 6922,
  CODE_FOR_vec_cmpv8hiv8hi = 6923,
  CODE_FOR_vec_cmpv4siv4si = 6924,
  CODE_FOR_vec_cmpv2div2di = 6925,
  CODE_FOR_vec_cmpv8sfv8si = 6926,
  CODE_FOR_vec_cmpv4dfv4di = 6927,
  CODE_FOR_vec_cmpv4sfv4si = 6928,
  CODE_FOR_vec_cmpv2dfv2di = 6929,
  CODE_FOR_vec_cmpuv16sihi = 6930,
  CODE_FOR_vec_cmpuv8siqi = 6931,
  CODE_FOR_vec_cmpuv4siqi = 6932,
  CODE_FOR_vec_cmpuv8diqi = 6933,
  CODE_FOR_vec_cmpuv4diqi = 6934,
  CODE_FOR_vec_cmpuv2diqi = 6935,
  CODE_FOR_vec_cmpuv64qidi = 6936,
  CODE_FOR_vec_cmpuv16qihi = 6937,
  CODE_FOR_vec_cmpuv32qisi = 6938,
  CODE_FOR_vec_cmpuv32hisi = 6939,
  CODE_FOR_vec_cmpuv16hihi = 6940,
  CODE_FOR_vec_cmpuv8hiqi = 6941,
  CODE_FOR_vec_cmpuv32qiv32qi = 6942,
  CODE_FOR_vec_cmpuv16hiv16hi = 6943,
  CODE_FOR_vec_cmpuv8siv8si = 6944,
  CODE_FOR_vec_cmpuv4div4di = 6945,
  CODE_FOR_vec_cmpuv16qiv16qi = 6946,
  CODE_FOR_vec_cmpuv8hiv8hi = 6947,
  CODE_FOR_vec_cmpuv4siv4si = 6948,
  CODE_FOR_vec_cmpuv2div2di = 6949,
  CODE_FOR_vec_cmpeqv2div2di = 6950,
  CODE_FOR_vcondv64qiv16sf = 6951,
  CODE_FOR_vcondv32hiv16sf = 6952,
  CODE_FOR_vcondv16siv16sf = 6953,
  CODE_FOR_vcondv8div16sf = 6954,
  CODE_FOR_vcondv16sfv16sf = 6955,
  CODE_FOR_vcondv8dfv16sf = 6956,
  CODE_FOR_vcondv64qiv8df = 6957,
  CODE_FOR_vcondv32hiv8df = 6958,
  CODE_FOR_vcondv16siv8df = 6959,
  CODE_FOR_vcondv8div8df = 6960,
  CODE_FOR_vcondv16sfv8df = 6961,
  CODE_FOR_vcondv8dfv8df = 6962,
  CODE_FOR_vcondv32qiv8sf = 6963,
  CODE_FOR_vcondv16hiv8sf = 6964,
  CODE_FOR_vcondv8siv8sf = 6965,
  CODE_FOR_vcondv4div8sf = 6966,
  CODE_FOR_vcondv8sfv8sf = 6967,
  CODE_FOR_vcondv4dfv8sf = 6968,
  CODE_FOR_vcondv32qiv4df = 6969,
  CODE_FOR_vcondv16hiv4df = 6970,
  CODE_FOR_vcondv8siv4df = 6971,
  CODE_FOR_vcondv4div4df = 6972,
  CODE_FOR_vcondv8sfv4df = 6973,
  CODE_FOR_vcondv4dfv4df = 6974,
  CODE_FOR_vcondv16qiv4sf = 6975,
  CODE_FOR_vcondv16qiv2df = 6976,
  CODE_FOR_vcondv8hiv4sf = 6977,
  CODE_FOR_vcondv8hiv2df = 6978,
  CODE_FOR_vcondv4siv4sf = 6979,
  CODE_FOR_vcondv4siv2df = 6980,
  CODE_FOR_vcondv2div4sf = 6981,
  CODE_FOR_vcondv2div2df = 6982,
  CODE_FOR_vcondv4sfv4sf = 6983,
  CODE_FOR_vcondv4sfv2df = 6984,
  CODE_FOR_vcondv2dfv4sf = 6985,
  CODE_FOR_vcondv2dfv2df = 6986,
  CODE_FOR_vcond_mask_v16sihi = 6987,
  CODE_FOR_vcond_mask_v8siqi = 6988,
  CODE_FOR_vcond_mask_v4siqi = 6989,
  CODE_FOR_vcond_mask_v8diqi = 6990,
  CODE_FOR_vcond_mask_v4diqi = 6991,
  CODE_FOR_vcond_mask_v2diqi = 6992,
  CODE_FOR_vcond_mask_v16sfhi = 6993,
  CODE_FOR_vcond_mask_v8sfqi = 6994,
  CODE_FOR_vcond_mask_v4sfqi = 6995,
  CODE_FOR_vcond_mask_v8dfqi = 6996,
  CODE_FOR_vcond_mask_v4dfqi = 6997,
  CODE_FOR_vcond_mask_v2dfqi = 6998,
  CODE_FOR_vcond_mask_v64qidi = 6999,
  CODE_FOR_vcond_mask_v16qihi = 7000,
  CODE_FOR_vcond_mask_v32qisi = 7001,
  CODE_FOR_vcond_mask_v32hisi = 7002,
  CODE_FOR_vcond_mask_v16hihi = 7003,
  CODE_FOR_vcond_mask_v8hiqi = 7004,
  CODE_FOR_vcond_mask_v32qiv32qi = 7005,
  CODE_FOR_vcond_mask_v16hiv16hi = 7006,
  CODE_FOR_vcond_mask_v8siv8si = 7007,
  CODE_FOR_vcond_mask_v4div4di = 7008,
  CODE_FOR_vcond_mask_v16qiv16qi = 7009,
  CODE_FOR_vcond_mask_v8hiv8hi = 7010,
  CODE_FOR_vcond_mask_v4siv4si = 7011,
  CODE_FOR_vcond_mask_v2div2di = 7012,
  CODE_FOR_vcond_mask_v8sfv8si = 7013,
  CODE_FOR_vcond_mask_v4dfv4di = 7014,
  CODE_FOR_vcond_mask_v4sfv4si = 7015,
  CODE_FOR_vcond_mask_v2dfv2di = 7016,
  CODE_FOR_andv8sf3 = 7017,
  CODE_FOR_andv8sf3_mask = 7018,
  CODE_FOR_iorv8sf3 = 7019,
  CODE_FOR_iorv8sf3_mask = 7020,
  CODE_FOR_xorv8sf3 = 7021,
  CODE_FOR_xorv8sf3_mask = 7022,
  CODE_FOR_andv4sf3 = 7023,
  CODE_FOR_andv4sf3_mask = 7024,
  CODE_FOR_iorv4sf3 = 7025,
  CODE_FOR_iorv4sf3_mask = 7026,
  CODE_FOR_xorv4sf3 = 7027,
  CODE_FOR_xorv4sf3_mask = 7028,
  CODE_FOR_andv4df3 = 7029,
  CODE_FOR_andv4df3_mask = 7030,
  CODE_FOR_iorv4df3 = 7031,
  CODE_FOR_iorv4df3_mask = 7032,
  CODE_FOR_xorv4df3 = 7033,
  CODE_FOR_xorv4df3_mask = 7034,
  CODE_FOR_andv2df3 = 7035,
  CODE_FOR_andv2df3_mask = 7036,
  CODE_FOR_iorv2df3 = 7037,
  CODE_FOR_iorv2df3_mask = 7038,
  CODE_FOR_xorv2df3 = 7039,
  CODE_FOR_xorv2df3_mask = 7040,
  CODE_FOR_andv16sf3 = 7041,
  CODE_FOR_andv16sf3_mask = 7042,
  CODE_FOR_iorv16sf3 = 7043,
  CODE_FOR_iorv16sf3_mask = 7044,
  CODE_FOR_xorv16sf3 = 7045,
  CODE_FOR_xorv16sf3_mask = 7046,
  CODE_FOR_andv8df3 = 7047,
  CODE_FOR_andv8df3_mask = 7048,
  CODE_FOR_iorv8df3 = 7049,
  CODE_FOR_iorv8df3_mask = 7050,
  CODE_FOR_xorv8df3 = 7051,
  CODE_FOR_xorv8df3_mask = 7052,
  CODE_FOR_copysignv16sf3 = 7053,
  CODE_FOR_copysignv8sf3 = 7054,
  CODE_FOR_copysignv4sf3 = 7055,
  CODE_FOR_copysignv8df3 = 7056,
  CODE_FOR_copysignv4df3 = 7057,
  CODE_FOR_copysignv2df3 = 7058,
  CODE_FOR_xorsignv16sf3 = 7059,
  CODE_FOR_xorsignv8sf3 = 7060,
  CODE_FOR_xorsignv4sf3 = 7061,
  CODE_FOR_xorsignv8df3 = 7062,
  CODE_FOR_xorsignv4df3 = 7063,
  CODE_FOR_xorsignv2df3 = 7064,
  CODE_FOR_signbitv16sf2 = 7065,
  CODE_FOR_signbitv8sf2 = 7066,
  CODE_FOR_signbitv4sf2 = 7067,
  CODE_FOR_andtf3 = 7068,
  CODE_FOR_iortf3 = 7069,
  CODE_FOR_xortf3 = 7070,
  CODE_FOR_fmasf4 = 7071,
  CODE_FOR_fmadf4 = 7072,
  CODE_FOR_fmav4sf4 = 7073,
  CODE_FOR_fmav2df4 = 7074,
  CODE_FOR_fmav8sf4 = 7075,
  CODE_FOR_fmav4df4 = 7076,
  CODE_FOR_fmav16sf4 = 7077,
  CODE_FOR_fmav8df4 = 7078,
  CODE_FOR_fmssf4 = 7079,
  CODE_FOR_fmsdf4 = 7080,
  CODE_FOR_fmsv4sf4 = 7081,
  CODE_FOR_fmsv2df4 = 7082,
  CODE_FOR_fmsv8sf4 = 7083,
  CODE_FOR_fmsv4df4 = 7084,
  CODE_FOR_fmsv16sf4 = 7085,
  CODE_FOR_fmsv8df4 = 7086,
  CODE_FOR_fnmasf4 = 7087,
  CODE_FOR_fnmadf4 = 7088,
  CODE_FOR_fnmav4sf4 = 7089,
  CODE_FOR_fnmav2df4 = 7090,
  CODE_FOR_fnmav8sf4 = 7091,
  CODE_FOR_fnmav4df4 = 7092,
  CODE_FOR_fnmav16sf4 = 7093,
  CODE_FOR_fnmav8df4 = 7094,
  CODE_FOR_fnmssf4 = 7095,
  CODE_FOR_fnmsdf4 = 7096,
  CODE_FOR_fnmsv4sf4 = 7097,
  CODE_FOR_fnmsv2df4 = 7098,
  CODE_FOR_fnmsv8sf4 = 7099,
  CODE_FOR_fnmsv4df4 = 7100,
  CODE_FOR_fnmsv16sf4 = 7101,
  CODE_FOR_fnmsv8df4 = 7102,
  CODE_FOR_fma4i_fmadd_sf = 7103,
  CODE_FOR_fma4i_fmadd_df = 7104,
  CODE_FOR_fma4i_fmadd_v4sf = 7105,
  CODE_FOR_fma4i_fmadd_v2df = 7106,
  CODE_FOR_fma4i_fmadd_v8sf = 7107,
  CODE_FOR_fma4i_fmadd_v4df = 7108,
  CODE_FOR_fma4i_fmadd_v16sf = 7109,
  CODE_FOR_fma4i_fmadd_v8df = 7110,
  CODE_FOR_fma4i_fmsub_sf = 7111,
  CODE_FOR_fma4i_fmsub_df = 7112,
  CODE_FOR_fma4i_fmsub_v4sf = 7113,
  CODE_FOR_fma4i_fmsub_v2df = 7114,
  CODE_FOR_fma4i_fmsub_v8sf = 7115,
  CODE_FOR_fma4i_fmsub_v4df = 7116,
  CODE_FOR_fma4i_fmsub_v16sf = 7117,
  CODE_FOR_fma4i_fmsub_v8df = 7118,
  CODE_FOR_fma4i_fnmadd_sf = 7119,
  CODE_FOR_fma4i_fnmadd_df = 7120,
  CODE_FOR_fma4i_fnmadd_v4sf = 7121,
  CODE_FOR_fma4i_fnmadd_v2df = 7122,
  CODE_FOR_fma4i_fnmadd_v8sf = 7123,
  CODE_FOR_fma4i_fnmadd_v4df = 7124,
  CODE_FOR_fma4i_fnmadd_v16sf = 7125,
  CODE_FOR_fma4i_fnmadd_v8df = 7126,
  CODE_FOR_fma4i_fnmsub_sf = 7127,
  CODE_FOR_fma4i_fnmsub_df = 7128,
  CODE_FOR_fma4i_fnmsub_v4sf = 7129,
  CODE_FOR_fma4i_fnmsub_v2df = 7130,
  CODE_FOR_fma4i_fnmsub_v8sf = 7131,
  CODE_FOR_fma4i_fnmsub_v4df = 7132,
  CODE_FOR_fma4i_fnmsub_v16sf = 7133,
  CODE_FOR_fma4i_fnmsub_v8df = 7134,
  CODE_FOR_avx512f_fmadd_v16sf_maskz = 7135,
  CODE_FOR_avx512f_fmadd_v16sf_maskz_round = 7136,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz = 7137,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz_round = 7138,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz = 7139,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz_round = 7140,
  CODE_FOR_avx512f_fmadd_v8df_maskz = 7141,
  CODE_FOR_avx512f_fmadd_v8df_maskz_round = 7142,
  CODE_FOR_avx512vl_fmadd_v4df_maskz = 7143,
  CODE_FOR_avx512vl_fmadd_v4df_maskz_round = 7144,
  CODE_FOR_avx512vl_fmadd_v2df_maskz = 7145,
  CODE_FOR_avx512vl_fmadd_v2df_maskz_round = 7146,
  CODE_FOR_avx512f_fmsub_v16sf_maskz = 7147,
  CODE_FOR_avx512f_fmsub_v16sf_maskz_round = 7148,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz = 7149,
  CODE_FOR_avx512vl_fmsub_v8sf_maskz_round = 7150,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz = 7151,
  CODE_FOR_avx512vl_fmsub_v4sf_maskz_round = 7152,
  CODE_FOR_avx512f_fmsub_v8df_maskz = 7153,
  CODE_FOR_avx512f_fmsub_v8df_maskz_round = 7154,
  CODE_FOR_avx512vl_fmsub_v4df_maskz = 7155,
  CODE_FOR_avx512vl_fmsub_v4df_maskz_round = 7156,
  CODE_FOR_avx512vl_fmsub_v2df_maskz = 7157,
  CODE_FOR_avx512vl_fmsub_v2df_maskz_round = 7158,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz = 7159,
  CODE_FOR_avx512f_fnmadd_v16sf_maskz_round = 7160,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz = 7161,
  CODE_FOR_avx512vl_fnmadd_v8sf_maskz_round = 7162,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz = 7163,
  CODE_FOR_avx512vl_fnmadd_v4sf_maskz_round = 7164,
  CODE_FOR_avx512f_fnmadd_v8df_maskz = 7165,
  CODE_FOR_avx512f_fnmadd_v8df_maskz_round = 7166,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz = 7167,
  CODE_FOR_avx512vl_fnmadd_v4df_maskz_round = 7168,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz = 7169,
  CODE_FOR_avx512vl_fnmadd_v2df_maskz_round = 7170,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz = 7171,
  CODE_FOR_avx512f_fnmsub_v16sf_maskz_round = 7172,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz = 7173,
  CODE_FOR_avx512vl_fnmsub_v8sf_maskz_round = 7174,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz = 7175,
  CODE_FOR_avx512vl_fnmsub_v4sf_maskz_round = 7176,
  CODE_FOR_avx512f_fnmsub_v8df_maskz = 7177,
  CODE_FOR_avx512f_fnmsub_v8df_maskz_round = 7178,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz = 7179,
  CODE_FOR_avx512vl_fnmsub_v4df_maskz_round = 7180,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz = 7181,
  CODE_FOR_avx512vl_fnmsub_v2df_maskz_round = 7182,
  CODE_FOR_fmaddsub_v16sf = 7183,
  CODE_FOR_fmaddsub_v8sf = 7184,
  CODE_FOR_fmaddsub_v4sf = 7185,
  CODE_FOR_fmaddsub_v8df = 7186,
  CODE_FOR_fmaddsub_v4df = 7187,
  CODE_FOR_fmaddsub_v2df = 7188,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz = 7189,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz_round = 7190,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz = 7191,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz_round = 7192,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz = 7193,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz_round = 7194,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz = 7195,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz_round = 7196,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz = 7197,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz_round = 7198,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz = 7199,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz_round = 7200,
  CODE_FOR_fmai_vmfmadd_v4sf = 7201,
  CODE_FOR_fmai_vmfmadd_v4sf_round = 7202,
  CODE_FOR_fmai_vmfmadd_v2df = 7203,
  CODE_FOR_fmai_vmfmadd_v2df_round = 7204,
  CODE_FOR_fmai_vmfmsub_v4sf = 7205,
  CODE_FOR_fmai_vmfmsub_v4sf_round = 7206,
  CODE_FOR_fmai_vmfmsub_v2df = 7207,
  CODE_FOR_fmai_vmfmsub_v2df_round = 7208,
  CODE_FOR_fmai_vmfnmadd_v4sf = 7209,
  CODE_FOR_fmai_vmfnmadd_v4sf_round = 7210,
  CODE_FOR_fmai_vmfnmadd_v2df = 7211,
  CODE_FOR_fmai_vmfnmadd_v2df_round = 7212,
  CODE_FOR_fmai_vmfnmsub_v4sf = 7213,
  CODE_FOR_fmai_vmfnmsub_v4sf_round = 7214,
  CODE_FOR_fmai_vmfnmsub_v2df = 7215,
  CODE_FOR_fmai_vmfnmsub_v2df_round = 7216,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz = 7217,
  CODE_FOR_avx512f_vmfmadd_v4sf_maskz_round = 7218,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz = 7219,
  CODE_FOR_avx512f_vmfmadd_v2df_maskz_round = 7220,
  CODE_FOR_fma4i_vmfmadd_v4sf = 7221,
  CODE_FOR_fma4i_vmfmadd_v2df = 7222,
  CODE_FOR_floatunsv16siv16sf2 = 7223,
  CODE_FOR_floatunsv8siv8sf2 = 7224,
  CODE_FOR_floatunsv4siv4sf2 = 7225,
  CODE_FOR_fixuns_truncv16sfv16si2 = 7226,
  CODE_FOR_fixuns_truncv8sfv8si2 = 7227,
  CODE_FOR_fixuns_truncv4sfv4si2 = 7228,
  CODE_FOR_floatv2siv2df2 = 7229,
  CODE_FOR_fix_truncv2dfv2si2 = 7230,
  CODE_FOR_avx512dq_floatv2div2sf2 = 7231,
  CODE_FOR_avx512dq_floatunsv2div2sf2 = 7232,
  CODE_FOR_floatv2div2sf2 = 7233,
  CODE_FOR_floatunsv2div2sf2 = 7234,
  CODE_FOR_vec_packs_float_v8di = 7235,
  CODE_FOR_vec_packu_float_v8di = 7236,
  CODE_FOR_vec_packs_float_v4di = 7237,
  CODE_FOR_vec_packu_float_v4di = 7238,
  CODE_FOR_vec_packs_float_v2di = 7239,
  CODE_FOR_vec_packu_float_v2di = 7240,
  CODE_FOR_floatv2div2sf2_mask = 7241,
  CODE_FOR_floatunsv2div2sf2_mask = 7242,
  CODE_FOR_avx_cvtpd2dq256_2 = 7243,
  CODE_FOR_fix_truncv2sfv2di2 = 7244,
  CODE_FOR_fixuns_truncv2sfv2di2 = 7245,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v16sf = 7246,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v16sf = 7247,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v8sf = 7248,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v8sf = 7249,
  CODE_FOR_vec_unpack_sfix_trunc_lo_v4sf = 7250,
  CODE_FOR_vec_unpack_ufix_trunc_lo_v4sf = 7251,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v16sf = 7252,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v16sf = 7253,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v8sf = 7254,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v8sf = 7255,
  CODE_FOR_vec_unpack_sfix_trunc_hi_v4sf = 7256,
  CODE_FOR_vec_unpack_ufix_trunc_hi_v4sf = 7257,
  CODE_FOR_avx_cvttpd2dq256_2 = 7258,
  CODE_FOR_sse2_cvtpd2ps = 7259,
  CODE_FOR_sse2_cvtpd2ps_mask = 7260,
  CODE_FOR_truncv8dfv8sf2 = 7261,
  CODE_FOR_truncv4dfv4sf2 = 7262,
  CODE_FOR_extendv8sfv8df2 = 7263,
  CODE_FOR_extendv4sfv4df2 = 7264,
  CODE_FOR_avx512bw_cvtmask2bv64qi = 7265,
  CODE_FOR_avx512vl_cvtmask2bv16qi = 7266,
  CODE_FOR_avx512vl_cvtmask2bv32qi = 7267,
  CODE_FOR_avx512bw_cvtmask2wv32hi = 7268,
  CODE_FOR_avx512vl_cvtmask2wv16hi = 7269,
  CODE_FOR_avx512vl_cvtmask2wv8hi = 7270,
  CODE_FOR_avx512f_cvtmask2dv16si = 7271,
  CODE_FOR_avx512vl_cvtmask2dv8si = 7272,
  CODE_FOR_avx512vl_cvtmask2dv4si = 7273,
  CODE_FOR_avx512f_cvtmask2qv8di = 7274,
  CODE_FOR_avx512vl_cvtmask2qv4di = 7275,
  CODE_FOR_avx512vl_cvtmask2qv2di = 7276,
  CODE_FOR_vec_unpacks_hi_v4sf = 7277,
  CODE_FOR_vec_unpacks_hi_v8sf = 7278,
  CODE_FOR_vec_unpacks_hi_v16sf = 7279,
  CODE_FOR_vec_unpacks_lo_v4sf = 7280,
  CODE_FOR_vec_unpacks_lo_v8sf = 7281,
  CODE_FOR_vec_unpacks_float_hi_v32hi = 7282,
  CODE_FOR_vec_unpacks_float_hi_v16hi = 7283,
  CODE_FOR_vec_unpacks_float_hi_v8hi = 7284,
  CODE_FOR_vec_unpacks_float_lo_v32hi = 7285,
  CODE_FOR_vec_unpacks_float_lo_v16hi = 7286,
  CODE_FOR_vec_unpacks_float_lo_v8hi = 7287,
  CODE_FOR_vec_unpacku_float_hi_v32hi = 7288,
  CODE_FOR_vec_unpacku_float_hi_v16hi = 7289,
  CODE_FOR_vec_unpacku_float_hi_v8hi = 7290,
  CODE_FOR_vec_unpacku_float_lo_v32hi = 7291,
  CODE_FOR_vec_unpacku_float_lo_v16hi = 7292,
  CODE_FOR_vec_unpacku_float_lo_v8hi = 7293,
  CODE_FOR_vec_unpacks_float_hi_v4si = 7294,
  CODE_FOR_vec_unpacks_float_lo_v4si = 7295,
  CODE_FOR_vec_unpacks_float_hi_v8si = 7296,
  CODE_FOR_vec_unpacks_float_lo_v8si = 7297,
  CODE_FOR_vec_unpacks_float_hi_v16si = 7298,
  CODE_FOR_vec_unpacks_float_lo_v16si = 7299,
  CODE_FOR_vec_unpacku_float_hi_v4si = 7300,
  CODE_FOR_vec_unpacku_float_lo_v4si = 7301,
  CODE_FOR_vec_unpacku_float_hi_v8si = 7302,
  CODE_FOR_vec_unpacku_float_hi_v16si = 7303,
  CODE_FOR_vec_unpacku_float_lo_v8si = 7304,
  CODE_FOR_vec_unpacku_float_lo_v16si = 7305,
  CODE_FOR_vec_pack_trunc_v8df = 7306,
  CODE_FOR_vec_pack_trunc_v4df = 7307,
  CODE_FOR_vec_pack_trunc_v2df = 7308,
  CODE_FOR_vec_pack_sfix_trunc_v8df = 7309,
  CODE_FOR_vec_pack_sfix_trunc_v4df = 7310,
  CODE_FOR_vec_pack_sfix_trunc_v2df = 7311,
  CODE_FOR_vec_pack_ufix_trunc_v8df = 7312,
  CODE_FOR_vec_pack_ufix_trunc_v4df = 7313,
  CODE_FOR_vec_pack_ufix_trunc_v2df = 7314,
  CODE_FOR_avx512f_vec_pack_sfix_v8df = 7315,
  CODE_FOR_vec_pack_sfix_v4df = 7316,
  CODE_FOR_vec_pack_sfix_v2df = 7317,
  CODE_FOR_sse_movhlps_exp = 7318,
  CODE_FOR_sse_movlhps_exp = 7319,
  CODE_FOR_vec_interleave_highv8sf = 7320,
  CODE_FOR_vec_interleave_lowv8sf = 7321,
  CODE_FOR_avx_shufps256 = 7322,
  CODE_FOR_avx_shufps256_mask = 7323,
  CODE_FOR_sse_shufps = 7324,
  CODE_FOR_sse_shufps_mask = 7325,
  CODE_FOR_sse_loadhps_exp = 7326,
  CODE_FOR_sse_loadlps_exp = 7327,
  CODE_FOR_vec_setv64qi = 7328,
  CODE_FOR_vec_setv32qi = 7329,
  CODE_FOR_vec_setv16qi = 7330,
  CODE_FOR_vec_setv32hi = 7331,
  CODE_FOR_vec_setv16hi = 7332,
  CODE_FOR_vec_setv8hi = 7333,
  CODE_FOR_vec_setv16si = 7334,
  CODE_FOR_vec_setv8si = 7335,
  CODE_FOR_vec_setv4si = 7336,
  CODE_FOR_vec_setv8di = 7337,
  CODE_FOR_vec_setv4di = 7338,
  CODE_FOR_vec_setv2di = 7339,
  CODE_FOR_vec_setv16sf = 7340,
  CODE_FOR_vec_setv8sf = 7341,
  CODE_FOR_vec_setv4sf = 7342,
  CODE_FOR_vec_setv8df = 7343,
  CODE_FOR_vec_setv4df = 7344,
  CODE_FOR_vec_setv2df = 7345,
  CODE_FOR_avx512dq_vextractf64x2_mask = 7346,
  CODE_FOR_avx512dq_vextracti64x2_mask = 7347,
  CODE_FOR_avx512f_vextractf32x4_mask = 7348,
  CODE_FOR_avx512f_vextracti32x4_mask = 7349,
  CODE_FOR_avx512dq_vextractf32x8_mask = 7350,
  CODE_FOR_avx512dq_vextracti32x8_mask = 7351,
  CODE_FOR_avx512f_vextractf64x4_mask = 7352,
  CODE_FOR_avx512f_vextracti64x4_mask = 7353,
  CODE_FOR_avx512vl_vextractf128v8si = 7354,
  CODE_FOR_avx512vl_vextractf128v8sf = 7355,
  CODE_FOR_avx512vl_vextractf128v4di = 7356,
  CODE_FOR_avx512vl_vextractf128v4df = 7357,
  CODE_FOR_avx_vextractf128v32qi = 7358,
  CODE_FOR_avx_vextractf128v16hi = 7359,
  CODE_FOR_avx_vextractf128v8si = 7360,
  CODE_FOR_avx_vextractf128v4di = 7361,
  CODE_FOR_avx_vextractf128v8sf = 7362,
  CODE_FOR_avx_vextractf128v4df = 7363,
  CODE_FOR_vec_extractv64qiqi = 7364,
  CODE_FOR_vec_extractv32qiqi = 7365,
  CODE_FOR_vec_extractv16qiqi = 7366,
  CODE_FOR_vec_extractv32hihi = 7367,
  CODE_FOR_vec_extractv16hihi = 7368,
  CODE_FOR_vec_extractv8hihi = 7369,
  CODE_FOR_vec_extractv16sisi = 7370,
  CODE_FOR_vec_extractv8sisi = 7371,
  CODE_FOR_vec_extractv4sisi = 7372,
  CODE_FOR_vec_extractv8didi = 7373,
  CODE_FOR_vec_extractv4didi = 7374,
  CODE_FOR_vec_extractv2didi = 7375,
  CODE_FOR_vec_extractv16sfsf = 7376,
  CODE_FOR_vec_extractv8sfsf = 7377,
  CODE_FOR_vec_extractv4sfsf = 7378,
  CODE_FOR_vec_extractv8dfdf = 7379,
  CODE_FOR_vec_extractv4dfdf = 7380,
  CODE_FOR_vec_extractv2dfdf = 7381,
  CODE_FOR_vec_extractv4titi = 7382,
  CODE_FOR_vec_extractv2titi = 7383,
  CODE_FOR_vec_extractv32qiv16qi = 7384,
  CODE_FOR_vec_extractv16hiv8hi = 7385,
  CODE_FOR_vec_extractv8siv4si = 7386,
  CODE_FOR_vec_extractv4div2di = 7387,
  CODE_FOR_vec_extractv8sfv4sf = 7388,
  CODE_FOR_vec_extractv4dfv2df = 7389,
  CODE_FOR_vec_extractv64qiv32qi = 7390,
  CODE_FOR_vec_extractv32hiv16hi = 7391,
  CODE_FOR_vec_extractv16siv8si = 7392,
  CODE_FOR_vec_extractv8div4di = 7393,
  CODE_FOR_vec_extractv16sfv8sf = 7394,
  CODE_FOR_vec_extractv8dfv4df = 7395,
  CODE_FOR_vec_interleave_highv4df = 7396,
  CODE_FOR_vec_interleave_highv2df = 7397,
  CODE_FOR_avx512f_movddup512 = 7398,
  CODE_FOR_avx512f_movddup512_mask = 7399,
  CODE_FOR_avx512f_unpcklpd512 = 7400,
  CODE_FOR_avx512f_unpcklpd512_mask = 7401,
  CODE_FOR_avx_movddup256 = 7402,
  CODE_FOR_avx_movddup256_mask = 7403,
  CODE_FOR_avx_unpcklpd256 = 7404,
  CODE_FOR_avx_unpcklpd256_mask = 7405,
  CODE_FOR_vec_interleave_lowv4df = 7406,
  CODE_FOR_vec_interleave_lowv2df = 7407,
  CODE_FOR_avx512f_vternlogv16si_maskz = 7408,
  CODE_FOR_avx512vl_vternlogv8si_maskz = 7409,
  CODE_FOR_avx512vl_vternlogv4si_maskz = 7410,
  CODE_FOR_avx512f_vternlogv8di_maskz = 7411,
  CODE_FOR_avx512vl_vternlogv4di_maskz = 7412,
  CODE_FOR_avx512vl_vternlogv2di_maskz = 7413,
  CODE_FOR_avx512f_shufps512_mask = 7414,
  CODE_FOR_avx512f_fixupimmv16sf_maskz = 7415,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_round = 7416,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz = 7417,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_round = 7418,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz = 7419,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_round = 7420,
  CODE_FOR_avx512f_fixupimmv8df_maskz = 7421,
  CODE_FOR_avx512f_fixupimmv8df_maskz_round = 7422,
  CODE_FOR_avx512vl_fixupimmv4df_maskz = 7423,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_round = 7424,
  CODE_FOR_avx512vl_fixupimmv2df_maskz = 7425,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_round = 7426,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz = 7427,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_round = 7428,
  CODE_FOR_avx512f_sfixupimmv2df_maskz = 7429,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_round = 7430,
  CODE_FOR_avx512f_shufpd512_mask = 7431,
  CODE_FOR_avx_shufpd256 = 7432,
  CODE_FOR_avx_shufpd256_mask = 7433,
  CODE_FOR_sse2_shufpd = 7434,
  CODE_FOR_sse2_shufpd_mask = 7435,
  CODE_FOR_sse2_loadhpd_exp = 7436,
  CODE_FOR_sse2_loadlpd_exp = 7437,
  CODE_FOR_truncv16siv16qi2 = 7438,
  CODE_FOR_truncv16siv16hi2 = 7439,
  CODE_FOR_truncv8div8si2 = 7440,
  CODE_FOR_truncv8div8hi2 = 7441,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask_store = 7442,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask_store = 7443,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask_store = 7444,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask_store = 7445,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask_store = 7446,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask_store = 7447,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask_store = 7448,
  CODE_FOR_avx512f_truncatev8div8si2_mask_store = 7449,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask_store = 7450,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask_store = 7451,
  CODE_FOR_avx512f_truncatev8div8hi2_mask_store = 7452,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask_store = 7453,
  CODE_FOR_truncv32hiv32qi2 = 7454,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask_store = 7455,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask_store = 7456,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask_store = 7457,
  CODE_FOR_truncv4div4si2 = 7458,
  CODE_FOR_truncv8siv8hi2 = 7459,
  CODE_FOR_truncv16hiv16qi2 = 7460,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask_store = 7461,
  CODE_FOR_avx512vl_truncatev4div4si2_mask_store = 7462,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask_store = 7463,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask_store = 7464,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask_store = 7465,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask_store = 7466,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask_store = 7467,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask_store = 7468,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask_store = 7469,
  CODE_FOR_truncv4div4qi2 = 7470,
  CODE_FOR_truncv2div2qi2 = 7471,
  CODE_FOR_truncv8siv8qi2 = 7472,
  CODE_FOR_truncv4siv4qi2 = 7473,
  CODE_FOR_truncv8hiv8qi2 = 7474,
  CODE_FOR_truncv4diV4HI2 = 7475,
  CODE_FOR_truncv2diV2HI2 = 7476,
  CODE_FOR_truncv4siV4HI2 = 7477,
  CODE_FOR_truncv2div2si2 = 7478,
  CODE_FOR_truncv8div8qi2 = 7479,
  CODE_FOR_negv64qi2 = 7480,
  CODE_FOR_negv32qi2 = 7481,
  CODE_FOR_negv16qi2 = 7482,
  CODE_FOR_negv32hi2 = 7483,
  CODE_FOR_negv16hi2 = 7484,
  CODE_FOR_negv8hi2 = 7485,
  CODE_FOR_negv16si2 = 7486,
  CODE_FOR_negv8si2 = 7487,
  CODE_FOR_negv4si2 = 7488,
  CODE_FOR_negv8di2 = 7489,
  CODE_FOR_negv4di2 = 7490,
  CODE_FOR_negv2di2 = 7491,
  CODE_FOR_addv64qi3 = 7492,
  CODE_FOR_subv64qi3 = 7493,
  CODE_FOR_addv32qi3 = 7494,
  CODE_FOR_subv32qi3 = 7495,
  CODE_FOR_addv16qi3 = 7496,
  CODE_FOR_subv16qi3 = 7497,
  CODE_FOR_addv32hi3 = 7498,
  CODE_FOR_subv32hi3 = 7499,
  CODE_FOR_addv16hi3 = 7500,
  CODE_FOR_subv16hi3 = 7501,
  CODE_FOR_addv8hi3 = 7502,
  CODE_FOR_subv8hi3 = 7503,
  CODE_FOR_addv16si3 = 7504,
  CODE_FOR_subv16si3 = 7505,
  CODE_FOR_addv8si3 = 7506,
  CODE_FOR_subv8si3 = 7507,
  CODE_FOR_addv4si3 = 7508,
  CODE_FOR_subv4si3 = 7509,
  CODE_FOR_addv8di3 = 7510,
  CODE_FOR_subv8di3 = 7511,
  CODE_FOR_addv4di3 = 7512,
  CODE_FOR_subv4di3 = 7513,
  CODE_FOR_addv2di3 = 7514,
  CODE_FOR_subv2di3 = 7515,
  CODE_FOR_addv16si3_mask = 7516,
  CODE_FOR_subv16si3_mask = 7517,
  CODE_FOR_addv8si3_mask = 7518,
  CODE_FOR_subv8si3_mask = 7519,
  CODE_FOR_addv4si3_mask = 7520,
  CODE_FOR_subv4si3_mask = 7521,
  CODE_FOR_addv8di3_mask = 7522,
  CODE_FOR_subv8di3_mask = 7523,
  CODE_FOR_addv4di3_mask = 7524,
  CODE_FOR_subv4di3_mask = 7525,
  CODE_FOR_addv2di3_mask = 7526,
  CODE_FOR_subv2di3_mask = 7527,
  CODE_FOR_addv64qi3_mask = 7528,
  CODE_FOR_subv64qi3_mask = 7529,
  CODE_FOR_addv16qi3_mask = 7530,
  CODE_FOR_subv16qi3_mask = 7531,
  CODE_FOR_addv32qi3_mask = 7532,
  CODE_FOR_subv32qi3_mask = 7533,
  CODE_FOR_addv32hi3_mask = 7534,
  CODE_FOR_subv32hi3_mask = 7535,
  CODE_FOR_addv16hi3_mask = 7536,
  CODE_FOR_subv16hi3_mask = 7537,
  CODE_FOR_addv8hi3_mask = 7538,
  CODE_FOR_subv8hi3_mask = 7539,
  CODE_FOR_avx512bw_ssaddv64qi3 = 7540,
  CODE_FOR_avx512bw_ssaddv64qi3_mask = 7541,
  CODE_FOR_avx512bw_usaddv64qi3 = 7542,
  CODE_FOR_avx512bw_usaddv64qi3_mask = 7543,
  CODE_FOR_avx512bw_sssubv64qi3 = 7544,
  CODE_FOR_avx512bw_sssubv64qi3_mask = 7545,
  CODE_FOR_avx512bw_ussubv64qi3 = 7546,
  CODE_FOR_avx512bw_ussubv64qi3_mask = 7547,
  CODE_FOR_avx2_ssaddv32qi3 = 7548,
  CODE_FOR_avx2_ssaddv32qi3_mask = 7549,
  CODE_FOR_avx2_usaddv32qi3 = 7550,
  CODE_FOR_avx2_usaddv32qi3_mask = 7551,
  CODE_FOR_avx2_sssubv32qi3 = 7552,
  CODE_FOR_avx2_sssubv32qi3_mask = 7553,
  CODE_FOR_avx2_ussubv32qi3 = 7554,
  CODE_FOR_avx2_ussubv32qi3_mask = 7555,
  CODE_FOR_sse2_ssaddv16qi3 = 7556,
  CODE_FOR_sse2_ssaddv16qi3_mask = 7557,
  CODE_FOR_sse2_usaddv16qi3 = 7558,
  CODE_FOR_sse2_usaddv16qi3_mask = 7559,
  CODE_FOR_sse2_sssubv16qi3 = 7560,
  CODE_FOR_sse2_sssubv16qi3_mask = 7561,
  CODE_FOR_sse2_ussubv16qi3 = 7562,
  CODE_FOR_sse2_ussubv16qi3_mask = 7563,
  CODE_FOR_avx512bw_ssaddv32hi3 = 7564,
  CODE_FOR_avx512bw_ssaddv32hi3_mask = 7565,
  CODE_FOR_avx512bw_usaddv32hi3 = 7566,
  CODE_FOR_avx512bw_usaddv32hi3_mask = 7567,
  CODE_FOR_avx512bw_sssubv32hi3 = 7568,
  CODE_FOR_avx512bw_sssubv32hi3_mask = 7569,
  CODE_FOR_avx512bw_ussubv32hi3 = 7570,
  CODE_FOR_avx512bw_ussubv32hi3_mask = 7571,
  CODE_FOR_avx2_ssaddv16hi3 = 7572,
  CODE_FOR_avx2_ssaddv16hi3_mask = 7573,
  CODE_FOR_avx2_usaddv16hi3 = 7574,
  CODE_FOR_avx2_usaddv16hi3_mask = 7575,
  CODE_FOR_avx2_sssubv16hi3 = 7576,
  CODE_FOR_avx2_sssubv16hi3_mask = 7577,
  CODE_FOR_avx2_ussubv16hi3 = 7578,
  CODE_FOR_avx2_ussubv16hi3_mask = 7579,
  CODE_FOR_sse2_ssaddv8hi3 = 7580,
  CODE_FOR_sse2_ssaddv8hi3_mask = 7581,
  CODE_FOR_sse2_usaddv8hi3 = 7582,
  CODE_FOR_sse2_usaddv8hi3_mask = 7583,
  CODE_FOR_sse2_sssubv8hi3 = 7584,
  CODE_FOR_sse2_sssubv8hi3_mask = 7585,
  CODE_FOR_sse2_ussubv8hi3 = 7586,
  CODE_FOR_sse2_ussubv8hi3_mask = 7587,
  CODE_FOR_mulv8qi3 = 7588,
  CODE_FOR_mulv64qi3 = 7589,
  CODE_FOR_mulv32qi3 = 7590,
  CODE_FOR_mulv16qi3 = 7591,
  CODE_FOR_mulv32hi3 = 7592,
  CODE_FOR_mulv32hi3_mask = 7593,
  CODE_FOR_mulv16hi3 = 7594,
  CODE_FOR_mulv16hi3_mask = 7595,
  CODE_FOR_mulv8hi3 = 7596,
  CODE_FOR_mulv8hi3_mask = 7597,
  CODE_FOR_smulv32hi3_highpart = 7598,
  CODE_FOR_smulv32hi3_highpart_mask = 7599,
  CODE_FOR_umulv32hi3_highpart = 7600,
  CODE_FOR_umulv32hi3_highpart_mask = 7601,
  CODE_FOR_smulv16hi3_highpart = 7602,
  CODE_FOR_smulv16hi3_highpart_mask = 7603,
  CODE_FOR_umulv16hi3_highpart = 7604,
  CODE_FOR_umulv16hi3_highpart_mask = 7605,
  CODE_FOR_smulv8hi3_highpart = 7606,
  CODE_FOR_smulv8hi3_highpart_mask = 7607,
  CODE_FOR_umulv8hi3_highpart = 7608,
  CODE_FOR_umulv8hi3_highpart_mask = 7609,
  CODE_FOR_vec_widen_umult_even_v16si = 7610,
  CODE_FOR_vec_widen_umult_even_v16si_mask = 7611,
  CODE_FOR_vec_widen_umult_even_v8si = 7612,
  CODE_FOR_vec_widen_umult_even_v8si_mask = 7613,
  CODE_FOR_vec_widen_umult_even_v4si = 7614,
  CODE_FOR_vec_widen_umult_even_v4si_mask = 7615,
  CODE_FOR_vec_widen_smult_even_v16si = 7616,
  CODE_FOR_vec_widen_smult_even_v16si_mask = 7617,
  CODE_FOR_vec_widen_smult_even_v8si = 7618,
  CODE_FOR_vec_widen_smult_even_v8si_mask = 7619,
  CODE_FOR_sse4_1_mulv2siv2di3 = 7620,
  CODE_FOR_sse4_1_mulv2siv2di3_mask = 7621,
  CODE_FOR_avx2_pmaddwd = 7622,
  CODE_FOR_sse2_pmaddwd = 7623,
  CODE_FOR_mulv16si3 = 7624,
  CODE_FOR_mulv16si3_mask = 7625,
  CODE_FOR_mulv8si3 = 7626,
  CODE_FOR_mulv8si3_mask = 7627,
  CODE_FOR_mulv4si3 = 7628,
  CODE_FOR_mulv4si3_mask = 7629,
  CODE_FOR_mulv8di3 = 7630,
  CODE_FOR_mulv4di3 = 7631,
  CODE_FOR_mulv2di3 = 7632,
  CODE_FOR_vec_widen_smult_hi_v32qi = 7633,
  CODE_FOR_vec_widen_umult_hi_v32qi = 7634,
  CODE_FOR_vec_widen_smult_hi_v16qi = 7635,
  CODE_FOR_vec_widen_umult_hi_v16qi = 7636,
  CODE_FOR_vec_widen_smult_hi_v16hi = 7637,
  CODE_FOR_vec_widen_umult_hi_v16hi = 7638,
  CODE_FOR_vec_widen_smult_hi_v8hi = 7639,
  CODE_FOR_vec_widen_umult_hi_v8hi = 7640,
  CODE_FOR_vec_widen_smult_hi_v8si = 7641,
  CODE_FOR_vec_widen_umult_hi_v8si = 7642,
  CODE_FOR_vec_widen_smult_hi_v4si = 7643,
  CODE_FOR_vec_widen_umult_hi_v4si = 7644,
  CODE_FOR_vec_widen_smult_lo_v32qi = 7645,
  CODE_FOR_vec_widen_umult_lo_v32qi = 7646,
  CODE_FOR_vec_widen_smult_lo_v16qi = 7647,
  CODE_FOR_vec_widen_umult_lo_v16qi = 7648,
  CODE_FOR_vec_widen_smult_lo_v16hi = 7649,
  CODE_FOR_vec_widen_umult_lo_v16hi = 7650,
  CODE_FOR_vec_widen_smult_lo_v8hi = 7651,
  CODE_FOR_vec_widen_umult_lo_v8hi = 7652,
  CODE_FOR_vec_widen_smult_lo_v8si = 7653,
  CODE_FOR_vec_widen_umult_lo_v8si = 7654,
  CODE_FOR_vec_widen_smult_lo_v4si = 7655,
  CODE_FOR_vec_widen_umult_lo_v4si = 7656,
  CODE_FOR_vec_widen_smult_even_v4si = 7657,
  CODE_FOR_vec_widen_smult_odd_v16si = 7658,
  CODE_FOR_vec_widen_umult_odd_v16si = 7659,
  CODE_FOR_vec_widen_smult_odd_v8si = 7660,
  CODE_FOR_vec_widen_umult_odd_v8si = 7661,
  CODE_FOR_vec_widen_smult_odd_v4si = 7662,
  CODE_FOR_vec_widen_umult_odd_v4si = 7663,
  CODE_FOR_sdot_prodv32hi = 7664,
  CODE_FOR_sdot_prodv16hi = 7665,
  CODE_FOR_sdot_prodv8hi = 7666,
  CODE_FOR_sdot_prodv4si = 7667,
  CODE_FOR_uavgv64qi3_ceil = 7668,
  CODE_FOR_uavgv32qi3_ceil = 7669,
  CODE_FOR_uavgv16qi3_ceil = 7670,
  CODE_FOR_uavgv32hi3_ceil = 7671,
  CODE_FOR_uavgv16hi3_ceil = 7672,
  CODE_FOR_uavgv8hi3_ceil = 7673,
  CODE_FOR_usadv16qi = 7674,
  CODE_FOR_usadv32qi = 7675,
  CODE_FOR_usadv64qi = 7676,
  CODE_FOR_vec_shl_v16qi = 7677,
  CODE_FOR_vec_shl_v8hi = 7678,
  CODE_FOR_vec_shl_v4si = 7679,
  CODE_FOR_vec_shl_v2di = 7680,
  CODE_FOR_vec_shl_v4sf = 7681,
  CODE_FOR_vec_shl_v2df = 7682,
  CODE_FOR_vec_shr_v16qi = 7683,
  CODE_FOR_vec_shr_v8hi = 7684,
  CODE_FOR_vec_shr_v4si = 7685,
  CODE_FOR_vec_shr_v2di = 7686,
  CODE_FOR_vec_shr_v4sf = 7687,
  CODE_FOR_vec_shr_v2df = 7688,
  CODE_FOR_smaxv32qi3 = 7689,
  CODE_FOR_sminv32qi3 = 7690,
  CODE_FOR_umaxv32qi3 = 7691,
  CODE_FOR_uminv32qi3 = 7692,
  CODE_FOR_smaxv16hi3 = 7693,
  CODE_FOR_sminv16hi3 = 7694,
  CODE_FOR_umaxv16hi3 = 7695,
  CODE_FOR_uminv16hi3 = 7696,
  CODE_FOR_smaxv8si3 = 7697,
  CODE_FOR_sminv8si3 = 7698,
  CODE_FOR_umaxv8si3 = 7699,
  CODE_FOR_uminv8si3 = 7700,
  CODE_FOR_smaxv64qi3 = 7701,
  CODE_FOR_sminv64qi3 = 7702,
  CODE_FOR_umaxv64qi3 = 7703,
  CODE_FOR_uminv64qi3 = 7704,
  CODE_FOR_smaxv32hi3 = 7705,
  CODE_FOR_sminv32hi3 = 7706,
  CODE_FOR_umaxv32hi3 = 7707,
  CODE_FOR_uminv32hi3 = 7708,
  CODE_FOR_smaxv16si3 = 7709,
  CODE_FOR_sminv16si3 = 7710,
  CODE_FOR_umaxv16si3 = 7711,
  CODE_FOR_uminv16si3 = 7712,
  CODE_FOR_smaxv16si3_mask = 7713,
  CODE_FOR_sminv16si3_mask = 7714,
  CODE_FOR_umaxv16si3_mask = 7715,
  CODE_FOR_uminv16si3_mask = 7716,
  CODE_FOR_smaxv8si3_mask = 7717,
  CODE_FOR_sminv8si3_mask = 7718,
  CODE_FOR_umaxv8si3_mask = 7719,
  CODE_FOR_uminv8si3_mask = 7720,
  CODE_FOR_smaxv4si3_mask = 7721,
  CODE_FOR_sminv4si3_mask = 7722,
  CODE_FOR_umaxv4si3_mask = 7723,
  CODE_FOR_uminv4si3_mask = 7724,
  CODE_FOR_smaxv8di3_mask = 7725,
  CODE_FOR_sminv8di3_mask = 7726,
  CODE_FOR_umaxv8di3_mask = 7727,
  CODE_FOR_uminv8di3_mask = 7728,
  CODE_FOR_smaxv4di3_mask = 7729,
  CODE_FOR_sminv4di3_mask = 7730,
  CODE_FOR_umaxv4di3_mask = 7731,
  CODE_FOR_uminv4di3_mask = 7732,
  CODE_FOR_smaxv2di3_mask = 7733,
  CODE_FOR_sminv2di3_mask = 7734,
  CODE_FOR_umaxv2di3_mask = 7735,
  CODE_FOR_uminv2di3_mask = 7736,
  CODE_FOR_smaxv8di3 = 7737,
  CODE_FOR_sminv8di3 = 7738,
  CODE_FOR_umaxv8di3 = 7739,
  CODE_FOR_uminv8di3 = 7740,
  CODE_FOR_smaxv4di3 = 7741,
  CODE_FOR_sminv4di3 = 7742,
  CODE_FOR_umaxv4di3 = 7743,
  CODE_FOR_uminv4di3 = 7744,
  CODE_FOR_smaxv2di3 = 7745,
  CODE_FOR_sminv2di3 = 7746,
  CODE_FOR_umaxv2di3 = 7747,
  CODE_FOR_uminv2di3 = 7748,
  CODE_FOR_smaxv16qi3 = 7749,
  CODE_FOR_sminv16qi3 = 7750,
  CODE_FOR_smaxv8hi3 = 7751,
  CODE_FOR_sminv8hi3 = 7752,
  CODE_FOR_smaxv4si3 = 7753,
  CODE_FOR_sminv4si3 = 7754,
  CODE_FOR_umaxv16qi3 = 7755,
  CODE_FOR_uminv16qi3 = 7756,
  CODE_FOR_umaxv8hi3 = 7757,
  CODE_FOR_uminv8hi3 = 7758,
  CODE_FOR_umaxv4si3 = 7759,
  CODE_FOR_uminv4si3 = 7760,
  CODE_FOR_avx2_eqv32qi3 = 7761,
  CODE_FOR_avx2_eqv16hi3 = 7762,
  CODE_FOR_avx2_eqv8si3 = 7763,
  CODE_FOR_avx2_eqv4di3 = 7764,
  CODE_FOR_avx512bw_eqv64qi3 = 7765,
  CODE_FOR_avx512bw_eqv64qi3_mask = 7766,
  CODE_FOR_avx512vl_eqv16qi3 = 7767,
  CODE_FOR_avx512vl_eqv16qi3_mask = 7768,
  CODE_FOR_avx512vl_eqv32qi3 = 7769,
  CODE_FOR_avx512vl_eqv32qi3_mask = 7770,
  CODE_FOR_avx512bw_eqv32hi3 = 7771,
  CODE_FOR_avx512bw_eqv32hi3_mask = 7772,
  CODE_FOR_avx512vl_eqv16hi3 = 7773,
  CODE_FOR_avx512vl_eqv16hi3_mask = 7774,
  CODE_FOR_avx512vl_eqv8hi3 = 7775,
  CODE_FOR_avx512vl_eqv8hi3_mask = 7776,
  CODE_FOR_avx512f_eqv16si3 = 7777,
  CODE_FOR_avx512f_eqv16si3_mask = 7778,
  CODE_FOR_avx512vl_eqv8si3 = 7779,
  CODE_FOR_avx512vl_eqv8si3_mask = 7780,
  CODE_FOR_avx512vl_eqv4si3 = 7781,
  CODE_FOR_avx512vl_eqv4si3_mask = 7782,
  CODE_FOR_avx512f_eqv8di3 = 7783,
  CODE_FOR_avx512f_eqv8di3_mask = 7784,
  CODE_FOR_avx512vl_eqv4di3 = 7785,
  CODE_FOR_avx512vl_eqv4di3_mask = 7786,
  CODE_FOR_avx512vl_eqv2di3 = 7787,
  CODE_FOR_avx512vl_eqv2di3_mask = 7788,
  CODE_FOR_sse2_eqv16qi3 = 7789,
  CODE_FOR_sse2_eqv8hi3 = 7790,
  CODE_FOR_sse2_eqv4si3 = 7791,
  CODE_FOR_sse4_1_eqv2di3 = 7792,
  CODE_FOR_vcondv64qiv16si = 7793,
  CODE_FOR_vcondv32hiv16si = 7794,
  CODE_FOR_vcondv16siv16si = 7795,
  CODE_FOR_vcondv8div16si = 7796,
  CODE_FOR_vcondv16sfv16si = 7797,
  CODE_FOR_vcondv8dfv16si = 7798,
  CODE_FOR_vcondv64qiv8di = 7799,
  CODE_FOR_vcondv32hiv8di = 7800,
  CODE_FOR_vcondv16siv8di = 7801,
  CODE_FOR_vcondv8div8di = 7802,
  CODE_FOR_vcondv16sfv8di = 7803,
  CODE_FOR_vcondv8dfv8di = 7804,
  CODE_FOR_vcondv64qiv32hi = 7805,
  CODE_FOR_vcondv32hiv32hi = 7806,
  CODE_FOR_vcondv16siv32hi = 7807,
  CODE_FOR_vcondv8div32hi = 7808,
  CODE_FOR_vcondv16sfv32hi = 7809,
  CODE_FOR_vcondv8dfv32hi = 7810,
  CODE_FOR_vcondv64qiv64qi = 7811,
  CODE_FOR_vcondv32hiv64qi = 7812,
  CODE_FOR_vcondv16siv64qi = 7813,
  CODE_FOR_vcondv8div64qi = 7814,
  CODE_FOR_vcondv16sfv64qi = 7815,
  CODE_FOR_vcondv8dfv64qi = 7816,
  CODE_FOR_vcondv32qiv32qi = 7817,
  CODE_FOR_vcondv16hiv32qi = 7818,
  CODE_FOR_vcondv8siv32qi = 7819,
  CODE_FOR_vcondv4div32qi = 7820,
  CODE_FOR_vcondv8sfv32qi = 7821,
  CODE_FOR_vcondv4dfv32qi = 7822,
  CODE_FOR_vcondv32qiv16hi = 7823,
  CODE_FOR_vcondv16hiv16hi = 7824,
  CODE_FOR_vcondv8siv16hi = 7825,
  CODE_FOR_vcondv4div16hi = 7826,
  CODE_FOR_vcondv8sfv16hi = 7827,
  CODE_FOR_vcondv4dfv16hi = 7828,
  CODE_FOR_vcondv32qiv8si = 7829,
  CODE_FOR_vcondv16hiv8si = 7830,
  CODE_FOR_vcondv8siv8si = 7831,
  CODE_FOR_vcondv4div8si = 7832,
  CODE_FOR_vcondv8sfv8si = 7833,
  CODE_FOR_vcondv4dfv8si = 7834,
  CODE_FOR_vcondv32qiv4di = 7835,
  CODE_FOR_vcondv16hiv4di = 7836,
  CODE_FOR_vcondv8siv4di = 7837,
  CODE_FOR_vcondv4div4di = 7838,
  CODE_FOR_vcondv8sfv4di = 7839,
  CODE_FOR_vcondv4dfv4di = 7840,
  CODE_FOR_vcondv16qiv16qi = 7841,
  CODE_FOR_vcondv8hiv16qi = 7842,
  CODE_FOR_vcondv4siv16qi = 7843,
  CODE_FOR_vcondv2div16qi = 7844,
  CODE_FOR_vcondv4sfv16qi = 7845,
  CODE_FOR_vcondv2dfv16qi = 7846,
  CODE_FOR_vcondv16qiv8hi = 7847,
  CODE_FOR_vcondv8hiv8hi = 7848,
  CODE_FOR_vcondv4siv8hi = 7849,
  CODE_FOR_vcondv2div8hi = 7850,
  CODE_FOR_vcondv4sfv8hi = 7851,
  CODE_FOR_vcondv2dfv8hi = 7852,
  CODE_FOR_vcondv16qiv4si = 7853,
  CODE_FOR_vcondv8hiv4si = 7854,
  CODE_FOR_vcondv4siv4si = 7855,
  CODE_FOR_vcondv2div4si = 7856,
  CODE_FOR_vcondv4sfv4si = 7857,
  CODE_FOR_vcondv2dfv4si = 7858,
  CODE_FOR_vcondv2div2di = 7859,
  CODE_FOR_vcondv2dfv2di = 7860,
  CODE_FOR_vconduv64qiv16si = 7861,
  CODE_FOR_vconduv32hiv16si = 7862,
  CODE_FOR_vconduv16siv16si = 7863,
  CODE_FOR_vconduv8div16si = 7864,
  CODE_FOR_vconduv16sfv16si = 7865,
  CODE_FOR_vconduv8dfv16si = 7866,
  CODE_FOR_vconduv64qiv8di = 7867,
  CODE_FOR_vconduv32hiv8di = 7868,
  CODE_FOR_vconduv16siv8di = 7869,
  CODE_FOR_vconduv8div8di = 7870,
  CODE_FOR_vconduv16sfv8di = 7871,
  CODE_FOR_vconduv8dfv8di = 7872,
  CODE_FOR_vconduv64qiv32hi = 7873,
  CODE_FOR_vconduv32hiv32hi = 7874,
  CODE_FOR_vconduv16siv32hi = 7875,
  CODE_FOR_vconduv8div32hi = 7876,
  CODE_FOR_vconduv16sfv32hi = 7877,
  CODE_FOR_vconduv8dfv32hi = 7878,
  CODE_FOR_vconduv64qiv64qi = 7879,
  CODE_FOR_vconduv32hiv64qi = 7880,
  CODE_FOR_vconduv16siv64qi = 7881,
  CODE_FOR_vconduv8div64qi = 7882,
  CODE_FOR_vconduv16sfv64qi = 7883,
  CODE_FOR_vconduv8dfv64qi = 7884,
  CODE_FOR_vconduv32qiv32qi = 7885,
  CODE_FOR_vconduv16hiv32qi = 7886,
  CODE_FOR_vconduv8siv32qi = 7887,
  CODE_FOR_vconduv4div32qi = 7888,
  CODE_FOR_vconduv8sfv32qi = 7889,
  CODE_FOR_vconduv4dfv32qi = 7890,
  CODE_FOR_vconduv32qiv16hi = 7891,
  CODE_FOR_vconduv16hiv16hi = 7892,
  CODE_FOR_vconduv8siv16hi = 7893,
  CODE_FOR_vconduv4div16hi = 7894,
  CODE_FOR_vconduv8sfv16hi = 7895,
  CODE_FOR_vconduv4dfv16hi = 7896,
  CODE_FOR_vconduv32qiv8si = 7897,
  CODE_FOR_vconduv16hiv8si = 7898,
  CODE_FOR_vconduv8siv8si = 7899,
  CODE_FOR_vconduv4div8si = 7900,
  CODE_FOR_vconduv8sfv8si = 7901,
  CODE_FOR_vconduv4dfv8si = 7902,
  CODE_FOR_vconduv32qiv4di = 7903,
  CODE_FOR_vconduv16hiv4di = 7904,
  CODE_FOR_vconduv8siv4di = 7905,
  CODE_FOR_vconduv4div4di = 7906,
  CODE_FOR_vconduv8sfv4di = 7907,
  CODE_FOR_vconduv4dfv4di = 7908,
  CODE_FOR_vconduv16qiv16qi = 7909,
  CODE_FOR_vconduv8hiv16qi = 7910,
  CODE_FOR_vconduv4siv16qi = 7911,
  CODE_FOR_vconduv2div16qi = 7912,
  CODE_FOR_vconduv4sfv16qi = 7913,
  CODE_FOR_vconduv2dfv16qi = 7914,
  CODE_FOR_vconduv16qiv8hi = 7915,
  CODE_FOR_vconduv8hiv8hi = 7916,
  CODE_FOR_vconduv4siv8hi = 7917,
  CODE_FOR_vconduv2div8hi = 7918,
  CODE_FOR_vconduv4sfv8hi = 7919,
  CODE_FOR_vconduv2dfv8hi = 7920,
  CODE_FOR_vconduv16qiv4si = 7921,
  CODE_FOR_vconduv8hiv4si = 7922,
  CODE_FOR_vconduv4siv4si = 7923,
  CODE_FOR_vconduv2div4si = 7924,
  CODE_FOR_vconduv4sfv4si = 7925,
  CODE_FOR_vconduv2dfv4si = 7926,
  CODE_FOR_vconduv2div2di = 7927,
  CODE_FOR_vconduv2dfv2di = 7928,
  CODE_FOR_vcondeqv2div2di = 7929,
  CODE_FOR_vcondeqv2dfv2di = 7930,
  CODE_FOR_vec_permv16qi = 7931,
  CODE_FOR_vec_permv8hi = 7932,
  CODE_FOR_vec_permv4si = 7933,
  CODE_FOR_vec_permv2di = 7934,
  CODE_FOR_vec_permv4sf = 7935,
  CODE_FOR_vec_permv2df = 7936,
  CODE_FOR_vec_permv32qi = 7937,
  CODE_FOR_vec_permv16hi = 7938,
  CODE_FOR_vec_permv8si = 7939,
  CODE_FOR_vec_permv4di = 7940,
  CODE_FOR_vec_permv8sf = 7941,
  CODE_FOR_vec_permv4df = 7942,
  CODE_FOR_vec_permv16sf = 7943,
  CODE_FOR_vec_permv8df = 7944,
  CODE_FOR_vec_permv16si = 7945,
  CODE_FOR_vec_permv8di = 7946,
  CODE_FOR_vec_permv32hi = 7947,
  CODE_FOR_vec_permv64qi = 7948,
  CODE_FOR_one_cmplv16si2 = 7949,
  CODE_FOR_one_cmplv8di2 = 7950,
  CODE_FOR_one_cmplv64qi2 = 7951,
  CODE_FOR_one_cmplv32qi2 = 7952,
  CODE_FOR_one_cmplv16qi2 = 7953,
  CODE_FOR_one_cmplv32hi2 = 7954,
  CODE_FOR_one_cmplv16hi2 = 7955,
  CODE_FOR_one_cmplv8hi2 = 7956,
  CODE_FOR_one_cmplv8si2 = 7957,
  CODE_FOR_one_cmplv4si2 = 7958,
  CODE_FOR_one_cmplv4di2 = 7959,
  CODE_FOR_one_cmplv2di2 = 7960,
  CODE_FOR_avx512bw_andnotv64qi3 = 7961,
  CODE_FOR_avx2_andnotv32qi3 = 7962,
  CODE_FOR_sse2_andnotv16qi3 = 7963,
  CODE_FOR_avx512bw_andnotv32hi3 = 7964,
  CODE_FOR_avx2_andnotv16hi3 = 7965,
  CODE_FOR_sse2_andnotv8hi3 = 7966,
  CODE_FOR_avx512f_andnotv16si3 = 7967,
  CODE_FOR_avx2_andnotv8si3 = 7968,
  CODE_FOR_sse2_andnotv4si3 = 7969,
  CODE_FOR_avx512f_andnotv8di3 = 7970,
  CODE_FOR_avx2_andnotv4di3 = 7971,
  CODE_FOR_sse2_andnotv2di3 = 7972,
  CODE_FOR_avx512f_andnotv16si3_mask = 7973,
  CODE_FOR_avx2_andnotv8si3_mask = 7974,
  CODE_FOR_sse2_andnotv4si3_mask = 7975,
  CODE_FOR_avx512f_andnotv8di3_mask = 7976,
  CODE_FOR_avx2_andnotv4di3_mask = 7977,
  CODE_FOR_sse2_andnotv2di3_mask = 7978,
  CODE_FOR_avx512bw_andnotv64qi3_mask = 7979,
  CODE_FOR_sse2_andnotv16qi3_mask = 7980,
  CODE_FOR_avx2_andnotv32qi3_mask = 7981,
  CODE_FOR_avx512bw_andnotv32hi3_mask = 7982,
  CODE_FOR_avx2_andnotv16hi3_mask = 7983,
  CODE_FOR_sse2_andnotv8hi3_mask = 7984,
  CODE_FOR_andv16si3 = 7985,
  CODE_FOR_iorv16si3 = 7986,
  CODE_FOR_xorv16si3 = 7987,
  CODE_FOR_andv8di3 = 7988,
  CODE_FOR_iorv8di3 = 7989,
  CODE_FOR_xorv8di3 = 7990,
  CODE_FOR_andv64qi3 = 7991,
  CODE_FOR_iorv64qi3 = 7992,
  CODE_FOR_xorv64qi3 = 7993,
  CODE_FOR_andv32qi3 = 7994,
  CODE_FOR_iorv32qi3 = 7995,
  CODE_FOR_xorv32qi3 = 7996,
  CODE_FOR_andv16qi3 = 7997,
  CODE_FOR_iorv16qi3 = 7998,
  CODE_FOR_xorv16qi3 = 7999,
  CODE_FOR_andv32hi3 = 8000,
  CODE_FOR_iorv32hi3 = 8001,
  CODE_FOR_xorv32hi3 = 8002,
  CODE_FOR_andv16hi3 = 8003,
  CODE_FOR_iorv16hi3 = 8004,
  CODE_FOR_xorv16hi3 = 8005,
  CODE_FOR_andv8hi3 = 8006,
  CODE_FOR_iorv8hi3 = 8007,
  CODE_FOR_xorv8hi3 = 8008,
  CODE_FOR_andv8si3 = 8009,
  CODE_FOR_iorv8si3 = 8010,
  CODE_FOR_xorv8si3 = 8011,
  CODE_FOR_andv4si3 = 8012,
  CODE_FOR_iorv4si3 = 8013,
  CODE_FOR_xorv4si3 = 8014,
  CODE_FOR_andv4di3 = 8015,
  CODE_FOR_iorv4di3 = 8016,
  CODE_FOR_xorv4di3 = 8017,
  CODE_FOR_andv2di3 = 8018,
  CODE_FOR_iorv2di3 = 8019,
  CODE_FOR_xorv2di3 = 8020,
  CODE_FOR_vec_pack_trunc_v32hi = 8021,
  CODE_FOR_vec_pack_trunc_v16hi = 8022,
  CODE_FOR_vec_pack_trunc_v8hi = 8023,
  CODE_FOR_vec_pack_trunc_v16si = 8024,
  CODE_FOR_vec_pack_trunc_v8si = 8025,
  CODE_FOR_vec_pack_trunc_v4si = 8026,
  CODE_FOR_vec_pack_trunc_v8di = 8027,
  CODE_FOR_vec_pack_trunc_v4di = 8028,
  CODE_FOR_vec_pack_trunc_v2di = 8029,
  CODE_FOR_vec_pack_trunc_qi = 8030,
  CODE_FOR_vec_pack_trunc_hi = 8031,
  CODE_FOR_vec_pack_trunc_si = 8032,
  CODE_FOR_vec_pack_sbool_trunc_qi = 8033,
  CODE_FOR_vec_interleave_highv32qi = 8034,
  CODE_FOR_vec_interleave_highv16hi = 8035,
  CODE_FOR_vec_interleave_highv8si = 8036,
  CODE_FOR_vec_interleave_highv4di = 8037,
  CODE_FOR_vec_interleave_lowv32qi = 8038,
  CODE_FOR_vec_interleave_lowv16hi = 8039,
  CODE_FOR_vec_interleave_lowv8si = 8040,
  CODE_FOR_vec_interleave_lowv4di = 8041,
  CODE_FOR_avx512dq_vinsertf64x2_mask = 8042,
  CODE_FOR_avx512dq_vinserti64x2_mask = 8043,
  CODE_FOR_avx512f_vinsertf32x4_mask = 8044,
  CODE_FOR_avx512f_vinserti32x4_mask = 8045,
  CODE_FOR_avx512dq_vinsertf32x8_mask = 8046,
  CODE_FOR_avx512dq_vinserti32x8_mask = 8047,
  CODE_FOR_avx512f_vinsertf64x4_mask = 8048,
  CODE_FOR_avx512f_vinserti64x4_mask = 8049,
  CODE_FOR_avx512dq_shuf_i64x2_mask = 8050,
  CODE_FOR_avx512dq_shuf_f64x2_mask = 8051,
  CODE_FOR_avx512f_shuf_f64x2_mask = 8052,
  CODE_FOR_avx512f_shuf_i64x2_mask = 8053,
  CODE_FOR_avx512vl_shuf_i32x4_mask = 8054,
  CODE_FOR_avx512vl_shuf_f32x4_mask = 8055,
  CODE_FOR_avx512f_shuf_f32x4_mask = 8056,
  CODE_FOR_avx512f_shuf_i32x4_mask = 8057,
  CODE_FOR_avx512f_pshufdv3_mask = 8058,
  CODE_FOR_avx512vl_pshufdv3_mask = 8059,
  CODE_FOR_avx2_pshufdv3 = 8060,
  CODE_FOR_avx512vl_pshufd_mask = 8061,
  CODE_FOR_sse2_pshufd = 8062,
  CODE_FOR_avx512vl_pshuflwv3_mask = 8063,
  CODE_FOR_avx2_pshuflwv3 = 8064,
  CODE_FOR_avx512vl_pshuflw_mask = 8065,
  CODE_FOR_sse2_pshuflw = 8066,
  CODE_FOR_avx2_pshufhwv3 = 8067,
  CODE_FOR_avx512vl_pshufhwv3_mask = 8068,
  CODE_FOR_avx512vl_pshufhw_mask = 8069,
  CODE_FOR_sse2_pshufhw = 8070,
  CODE_FOR_sse2_loadd = 8071,
  CODE_FOR_vec_unpacks_lo_v64qi = 8072,
  CODE_FOR_vec_unpacks_lo_v32qi = 8073,
  CODE_FOR_vec_unpacks_lo_v16qi = 8074,
  CODE_FOR_vec_unpacks_lo_v32hi = 8075,
  CODE_FOR_vec_unpacks_lo_v16hi = 8076,
  CODE_FOR_vec_unpacks_lo_v8hi = 8077,
  CODE_FOR_vec_unpacks_lo_v16si = 8078,
  CODE_FOR_vec_unpacks_lo_v8si = 8079,
  CODE_FOR_vec_unpacks_lo_v4si = 8080,
  CODE_FOR_vec_unpacks_hi_v64qi = 8081,
  CODE_FOR_vec_unpacks_hi_v32qi = 8082,
  CODE_FOR_vec_unpacks_hi_v16qi = 8083,
  CODE_FOR_vec_unpacks_hi_v32hi = 8084,
  CODE_FOR_vec_unpacks_hi_v16hi = 8085,
  CODE_FOR_vec_unpacks_hi_v8hi = 8086,
  CODE_FOR_vec_unpacks_hi_v16si = 8087,
  CODE_FOR_vec_unpacks_hi_v8si = 8088,
  CODE_FOR_vec_unpacks_hi_v4si = 8089,
  CODE_FOR_vec_unpacku_lo_v64qi = 8090,
  CODE_FOR_vec_unpacku_lo_v32qi = 8091,
  CODE_FOR_vec_unpacku_lo_v16qi = 8092,
  CODE_FOR_vec_unpacku_lo_v32hi = 8093,
  CODE_FOR_vec_unpacku_lo_v16hi = 8094,
  CODE_FOR_vec_unpacku_lo_v8hi = 8095,
  CODE_FOR_vec_unpacku_lo_v16si = 8096,
  CODE_FOR_vec_unpacku_lo_v8si = 8097,
  CODE_FOR_vec_unpacku_lo_v4si = 8098,
  CODE_FOR_vec_unpacks_sbool_lo_qi = 8099,
  CODE_FOR_vec_unpacks_lo_hi = 8100,
  CODE_FOR_vec_unpacks_lo_si = 8101,
  CODE_FOR_vec_unpacks_lo_di = 8102,
  CODE_FOR_vec_unpacku_hi_v64qi = 8103,
  CODE_FOR_vec_unpacku_hi_v32qi = 8104,
  CODE_FOR_vec_unpacku_hi_v16qi = 8105,
  CODE_FOR_vec_unpacku_hi_v32hi = 8106,
  CODE_FOR_vec_unpacku_hi_v16hi = 8107,
  CODE_FOR_vec_unpacku_hi_v8hi = 8108,
  CODE_FOR_vec_unpacku_hi_v16si = 8109,
  CODE_FOR_vec_unpacku_hi_v8si = 8110,
  CODE_FOR_vec_unpacku_hi_v4si = 8111,
  CODE_FOR_vec_unpacks_sbool_hi_qi = 8112,
  CODE_FOR_vec_unpacks_hi_hi = 8113,
  CODE_FOR_vec_unpacks_hi_si = 8114,
  CODE_FOR_vec_unpacks_hi_di = 8115,
  CODE_FOR_avx512bw_uavgv64qi3 = 8116,
  CODE_FOR_avx512bw_uavgv64qi3_mask = 8117,
  CODE_FOR_avx2_uavgv32qi3 = 8118,
  CODE_FOR_avx2_uavgv32qi3_mask = 8119,
  CODE_FOR_sse2_uavgv16qi3 = 8120,
  CODE_FOR_sse2_uavgv16qi3_mask = 8121,
  CODE_FOR_avx512bw_uavgv32hi3 = 8122,
  CODE_FOR_avx512bw_uavgv32hi3_mask = 8123,
  CODE_FOR_avx2_uavgv16hi3 = 8124,
  CODE_FOR_avx2_uavgv16hi3_mask = 8125,
  CODE_FOR_sse2_uavgv8hi3 = 8126,
  CODE_FOR_sse2_uavgv8hi3_mask = 8127,
  CODE_FOR_sse2_maskmovdqu = 8128,
  CODE_FOR_ssse3_pmulhrswv8hi3_mask = 8129,
  CODE_FOR_avx2_pmulhrswv16hi3_mask = 8130,
  CODE_FOR_ssse3_pmulhrswv8hi3 = 8131,
  CODE_FOR_avx2_pmulhrswv16hi3 = 8132,
  CODE_FOR_smulhrsv32hi3 = 8133,
  CODE_FOR_smulhrsv16hi3 = 8134,
  CODE_FOR_smulhrsv8hi3 = 8135,
  CODE_FOR_smulhrsv4hi3 = 8136,
  CODE_FOR_ssse3_pmulhrswv4hi3 = 8137,
  CODE_FOR_ssse3_pshufbv8qi3 = 8138,
  CODE_FOR_absv64qi2 = 8139,
  CODE_FOR_absv32qi2 = 8140,
  CODE_FOR_absv16qi2 = 8141,
  CODE_FOR_absv32hi2 = 8142,
  CODE_FOR_absv16hi2 = 8143,
  CODE_FOR_absv8hi2 = 8144,
  CODE_FOR_absv16si2 = 8145,
  CODE_FOR_absv8si2 = 8146,
  CODE_FOR_absv4si2 = 8147,
  CODE_FOR_absv8di2 = 8148,
  CODE_FOR_absv4di2 = 8149,
  CODE_FOR_absv2di2 = 8150,
  CODE_FOR_absv8qi2 = 8151,
  CODE_FOR_absv4hi2 = 8152,
  CODE_FOR_absv2si2 = 8153,
  CODE_FOR_avx2_pblendw = 8154,
  CODE_FOR_extendv16qiv16hi2 = 8155,
  CODE_FOR_zero_extendv16qiv16hi2 = 8156,
  CODE_FOR_extendv32qiv32hi2 = 8157,
  CODE_FOR_zero_extendv32qiv32hi2 = 8158,
  CODE_FOR_extendv8qiv8hi2 = 8159,
  CODE_FOR_zero_extendv8qiv8hi2 = 8160,
  CODE_FOR_extendv16qiv16si2 = 8161,
  CODE_FOR_zero_extendv16qiv16si2 = 8162,
  CODE_FOR_extendv8qiv8si2 = 8163,
  CODE_FOR_zero_extendv8qiv8si2 = 8164,
  CODE_FOR_extendv4qiv4si2 = 8165,
  CODE_FOR_zero_extendv4qiv4si2 = 8166,
  CODE_FOR_extendv16hiv16si2 = 8167,
  CODE_FOR_zero_extendv16hiv16si2 = 8168,
  CODE_FOR_extendv8hiv8si2 = 8169,
  CODE_FOR_zero_extendv8hiv8si2 = 8170,
  CODE_FOR_extendv4hiv4si2 = 8171,
  CODE_FOR_zero_extendv4hiv4si2 = 8172,
  CODE_FOR_extendv8qiv8di2 = 8173,
  CODE_FOR_zero_extendv8qiv8di2 = 8174,
  CODE_FOR_extendv4qiv4di2 = 8175,
  CODE_FOR_zero_extendv4qiv4di2 = 8176,
  CODE_FOR_extendv2qiv2di2 = 8177,
  CODE_FOR_zero_extendv2qiv2di2 = 8178,
  CODE_FOR_extendv8hiv8di2 = 8179,
  CODE_FOR_zero_extendv8hiv8di2 = 8180,
  CODE_FOR_extendv4hiv4di2 = 8181,
  CODE_FOR_zero_extendv4hiv4di2 = 8182,
  CODE_FOR_extendv2hiv2di2 = 8183,
  CODE_FOR_zero_extendv2hiv2di2 = 8184,
  CODE_FOR_extendv8siv8di2 = 8185,
  CODE_FOR_zero_extendv8siv8di2 = 8186,
  CODE_FOR_extendv4siv4di2 = 8187,
  CODE_FOR_zero_extendv4siv4di2 = 8188,
  CODE_FOR_extendv2siv2di2 = 8189,
  CODE_FOR_zero_extendv2siv2di2 = 8190,
  CODE_FOR_nearbyintv16sf2 = 8191,
  CODE_FOR_nearbyintv8sf2 = 8192,
  CODE_FOR_nearbyintv4sf2 = 8193,
  CODE_FOR_nearbyintv8df2 = 8194,
  CODE_FOR_nearbyintv4df2 = 8195,
  CODE_FOR_nearbyintv2df2 = 8196,
  CODE_FOR_rintv16sf2 = 8197,
  CODE_FOR_rintv8sf2 = 8198,
  CODE_FOR_rintv4sf2 = 8199,
  CODE_FOR_rintv8df2 = 8200,
  CODE_FOR_rintv4df2 = 8201,
  CODE_FOR_rintv2df2 = 8202,
  CODE_FOR_avx_roundps_sfix256 = 8203,
  CODE_FOR_sse4_1_roundps_sfix = 8204,
  CODE_FOR_avx512f_roundps512 = 8205,
  CODE_FOR_avx512f_roundpd512 = 8206,
  CODE_FOR_avx512f_roundps512_sfix = 8207,
  CODE_FOR_avx512f_roundpd_vec_pack_sfix512 = 8208,
  CODE_FOR_avx_roundpd_vec_pack_sfix256 = 8209,
  CODE_FOR_sse4_1_roundpd_vec_pack_sfix = 8210,
  CODE_FOR_roundv16sf2 = 8211,
  CODE_FOR_roundv8sf2 = 8212,
  CODE_FOR_roundv4sf2 = 8213,
  CODE_FOR_roundv8df2 = 8214,
  CODE_FOR_roundv4df2 = 8215,
  CODE_FOR_roundv2df2 = 8216,
  CODE_FOR_roundv16sf2_sfix = 8217,
  CODE_FOR_roundv8sf2_sfix = 8218,
  CODE_FOR_roundv4sf2_sfix = 8219,
  CODE_FOR_roundv8df2_vec_pack_sfix = 8220,
  CODE_FOR_roundv4df2_vec_pack_sfix = 8221,
  CODE_FOR_roundv2df2_vec_pack_sfix = 8222,
  CODE_FOR_avx512pf_gatherpfv16sisf = 8223,
  CODE_FOR_avx512pf_gatherpfv8disf = 8224,
  CODE_FOR_avx512pf_gatherpfv8sidf = 8225,
  CODE_FOR_avx512pf_gatherpfv8didf = 8226,
  CODE_FOR_avx512pf_scatterpfv16sisf = 8227,
  CODE_FOR_avx512pf_scatterpfv8disf = 8228,
  CODE_FOR_avx512pf_scatterpfv8sidf = 8229,
  CODE_FOR_avx512pf_scatterpfv8didf = 8230,
  CODE_FOR_rotlv16qi3 = 8231,
  CODE_FOR_rotlv8hi3 = 8232,
  CODE_FOR_rotlv4si3 = 8233,
  CODE_FOR_rotlv2di3 = 8234,
  CODE_FOR_rotrv16qi3 = 8235,
  CODE_FOR_rotrv8hi3 = 8236,
  CODE_FOR_rotrv4si3 = 8237,
  CODE_FOR_rotrv2di3 = 8238,
  CODE_FOR_vrotrv16qi3 = 8239,
  CODE_FOR_vrotrv8hi3 = 8240,
  CODE_FOR_vrotrv4si3 = 8241,
  CODE_FOR_vrotrv2di3 = 8242,
  CODE_FOR_vrotlv16qi3 = 8243,
  CODE_FOR_vrotlv8hi3 = 8244,
  CODE_FOR_vrotlv4si3 = 8245,
  CODE_FOR_vrotlv2di3 = 8246,
  CODE_FOR_vlshrv16qi3 = 8247,
  CODE_FOR_vlshrv8hi3 = 8248,
  CODE_FOR_vlshrv4si3 = 8249,
  CODE_FOR_vlshrv2di3 = 8250,
  CODE_FOR_vlshrv16si3 = 8251,
  CODE_FOR_vlshrv8di3 = 8252,
  CODE_FOR_vlshrv8si3 = 8253,
  CODE_FOR_vlshrv4di3 = 8254,
  CODE_FOR_vashrv8hi3 = 8255,
  CODE_FOR_vashrv8hi3_mask = 8256,
  CODE_FOR_vashrv16qi3 = 8257,
  CODE_FOR_vashrv2di3 = 8258,
  CODE_FOR_vashrv2di3_mask = 8259,
  CODE_FOR_vashrv4si3 = 8260,
  CODE_FOR_vashrv16si3 = 8261,
  CODE_FOR_vashrv8si3 = 8262,
  CODE_FOR_vashlv16qi3 = 8263,
  CODE_FOR_vashlv8hi3 = 8264,
  CODE_FOR_vashlv4si3 = 8265,
  CODE_FOR_vashlv2di3 = 8266,
  CODE_FOR_vashlv16si3 = 8267,
  CODE_FOR_vashlv8di3 = 8268,
  CODE_FOR_vashlv8si3 = 8269,
  CODE_FOR_vashlv4di3 = 8270,
  CODE_FOR_ashlv64qi3 = 8271,
  CODE_FOR_lshrv64qi3 = 8272,
  CODE_FOR_ashrv64qi3 = 8273,
  CODE_FOR_ashlv32qi3 = 8274,
  CODE_FOR_lshrv32qi3 = 8275,
  CODE_FOR_ashrv32qi3 = 8276,
  CODE_FOR_ashlv16qi3 = 8277,
  CODE_FOR_lshrv16qi3 = 8278,
  CODE_FOR_ashrv16qi3 = 8279,
  CODE_FOR_ashrv2di3 = 8280,
  CODE_FOR_xop_vmfrczv4sf2 = 8281,
  CODE_FOR_xop_vmfrczv2df2 = 8282,
  CODE_FOR_avx_vzeroall = 8283,
  CODE_FOR_avx_vzeroupper = 8284,
  CODE_FOR_avx512f_vpermilv8df = 8285,
  CODE_FOR_avx512f_vpermilv8df_mask = 8286,
  CODE_FOR_avx_vpermilv4df = 8287,
  CODE_FOR_avx_vpermilv4df_mask = 8288,
  CODE_FOR_avx_vpermilv2df = 8289,
  CODE_FOR_avx_vpermilv2df_mask = 8290,
  CODE_FOR_avx512f_vpermilv16sf = 8291,
  CODE_FOR_avx512f_vpermilv16sf_mask = 8292,
  CODE_FOR_avx_vpermilv8sf = 8293,
  CODE_FOR_avx_vpermilv8sf_mask = 8294,
  CODE_FOR_avx_vpermilv4sf = 8295,
  CODE_FOR_avx_vpermilv4sf_mask = 8296,
  CODE_FOR_avx2_permv4di = 8297,
  CODE_FOR_avx2_permv4df = 8298,
  CODE_FOR_avx512vl_permv4di_mask = 8299,
  CODE_FOR_avx512vl_permv4df_mask = 8300,
  CODE_FOR_avx512f_permv8df = 8301,
  CODE_FOR_avx512f_permv8di = 8302,
  CODE_FOR_avx512f_permv8df_mask = 8303,
  CODE_FOR_avx512f_permv8di_mask = 8304,
  CODE_FOR_avx512f_vpermi2varv16si3_mask = 8305,
  CODE_FOR_avx512f_vpermi2varv16sf3_mask = 8306,
  CODE_FOR_avx512f_vpermi2varv8di3_mask = 8307,
  CODE_FOR_avx512f_vpermi2varv8df3_mask = 8308,
  CODE_FOR_avx512vl_vpermi2varv8si3_mask = 8309,
  CODE_FOR_avx512vl_vpermi2varv8sf3_mask = 8310,
  CODE_FOR_avx512vl_vpermi2varv4di3_mask = 8311,
  CODE_FOR_avx512vl_vpermi2varv4df3_mask = 8312,
  CODE_FOR_avx512vl_vpermi2varv4si3_mask = 8313,
  CODE_FOR_avx512vl_vpermi2varv4sf3_mask = 8314,
  CODE_FOR_avx512vl_vpermi2varv2di3_mask = 8315,
  CODE_FOR_avx512vl_vpermi2varv2df3_mask = 8316,
  CODE_FOR_avx512bw_vpermi2varv32hi3_mask = 8317,
  CODE_FOR_avx512vl_vpermi2varv16hi3_mask = 8318,
  CODE_FOR_avx512vl_vpermi2varv8hi3_mask = 8319,
  CODE_FOR_avx512bw_vpermi2varv64qi3_mask = 8320,
  CODE_FOR_avx512vl_vpermi2varv32qi3_mask = 8321,
  CODE_FOR_avx512vl_vpermi2varv16qi3_mask = 8322,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz = 8323,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz = 8324,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz = 8325,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz = 8326,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz = 8327,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz = 8328,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz = 8329,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz = 8330,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz = 8331,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz = 8332,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz = 8333,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz = 8334,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz = 8335,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz = 8336,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz = 8337,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz = 8338,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz = 8339,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz = 8340,
  CODE_FOR_avx_vperm2f128v8si3 = 8341,
  CODE_FOR_avx_vperm2f128v8sf3 = 8342,
  CODE_FOR_avx_vperm2f128v4df3 = 8343,
  CODE_FOR_avx512vl_vinsertv8si = 8344,
  CODE_FOR_avx512vl_vinsertv8sf = 8345,
  CODE_FOR_avx512vl_vinsertv4di = 8346,
  CODE_FOR_avx512vl_vinsertv4df = 8347,
  CODE_FOR_avx_vinsertf128v32qi = 8348,
  CODE_FOR_avx_vinsertf128v16hi = 8349,
  CODE_FOR_avx_vinsertf128v8si = 8350,
  CODE_FOR_avx_vinsertf128v4di = 8351,
  CODE_FOR_avx_vinsertf128v8sf = 8352,
  CODE_FOR_avx_vinsertf128v4df = 8353,
  CODE_FOR_maskloadv4sfv4si = 8354,
  CODE_FOR_maskloadv2dfv2di = 8355,
  CODE_FOR_maskloadv8sfv8si = 8356,
  CODE_FOR_maskloadv4dfv4di = 8357,
  CODE_FOR_maskloadv4siv4si = 8358,
  CODE_FOR_maskloadv2div2di = 8359,
  CODE_FOR_maskloadv8siv8si = 8360,
  CODE_FOR_maskloadv4div4di = 8361,
  CODE_FOR_maskloadv16sihi = 8362,
  CODE_FOR_maskloadv8siqi = 8363,
  CODE_FOR_maskloadv4siqi = 8364,
  CODE_FOR_maskloadv8diqi = 8365,
  CODE_FOR_maskloadv4diqi = 8366,
  CODE_FOR_maskloadv2diqi = 8367,
  CODE_FOR_maskloadv16sfhi = 8368,
  CODE_FOR_maskloadv8sfqi = 8369,
  CODE_FOR_maskloadv4sfqi = 8370,
  CODE_FOR_maskloadv8dfqi = 8371,
  CODE_FOR_maskloadv4dfqi = 8372,
  CODE_FOR_maskloadv2dfqi = 8373,
  CODE_FOR_maskloadv64qidi = 8374,
  CODE_FOR_maskloadv16qihi = 8375,
  CODE_FOR_maskloadv32qisi = 8376,
  CODE_FOR_maskloadv32hisi = 8377,
  CODE_FOR_maskloadv16hihi = 8378,
  CODE_FOR_maskloadv8hiqi = 8379,
  CODE_FOR_maskstorev4sfv4si = 8380,
  CODE_FOR_maskstorev2dfv2di = 8381,
  CODE_FOR_maskstorev8sfv8si = 8382,
  CODE_FOR_maskstorev4dfv4di = 8383,
  CODE_FOR_maskstorev4siv4si = 8384,
  CODE_FOR_maskstorev2div2di = 8385,
  CODE_FOR_maskstorev8siv8si = 8386,
  CODE_FOR_maskstorev4div4di = 8387,
  CODE_FOR_maskstorev16sihi = 8388,
  CODE_FOR_maskstorev8siqi = 8389,
  CODE_FOR_maskstorev4siqi = 8390,
  CODE_FOR_maskstorev8diqi = 8391,
  CODE_FOR_maskstorev4diqi = 8392,
  CODE_FOR_maskstorev2diqi = 8393,
  CODE_FOR_maskstorev16sfhi = 8394,
  CODE_FOR_maskstorev8sfqi = 8395,
  CODE_FOR_maskstorev4sfqi = 8396,
  CODE_FOR_maskstorev8dfqi = 8397,
  CODE_FOR_maskstorev4dfqi = 8398,
  CODE_FOR_maskstorev2dfqi = 8399,
  CODE_FOR_maskstorev64qidi = 8400,
  CODE_FOR_maskstorev16qihi = 8401,
  CODE_FOR_maskstorev32qisi = 8402,
  CODE_FOR_maskstorev32hisi = 8403,
  CODE_FOR_maskstorev16hihi = 8404,
  CODE_FOR_maskstorev8hiqi = 8405,
  CODE_FOR_cbranchv4si4 = 8406,
  CODE_FOR_cbranchv2di4 = 8407,
  CODE_FOR_cbranchv8si4 = 8408,
  CODE_FOR_cbranchv4di4 = 8409,
  CODE_FOR_vec_initv64qiqi = 8410,
  CODE_FOR_vec_initv32qiqi = 8411,
  CODE_FOR_vec_initv16qiqi = 8412,
  CODE_FOR_vec_initv32hihi = 8413,
  CODE_FOR_vec_initv16hihi = 8414,
  CODE_FOR_vec_initv8hihi = 8415,
  CODE_FOR_vec_initv16sisi = 8416,
  CODE_FOR_vec_initv8sisi = 8417,
  CODE_FOR_vec_initv4sisi = 8418,
  CODE_FOR_vec_initv8didi = 8419,
  CODE_FOR_vec_initv4didi = 8420,
  CODE_FOR_vec_initv2didi = 8421,
  CODE_FOR_vec_initv16sfsf = 8422,
  CODE_FOR_vec_initv8sfsf = 8423,
  CODE_FOR_vec_initv4sfsf = 8424,
  CODE_FOR_vec_initv8dfdf = 8425,
  CODE_FOR_vec_initv4dfdf = 8426,
  CODE_FOR_vec_initv2dfdf = 8427,
  CODE_FOR_vec_initv4titi = 8428,
  CODE_FOR_vec_initv2titi = 8429,
  CODE_FOR_vec_initv64qiv32qi = 8430,
  CODE_FOR_vec_initv32qiv16qi = 8431,
  CODE_FOR_vec_initv16qiv8qi = 8432,
  CODE_FOR_vec_initv32hiv16hi = 8433,
  CODE_FOR_vec_initv16hiv8hi = 8434,
  CODE_FOR_vec_initv8hiv4hi = 8435,
  CODE_FOR_vec_initv16siv8si = 8436,
  CODE_FOR_vec_initv8siv4si = 8437,
  CODE_FOR_vec_initv4siv2si = 8438,
  CODE_FOR_vec_initv8div4di = 8439,
  CODE_FOR_vec_initv4div2di = 8440,
  CODE_FOR_vec_initv16sfv8sf = 8441,
  CODE_FOR_vec_initv8sfv4sf = 8442,
  CODE_FOR_vec_initv4sfv2sf = 8443,
  CODE_FOR_vec_initv8dfv4df = 8444,
  CODE_FOR_vec_initv4dfv2df = 8445,
  CODE_FOR_vec_initv4tiv2ti = 8446,
  CODE_FOR_vcvtps2ph_mask = 8447,
  CODE_FOR_vcvtps2ph = 8448,
  CODE_FOR_avx2_gathersiv2di = 8449,
  CODE_FOR_avx2_gathersiv2df = 8450,
  CODE_FOR_avx2_gathersiv4di = 8451,
  CODE_FOR_avx2_gathersiv4df = 8452,
  CODE_FOR_avx2_gathersiv4si = 8453,
  CODE_FOR_avx2_gathersiv4sf = 8454,
  CODE_FOR_avx2_gathersiv8si = 8455,
  CODE_FOR_avx2_gathersiv8sf = 8456,
  CODE_FOR_avx2_gatherdiv2di = 8457,
  CODE_FOR_avx2_gatherdiv2df = 8458,
  CODE_FOR_avx2_gatherdiv4di = 8459,
  CODE_FOR_avx2_gatherdiv4df = 8460,
  CODE_FOR_avx2_gatherdiv4si = 8461,
  CODE_FOR_avx2_gatherdiv4sf = 8462,
  CODE_FOR_avx2_gatherdiv8si = 8463,
  CODE_FOR_avx2_gatherdiv8sf = 8464,
  CODE_FOR_avx512f_gathersiv16si = 8465,
  CODE_FOR_avx512f_gathersiv16sf = 8466,
  CODE_FOR_avx512f_gathersiv8di = 8467,
  CODE_FOR_avx512f_gathersiv8df = 8468,
  CODE_FOR_avx512vl_gathersiv8si = 8469,
  CODE_FOR_avx512vl_gathersiv8sf = 8470,
  CODE_FOR_avx512vl_gathersiv4di = 8471,
  CODE_FOR_avx512vl_gathersiv4df = 8472,
  CODE_FOR_avx512vl_gathersiv4si = 8473,
  CODE_FOR_avx512vl_gathersiv4sf = 8474,
  CODE_FOR_avx512vl_gathersiv2di = 8475,
  CODE_FOR_avx512vl_gathersiv2df = 8476,
  CODE_FOR_avx512f_gatherdiv16si = 8477,
  CODE_FOR_avx512f_gatherdiv16sf = 8478,
  CODE_FOR_avx512f_gatherdiv8di = 8479,
  CODE_FOR_avx512f_gatherdiv8df = 8480,
  CODE_FOR_avx512vl_gatherdiv8si = 8481,
  CODE_FOR_avx512vl_gatherdiv8sf = 8482,
  CODE_FOR_avx512vl_gatherdiv4di = 8483,
  CODE_FOR_avx512vl_gatherdiv4df = 8484,
  CODE_FOR_avx512vl_gatherdiv4si = 8485,
  CODE_FOR_avx512vl_gatherdiv4sf = 8486,
  CODE_FOR_avx512vl_gatherdiv2di = 8487,
  CODE_FOR_avx512vl_gatherdiv2df = 8488,
  CODE_FOR_avx512f_scattersiv16si = 8489,
  CODE_FOR_avx512f_scattersiv16sf = 8490,
  CODE_FOR_avx512f_scattersiv8di = 8491,
  CODE_FOR_avx512f_scattersiv8df = 8492,
  CODE_FOR_avx512vl_scattersiv8si = 8493,
  CODE_FOR_avx512vl_scattersiv8sf = 8494,
  CODE_FOR_avx512vl_scattersiv4di = 8495,
  CODE_FOR_avx512vl_scattersiv4df = 8496,
  CODE_FOR_avx512vl_scattersiv4si = 8497,
  CODE_FOR_avx512vl_scattersiv4sf = 8498,
  CODE_FOR_avx512vl_scattersiv2di = 8499,
  CODE_FOR_avx512vl_scattersiv2df = 8500,
  CODE_FOR_avx512f_scatterdiv16si = 8501,
  CODE_FOR_avx512f_scatterdiv16sf = 8502,
  CODE_FOR_avx512f_scatterdiv8di = 8503,
  CODE_FOR_avx512f_scatterdiv8df = 8504,
  CODE_FOR_avx512vl_scatterdiv8si = 8505,
  CODE_FOR_avx512vl_scatterdiv8sf = 8506,
  CODE_FOR_avx512vl_scatterdiv4di = 8507,
  CODE_FOR_avx512vl_scatterdiv4df = 8508,
  CODE_FOR_avx512vl_scatterdiv4si = 8509,
  CODE_FOR_avx512vl_scatterdiv4sf = 8510,
  CODE_FOR_avx512vl_scatterdiv2di = 8511,
  CODE_FOR_avx512vl_scatterdiv2df = 8512,
  CODE_FOR_avx512f_expandv16si_maskz = 8513,
  CODE_FOR_avx512f_expandv16sf_maskz = 8514,
  CODE_FOR_avx512f_expandv8di_maskz = 8515,
  CODE_FOR_avx512f_expandv8df_maskz = 8516,
  CODE_FOR_avx512vl_expandv8si_maskz = 8517,
  CODE_FOR_avx512vl_expandv8sf_maskz = 8518,
  CODE_FOR_avx512vl_expandv4di_maskz = 8519,
  CODE_FOR_avx512vl_expandv4df_maskz = 8520,
  CODE_FOR_avx512vl_expandv4si_maskz = 8521,
  CODE_FOR_avx512vl_expandv4sf_maskz = 8522,
  CODE_FOR_avx512vl_expandv2di_maskz = 8523,
  CODE_FOR_avx512vl_expandv2df_maskz = 8524,
  CODE_FOR_expandv64qi_maskz = 8525,
  CODE_FOR_expandv16qi_maskz = 8526,
  CODE_FOR_expandv32qi_maskz = 8527,
  CODE_FOR_expandv32hi_maskz = 8528,
  CODE_FOR_expandv16hi_maskz = 8529,
  CODE_FOR_expandv8hi_maskz = 8530,
  CODE_FOR_vpamdd52huqv8di_maskz = 8531,
  CODE_FOR_vpamdd52huqv4di_maskz = 8532,
  CODE_FOR_vpamdd52huqv2di_maskz = 8533,
  CODE_FOR_vpamdd52luqv8di_maskz = 8534,
  CODE_FOR_vpamdd52luqv4di_maskz = 8535,
  CODE_FOR_vpamdd52luqv2di_maskz = 8536,
  CODE_FOR_movv64sf = 8537,
  CODE_FOR_movv64si = 8538,
  CODE_FOR_popcountv16si2 = 8539,
  CODE_FOR_popcountv8si2 = 8540,
  CODE_FOR_popcountv4si2 = 8541,
  CODE_FOR_popcountv8di2 = 8542,
  CODE_FOR_popcountv4di2 = 8543,
  CODE_FOR_popcountv2di2 = 8544,
  CODE_FOR_popcountv64qi2 = 8545,
  CODE_FOR_popcountv16qi2 = 8546,
  CODE_FOR_popcountv32qi2 = 8547,
  CODE_FOR_popcountv32hi2 = 8548,
  CODE_FOR_popcountv16hi2 = 8549,
  CODE_FOR_popcountv8hi2 = 8550,
  CODE_FOR_vpshrdv_v32hi_maskz = 8551,
  CODE_FOR_vpshrdv_v16si_maskz = 8552,
  CODE_FOR_vpshrdv_v8di_maskz = 8553,
  CODE_FOR_vpshrdv_v16hi_maskz = 8554,
  CODE_FOR_vpshrdv_v8si_maskz = 8555,
  CODE_FOR_vpshrdv_v4di_maskz = 8556,
  CODE_FOR_vpshrdv_v8hi_maskz = 8557,
  CODE_FOR_vpshrdv_v4si_maskz = 8558,
  CODE_FOR_vpshrdv_v2di_maskz = 8559,
  CODE_FOR_vpshldv_v32hi_maskz = 8560,
  CODE_FOR_vpshldv_v16si_maskz = 8561,
  CODE_FOR_vpshldv_v8di_maskz = 8562,
  CODE_FOR_vpshldv_v16hi_maskz = 8563,
  CODE_FOR_vpshldv_v8si_maskz = 8564,
  CODE_FOR_vpshldv_v4di_maskz = 8565,
  CODE_FOR_vpshldv_v8hi_maskz = 8566,
  CODE_FOR_vpshldv_v4si_maskz = 8567,
  CODE_FOR_vpshldv_v2di_maskz = 8568,
  CODE_FOR_vpdpbusd_v16si_maskz = 8569,
  CODE_FOR_vpdpbusd_v8si_maskz = 8570,
  CODE_FOR_vpdpbusd_v4si_maskz = 8571,
  CODE_FOR_vpdpbusds_v16si_maskz = 8572,
  CODE_FOR_vpdpbusds_v8si_maskz = 8573,
  CODE_FOR_vpdpbusds_v4si_maskz = 8574,
  CODE_FOR_vpdpwssd_v16si_maskz = 8575,
  CODE_FOR_vpdpwssd_v8si_maskz = 8576,
  CODE_FOR_vpdpwssd_v4si_maskz = 8577,
  CODE_FOR_vpdpwssds_v16si_maskz = 8578,
  CODE_FOR_vpdpwssds_v8si_maskz = 8579,
  CODE_FOR_vpdpwssds_v4si_maskz = 8580,
  CODE_FOR_movp2qi = 8581,
  CODE_FOR_movp2hi = 8582,
  CODE_FOR_avx512f_cvtne2ps2bf16_v32hi_maskz = 8583,
  CODE_FOR_avx512f_cvtne2ps2bf16_v16hi_maskz = 8584,
  CODE_FOR_avx512f_cvtne2ps2bf16_v8hi_maskz = 8585,
  CODE_FOR_avx512f_cvtneps2bf16_v16sf_maskz = 8586,
  CODE_FOR_avx512f_cvtneps2bf16_v8sf_maskz = 8587,
  CODE_FOR_avx512f_cvtneps2bf16_v4sf_maskz = 8588,
  CODE_FOR_avx512f_dpbf16ps_v16sf_maskz = 8589,
  CODE_FOR_avx512f_dpbf16ps_v8sf_maskz = 8590,
  CODE_FOR_avx512f_dpbf16ps_v4sf_maskz = 8591,
  CODE_FOR_encodekey128u32 = 8592,
  CODE_FOR_encodekey256u32 = 8593,
  CODE_FOR_aesdecwide128klu8 = 8594,
  CODE_FOR_aesdecwide256klu8 = 8595,
  CODE_FOR_aesencwide128klu8 = 8596,
  CODE_FOR_aesencwide256klu8 = 8597,
  CODE_FOR_sse2_lfence = 8598,
  CODE_FOR_sse_sfence = 8599,
  CODE_FOR_sse2_mfence = 8600,
  CODE_FOR_mem_thread_fence = 8601,
  CODE_FOR_atomic_loadqi = 8602,
  CODE_FOR_atomic_loadhi = 8603,
  CODE_FOR_atomic_loadsi = 8604,
  CODE_FOR_atomic_loaddi = 8605,
  CODE_FOR_atomic_storeqi = 8606,
  CODE_FOR_atomic_storehi = 8607,
  CODE_FOR_atomic_storesi = 8608,
  CODE_FOR_atomic_storedi = 8609,
  CODE_FOR_atomic_compare_and_swapqi = 8610,
  CODE_FOR_atomic_compare_and_swaphi = 8611,
  CODE_FOR_atomic_compare_and_swapsi = 8612,
  CODE_FOR_atomic_compare_and_swapdi = 8613,
  CODE_FOR_atomic_compare_and_swapti = 8614,
  CODE_FOR_atomic_bit_test_and_sethi = 8615,
  CODE_FOR_atomic_bit_test_and_setsi = 8616,
  CODE_FOR_atomic_bit_test_and_setdi = 8617,
  CODE_FOR_atomic_bit_test_and_complementhi = 8618,
  CODE_FOR_atomic_bit_test_and_complementsi = 8619,
  CODE_FOR_atomic_bit_test_and_complementdi = 8620,
  CODE_FOR_atomic_bit_test_and_resethi = 8621,
  CODE_FOR_atomic_bit_test_and_resetsi = 8622,
  CODE_FOR_atomic_bit_test_and_resetdi = 8623
};

const unsigned int NUM_INSN_CODES = 8624;
#endif /* GCC_INSN_CODES_H */
